/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "CollectionModel/collectionmodel.h"
#include "MediaItem/mediaitem.h"
//#include "debug.h"

#include <QRegExp>

CollectionModel* CollectionModel::INSTANCE = 0;


/*
********************************************************************************
*                                                                              *
*    Class CollectionModel                                                     *
*                                                                              *
********************************************************************************
*/
CollectionModel::CollectionModel(QObject *parent) : QObject(parent)
{
    INSTANCE = this;
         
    m_rootItem = MediaItemPtr(new MediaItem(TYPE_UNKWON));
    
    m_filter_pattern     = "";
    m_playing_sql_idx  = -1;
    
}

CollectionModel::~CollectionModel()
{
    m_rootItem.reset();
}


MediaItemPtr CollectionModel::getItem(const MediaItemPtr parent,const int childIdx)
{
    if(!parent) return MediaItemPtr();
    
    if(childIdx < 0 || childIdx >= parent.data()->childCount()) 
      return MediaItemPtr(0);
    
    return parent->child(childIdx);
}


MediaItemPtr CollectionModel::rootItem()
{
    return m_rootItem;
}


MediaItemPtr CollectionModel::addChild(const MediaItemPtr parent, T_TYPE t)
{
    if(!parent) return MediaItemPtr(0);
  
    return parent->addChildren(t);
}


void CollectionModel::clear()
{
    m_rootItem.reset();
    m_rootItem = MediaItemPtr(new MediaItem(TYPE_UNKWON));

    trackItemHash.clear();
    albumItemList.clear();
    
    m_playing_sql_idx = -1;
    
    emit modelCleared();
}

//! ----------------------- Get tracks method ----------------------------------
QList<MediaItemPtr> CollectionModel::getItemChildrenTracks(const MediaItemPtr parent)
{
    QList<MediaItemPtr> result;
    if(!parent) return result;

    MediaItemPtr item = parent;
    
    if (item->type == TYPE_TRACK) {
      if(isTrackFiltered(item)) {
        result.append( item );
      }
    } 
    else {
      //! Recursive !!
      for (int i = 0; i < item->childCount(); i++) {
        result.append( getItemChildrenTracks(item->child(i)) );
      }
    }
    return result;
}

QList<MediaItemPtr> CollectionModel::getAlbumChildrenTracksGenre(const MediaItemPtr album,const QString& genre) 
{
    QList<MediaItemPtr> result;
    
    if(!album) return result;
    if(album->type != TYPE_ALBUM) return result;

    for (int i = 0; i < album->childCount(); i++) {
    
      if(!isTrackFiltered(album->child(i))) continue;
      
      if (!QRegExp(genre, Qt::CaseSensitive).exactMatch(album->child(i)->data.at(TRACK::genre).toString())) continue;
      
      result.append( album->child(i) );
    }
    return result;
}


//! ----------------------- Get rating method ----------------------------------
float CollectionModel::getItemAutoRating(const MediaItemPtr media)
{
    float auto_rating = 0.0;
    int   count = 0;
    if(!media) return auto_rating;

    if (media->type == TYPE_TRACK) 
    {
        //Debug::debug() << " => getItemAutoRating : return track rating = " << media->data.at(TRACK::rating).toFloat();

        return media->data.at(TRACK::rating).toFloat();
    } 
    else {
      //! Recursive !!
      for (count; count < media->childCount(); count++) 
      {
        auto_rating += getItemAutoRating( media->child(count) );
      }
    }

    float result =  media->childCount()!=0 ? float(auto_rating/(media->childCount())) : 0 ;
    result = double(int(result * 5 * 2 + 0.5)) / (5 * 2);
    //Debug::debug() << " => getItemAutoRating : rating = " << result;
    return result;
}

//! ----------------------- Favorite status method -----------------------------
bool CollectionModel::itemIsFavorite(const MediaItemPtr item) 
{
    switch (item->type) {
      case TYPE_TRACK  : break; //! unexpected case
      case TYPE_STREAM : return item->data.at(STREAM::isFavorite).toBool();break;
      case TYPE_ARTIST : return item->data.at(ARTIST::isFavorite).toBool();break;
      case TYPE_ALBUM  : return item->data.at(ALBUM::isFavorite).toBool();break;
      case TYPE_PLAYLIST : return item->data.at(PLAYLIST::isFavorite).toBool();break;
      default: break;
    }
    return false;
}


void CollectionModel::setItemIsFavorite(const MediaItemPtr item, bool isfavorite) 
{
    switch (item->type) {
      case TYPE_TRACK  : break; //! unexpected case  
      case TYPE_STREAM : item->data[STREAM::isFavorite] = isfavorite;break;
      case TYPE_ARTIST : item->data[ARTIST::isFavorite] = isfavorite;break;
      case TYPE_ALBUM  : item->data[ALBUM::isFavorite] = isfavorite;break;
      case TYPE_PLAYLIST : item->data[PLAYLIST::isFavorite] = isfavorite;break;
      default: break;
    }
}


//! ----------------------- playing status method ------------------------------
bool CollectionModel::itemIsPlaying(const MediaItemPtr item)
{
    switch (item->type) {
      case TYPE_TRACK  : return item->data.at(TRACK::isPlaying).toBool();break;
      case TYPE_STREAM : return item->data.at(STREAM::isPlaying).toBool();break;
      case TYPE_ARTIST : return item->data.at(ARTIST::isPlaying).toBool();break;
      case TYPE_ALBUM  : return item->data.at(ALBUM::isPlaying).toBool();break;
      case TYPE_PLAYLIST : return item->data.at(PLAYLIST::isPlaying).toBool();break;
      default: break;
    }
    return false;
}


void CollectionModel::setItemIsPlaying(const MediaItemPtr item, bool isplaying) 
{
    switch (item->type) {
      case TYPE_TRACK  : item->data[TRACK::isPlaying] = isplaying;break;
      case TYPE_STREAM : item->data[STREAM::isPlaying] = isplaying;break;
      case TYPE_ARTIST : item->data[ARTIST::isPlaying] = isplaying;break;
      case TYPE_ALBUM  : item->data[ALBUM::isPlaying] = isplaying;break;
      case TYPE_PLAYLIST : item->data[PLAYLIST::isPlaying] = isplaying;break;
      default: break;
    }
  
    emit signalPlayingStatusChanged();
}


MediaItemPtr CollectionModel::getPlayingTrackItem() 
{ 
    if(m_playing_sql_idx == -1)
      return MediaItemPtr(0);
    
    return trackItemHash.value (m_playing_sql_idx);
}

QString CollectionModel::getPlayingTrackItemGenre() 
{
    if(m_playing_sql_idx == -1) 
      return QString();

    return trackItemHash.value(m_playing_sql_idx)->data.at(TRACK::genre).toString();
}


void CollectionModel::updatePlayingItem(const QString& filename)
{
  //Debug::debug() << " ---- CollectionModel::updatePlayingItem << filename" << filename;

  for ( int i = 0; i < m_rootItem->childCount(); i++ )
   {
     MediaItemPtr artistItem = m_rootItem->child(i);
     for (int j = 0; j < artistItem->childCount(); j++)
     {
       MediaItemPtr albumItem = artistItem->child(j);
       for (int k = 0; k < albumItem->childCount(); k++)
       {
         MediaItemPtr trackItem = albumItem->child(k);
	 
         if (filename == trackItem->data.at(TRACK::url).toString()) 
         {
           m_playing_sql_idx = trackItem->data.at(TRACK::id).toInt();
           
           trackItem->data[TRACK::isPlaying]   = true;
           albumItem->data[ALBUM::isPlaying]   = true;
           artistItem->data[ARTIST::isPlaying] = true;

           emit signalPlayingStatusChanged();
           return;
          }
      } // end for track
    } // end for album
  }  // end for artist
}

void CollectionModel::updateBrokenItem(const QString filename)
{
  for ( int i = 0; i < m_rootItem->childCount(); i++ )
   {
     MediaItemPtr artistItem = m_rootItem->child(i);
     for (int j = 0; j < artistItem->childCount(); j++)
     {
       MediaItemPtr albumItem = artistItem->child(j);
       for (int k = 0; k < albumItem->childCount(); k++)
       {
         MediaItemPtr trackItem = albumItem->child(k);

         if (filename == trackItem->data.at(TRACK::url).toString()) 
         {
           trackItem->data[TRACK::isBroken] = true;
           emit signalPlayingStatusChanged();
           return;
          }
      } // end for track
    } // end for album
  }  // end for artist
}

void CollectionModel::stopPlayingItem()
{
    MediaItemPtr trackItem =  getPlayingTrackItem();
    
    if(!trackItem)
      return;
    
    trackItem->data[TRACK::isPlaying] = false;
    
    MediaItemPtr albumItem = trackItem->parent();
    if(albumItem)
      albumItem->data[ALBUM::isPlaying] = false;
  
    MediaItemPtr artistItem = albumItem->parent();
    if(artistItem)
      artistItem->data[ARTIST::isPlaying] = false;
 
    m_playing_sql_idx = -1;
    emit signalPlayingStatusChanged();
}



//! ------------------------- filtering method ---------------------------------
// 0.0.43 : ajout return true if filterPatter = ""
bool CollectionModel::isArtistFiltered(const MediaItemPtr artistItem) 
{
  if (!artistItem) return false;
  if (m_filter_pattern.isEmpty()) return true;
    
  //! check artist item 
  if (artistItem->data.at(ARTIST::name).toString().contains(m_filter_pattern, Qt::CaseInsensitive))
    return true;

  //! check child album
  for (int i = 0; i < artistItem->childCount(); i++) 
  {
    MediaItemPtr albumItem = artistItem->child(i);
    if (albumItem->data.at(ALBUM::name).toString().contains(m_filter_pattern, Qt::CaseInsensitive))
      return true;

    //! check child track
    for (int j = 0; j < albumItem->childCount(); j++)
    {
      MediaItemPtr trackItem = albumItem->child(j);
      if ( trackItem->data.at(TRACK::url).toString().contains(m_filter_pattern, Qt::CaseInsensitive) ||
           trackItem->data.at(TRACK::genre).toString().contains(m_filter_pattern, Qt::CaseInsensitive) )
        return true;
     }
  }

  return false;
}

bool CollectionModel::isAlbumFiltered(const MediaItemPtr albumItem) 
{
  if (!albumItem) return false;
  if (m_filter_pattern.isEmpty()) return true;
      
  //! check album item 
  if (albumItem->data.at(ALBUM::name).toString().contains(m_filter_pattern, Qt::CaseInsensitive))
    return true;
    
  //! check artist parent
  MediaItemPtr artistItem = albumItem->parent();
  if (artistItem->data.at(ARTIST::name).toString().contains(m_filter_pattern, Qt::CaseInsensitive))
      return true;
  
  //! check child track
  for (int i = 0; i < albumItem->childCount(); i++)
  {
    MediaItemPtr trackItem = albumItem->child(i);
    if ( trackItem->data.at(TRACK::url).toString().contains(m_filter_pattern, Qt::CaseInsensitive) ||
         trackItem->data.at(TRACK::genre).toString().contains(m_filter_pattern, Qt::CaseInsensitive) )
      return true;
  }

  return false;
}


bool CollectionModel::isTrackFiltered(const MediaItemPtr trackItem) 
{
  if (!trackItem) return false;
  if (m_filter_pattern.isEmpty()) return true;
      
  //! check track item 
  if ( trackItem->data.at(TRACK::url).toString().contains(m_filter_pattern, Qt::CaseInsensitive) ||
       trackItem->data.at(TRACK::genre).toString().contains(m_filter_pattern, Qt::CaseInsensitive) )
    return true;
    
  //! check album parent
  MediaItemPtr albumItem = trackItem->parent();
  if (albumItem->data.at(ALBUM::name).toString().contains(m_filter_pattern, Qt::CaseInsensitive))
      return true;
  
  //! check artist parent
  MediaItemPtr artistItem = albumItem->parent();
  if (artistItem->data.at(ARTIST::name).toString().contains(m_filter_pattern, Qt::CaseInsensitive))
      return true;

  return false;
}




