/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "CollectionView/collectionscene.h"

#include "button_item.h"

#include "CollectionModel/collectionmodel.h"
#include "CollectionModel/plsfilemodel.h"
#include "CollectionModel/histomodel.h"
#include "Playlist/playlistmodel.h"

#include "MediaItem/mediaitem.h"
#include "MediaItem/mediamimedata.h"

#include "Database/database.h"
#include "Database/databasemanager.h"    
#include "Histo/histomanager.h"          

#include "Dialog/mediaitem_edit_dialog.h"
#include "SmartPlaylist/smartplaylist.h"
#include "Playlist/playlisteditor.h"

#include "threadmanager.h"
#include "yarocksettings.h"
#include "utilities.h"
#include "global_actions.h"
#include "debug.h"

#include <QSqlField>
#include <QSqlQuery>
#include <QSqlDriver>


/*
********************************************************************************
*                                                                              *
*    Class CollectionScene                                                     *
*                                                                              *
********************************************************************************
*/ 
CollectionScene::CollectionScene(QWidget *parent) : QGraphicsScene(parent)
{
    _parentWidget = parent;
    this->setParent(_parentWidget);
    this->setItemIndexMethod(QGraphicsScene::NoIndex);
    this->setStickyFocus(true);
  
    //! set model  
    _collectionModel = CollectionModel::instance();  
    _plsFileModel    = PlsFileModel::instance();
    _histoModel      = HistoModel::instance();
    _playlistModel   = PlaylistModel::instance();
    
    connect(_collectionModel, SIGNAL(signalPlayingStatusChanged()), this, SLOT(update()));
    connect(_plsFileModel, SIGNAL(signalPlayingStatusChanged()), this, SLOT(update()));
    connect(_histoModel, SIGNAL(signalPlayingStatusChanged()), this, SLOT(update()));
    
    _infoSize            = 0;
    _mouseGrabbedItem    = 0;

    //! graphic item context menu
    m_graphic_item_menu = new GraphicsItemMenu(0);
    QObject::connect(m_graphic_item_menu, SIGNAL(menu_action_triggered(ENUM_ACTION_ITEM_MENU)), this, SLOT(slot_contextmenu_triggered(ENUM_ACTION_ITEM_MENU)), Qt::DirectConnection);

    //! gestion du double click par action globale et data
    ACTIONS()->insert(BROWSER_ITEM_DOUBLE_CLICK, new QAction(this));  
    QObject::connect(ACTIONS()->value(BROWSER_ITEM_DOUBLE_CLICK), SIGNAL(triggered()), this, SLOT(slot_item_doubleclick()), Qt::DirectConnection);
}   

CollectionScene::~CollectionScene()
{
    clear(); 
}   


//! --------------------- Change view ------------------------------------------
void CollectionScene::slot_change_album_view()
{
    ButtonItem *button = qobject_cast<ButtonItem *>(sender());
    if(!button) return;
  
    MAIN::ViewAlbum_Type type =   MAIN::ViewAlbum_Type(button->data().toInt());
     
    SETTINGS()->_album_view_type = (type == MAIN::album_grid) ? 0 : 1;

    populateScene();    
}

void CollectionScene::slot_change_playlist_view()
{
    ButtonItem *button = qobject_cast<ButtonItem *>(sender());
    if(!button) return;
  
    MAIN::ViewPlaylist_Type type =   MAIN::ViewPlaylist_Type(button->data().toInt());
     
    SETTINGS()->_playlist_view_type = (type == MAIN::playlist_overview) ? 0 : 1;

    populateScene();
}




//! --------------------- playing status method -------------------------------- 
void CollectionScene::updateTrackPlaying(const QString& filename)
{  
  if(!filename.isEmpty()) {
     _collectionModel->stopPlayingItem();
     _collectionModel->updatePlayingItem(QFileInfo(filename).absoluteFilePath());
      
    _plsFileModel->stopPlayingItem();
    _plsFileModel->updatePlayingItem(filename);
      
    _histoModel->stopPlayingItem();
    _histoModel->updatePlayingItem(filename);
  }
  else {  
     _collectionModel->stopPlayingItem();
     _plsFileModel->stopPlayingItem();
     _histoModel->stopPlayingItem();
  }
}

void CollectionScene::updateTrackBroken(const QString& filename)
{
    //Debug::debug() << "CollectionScene::updateTrackBroken" << filename;
    if(filename.isEmpty())
      return;
      
    if(MEDIA::isUrlLocal(filename))
      _collectionModel->updateBrokenItem(QFileInfo(filename).absoluteFilePath());
    
     /* can be local file or stream */    
    _plsFileModel->updateBrokenItem(QFileInfo(filename).absoluteFilePath());
    _histoModel->updateBrokenItem(QFileInfo(filename).absoluteFilePath());
}


/*******************************************************************************
    Collection Scene Populator
*******************************************************************************/
void CollectionScene::populateScene() 
{
    Debug::debug() << "CollectionScene::PopulateScene";

    //! clear scene and delete all items
    clear();
    switch(_collectionMode)
    {
      case MAIN::ViewAlbum          :
        if( MAIN::ViewAlbum_Type(SETTINGS()->_album_view_type) == MAIN::album_grid)
          populateAlbumGridScene();
        else
          populateAlbumScene();
      break;
    
      case MAIN::ViewArtist         : populateArtistScene(); break;
      case MAIN::ViewTrack          : populateTrackScene(); break;
      case MAIN::ViewGenre          : populateGenreScene(); break;
      case MAIN::ViewYear           : populateYearScene(); break;
      case MAIN::ViewFavorite       : populateFavoriteScene(); break;
      case MAIN::ViewPlaylist       : 
        if( MAIN::ViewPlaylist_Type(SETTINGS()->_playlist_view_type) == MAIN::playlist_overview)
          populatePlaylistOverviewScene();
        else
          populatePlaylistByTrackScene();
        break;      
      case MAIN::ViewSmartPlaylist  : populatePlaylistSmartScene(); break;
      case MAIN::ViewHistory        : populateHistoScene(); break;
      case MAIN::ViewDashBoard      : populateDashBoardScene(); break;
      default: break;
    } 
  
    //! we need to ajust SceneRect 
    int diff = _parentWidget->height() - itemsBoundingRect().height() + 40;
    int ajust  = ( diff > 0) ? diff : 0; 
    setSceneRect ( itemsBoundingRect().adjusted(-10, -10, +10, +10 + ajust) );
}


void CollectionScene::populateAlbumScene() 
{
    int albumRow  = 0;
    int artistRow = 0;
    int Column    = 0;
    _infoSize     = 0;
  
    int nbItem = (_parentWidget->width()/160 > 2) ? _parentWidget->width()/160 : 2;
  
    //! categories list (for up/down navigation)
    _listCategorie.clear();

    //!  button view by icon grid
    ButtonItem* ui_button           = new ButtonItem();
    ui_button->setText(tr("view grid"));
    ui_button->setData(QVariant(int(MAIN::album_grid)));
    addItem(ui_button);
    connect(ui_button, SIGNAL(clicked()), this, SLOT(slot_change_album_view()));    
    
    ButtonItem* ui_button_2          = new ButtonItem();
    ui_button_2->setText(tr("cover"));
    ui_button_2->setToolTip( ACTIONS()->value(TASK_COVER_SEARCH)->text() );  
    ui_button_2->setPixmap(QPixmap(":/images/lastfm.png"));
    addItem(ui_button_2);
    connect(ui_button_2, SIGNAL(clicked()), ACTIONS()->value(TASK_COVER_SEARCH), SIGNAL(triggered()));    
    
    int xPos = ui_button->width() + ui_button_2->width() +20+30;
    xPos = (xPos > _parentWidget->width()) ? 30 : (_parentWidget->width() - xPos)/2;

    ui_button->setPos(xPos, 25);
    ui_button_2->setPos(xPos+ ui_button->width() + 20, 25);

    
    artistRow++;
  
  
    //! artist loop
    for ( int i = 0; i < _collectionModel->rootItem()->childCount(); i++ ) 
    {
      MediaItemPtr artistItem = _collectionModel->rootItem()->child(i);
    
      if(!_collectionModel->isArtistFiltered(artistItem) ) continue;
    
      CategorieGraphicItem *_artist = new CategorieGraphicItem();
      _artist->_catName = artistItem->data.at(ARTIST::name).toString();
      _artist->_width   = _parentWidget->width() -20;
      _artist->setPos( 0 ,10 + artistRow*50 + albumRow*170);

      addItem(_artist);
      _listCategorie.append(_artist);
    
      Column = 0;
      artistRow++;

      //! album loop
      for (int j = 0; j < artistItem->childCount(); j++) 
      {
        MediaItemPtr albumItem = artistItem->child(j);
        if(!_collectionModel->isAlbumFiltered(albumItem) ) continue;
      
        _infoSize++;
      
        AlbumGraphicItem_v2 *_album = new AlbumGraphicItem_v2();
        _album->media = albumItem;
        _album->setPos(4+160*Column, artistRow*50 + albumRow*170);
        addItem(_album);
      

        if(Column < (nbItem-1)) {
          Column++;
        } 
        else {
          Column = 0;
          albumRow++;
        }
      }
      if(Column>0) albumRow++;
    }
    
    if(_infoSize==0) {
      CategorieGraphicItem *cat = new CategorieGraphicItem();
      cat->_catName = tr("Albums");
      cat->_width   = _parentWidget->width()-20;
      cat->setPos( 0 ,10 + artistRow*50 + albumRow*170 );

      addItem(cat);
      artistRow++;
      
      InfoGraphicItem *item = new InfoGraphicItem();
      item->_text = tr("No entry found");
      item->_width   = _parentWidget->width();
      item->setPos( 0 , 10 + artistRow*50 + albumRow*20);
      addItem(item);
    }    
}

void CollectionScene::populateAlbumGridScene()
{
    int albumRow      = 0;
    int Column        = 0;
    _infoSize         = 0;
    int categorieRow  = 0;
    int nbItem = (_parentWidget->width()/160 > 2) ? _parentWidget->width()/160 : 2;

    //! categories list (for up/down navigation)
    _listCategorie.clear();

    //!  button view by icon grid
    ButtonItem* ui_button           = new ButtonItem();
    ui_button->setText(tr("view extended"));
    ui_button->setData(QVariant(int(MAIN::album_extended)));
    addItem(ui_button);
    connect(ui_button, SIGNAL(clicked()), this, SLOT(slot_change_album_view()));    
    
    ButtonItem* ui_button_2          = new ButtonItem();
    ui_button_2->setText(tr("cover"));
    ui_button_2->setPixmap(QPixmap(":/images/lastfm.png"));
    connect(ui_button, SIGNAL(clicked()), ACTIONS()->value(TASK_COVER_SEARCH), SIGNAL(triggered()));    
    addItem(ui_button_2);
  
    int xPos = ui_button->width() + ui_button_2->width() +20+30;
    xPos = (xPos > _parentWidget->width()) ? 30 : (_parentWidget->width() - xPos)/2;

    ui_button->setPos(xPos, 25);
    ui_button_2->setPos(xPos+ ui_button->width() + 20, 25);
    
    categorieRow++;
      
    
    CategorieGraphicItem *cat = new CategorieGraphicItem();
    cat->_catName = tr("All Albums");
    cat->_width   = _parentWidget->width() - 20;

    cat->setPos( 0 ,10 + categorieRow*50);
    categorieRow++;
    addItem(cat);
    _listCategorie.append(cat);
    
    
    //! artist loop
    for ( int i = 0; i < _collectionModel->rootItem()->childCount(); i++ ) 
    {
      MediaItemPtr artistItem = _collectionModel->rootItem()->child(i);
    
      if(!_collectionModel->isArtistFiltered(artistItem) ) continue;

      //! album loop
      for (int j = 0; j < artistItem->childCount(); j++) 
      {
        MediaItemPtr albumItem = artistItem->child(j);
        if(!_collectionModel->isAlbumFiltered(albumItem) ) continue;
      
        _infoSize++;
      
        AlbumGraphicItem *_album = new AlbumGraphicItem();
        _album->media = albumItem;
        _album->setPos(4+160*Column, 10 + albumRow*170 + categorieRow*50);
        addItem(_album);
      
        if(Column < (nbItem-1)) {
          Column++;
        } 
        else {
          Column = 0;
          albumRow++;
        }
      }
    }

    if(_infoSize==0) {
      InfoGraphicItem *item = new InfoGraphicItem();
      item->_text = tr("No entry found");
      item->_width   = _parentWidget->width();
      item->setPos( 0 ,10 + categorieRow*50);
      addItem(item);
    }
}



void CollectionScene::populateArtistScene()
{
    int artistRow    = 0;
    int categorieRow = 0;
    int Column       = 0;
    _infoSize        = 0;  
    int idx          = 0;
    int nbItem = (_parentWidget->width()/160 > 2) ? _parentWidget->width()/160 : 2;
    
    QStringList coverPathList = QStringList();

    //! categories list (for up/down navigation)
    _listCategorie.clear();
  
    //! artist loop
    QChar start_char  = QChar(); 
    for (int i=0 ; i < _collectionModel->rootItem()->childCount(); i++ ) 
    {
        MediaItemPtr artistItem = _collectionModel->rootItem()->child(i);
        if(! _collectionModel->isArtistFiltered(artistItem) ) continue;
            
        QChar current_char  = QChar(); 
        current_char = artistItem->data.at(ARTIST::name).toString().at(0);
        current_char.toLower();
        
        if(start_char !=  current_char) {
          //! new category
          start_char = current_char;
          if(idx>0) artistRow++;
                
          CategorieGraphicItem *_cat = new CategorieGraphicItem();
          _cat->_catName = QString(start_char);
          _cat->_width   = _parentWidget->width() - 20;
          _cat->setPos( 0 , 10 + categorieRow*50 + artistRow*165);
          addItem(_cat);
          _listCategorie.append(_cat);
          
          Column = 0;
          categorieRow++;
        }
        
        //! add new artist item
        _infoSize++;
        idx++;

        //! New Artist Item
        ArtistGraphicItem *_artist = new ArtistGraphicItem();
        _artist->media = artistItem;
        _artist->setPos( 4 + 160*Column , categorieRow*50 + artistRow*165);
        addItem(_artist);
        
        //! ALBUM coverart LOOP -> set Album cover path list
        for(int j = artistItem->childCount()-1 ; j >= 0; j--) {
          MediaItemPtr albumItem = artistItem->child(j);
          if(!_collectionModel->isAlbumFiltered(albumItem) ) continue;
           
          coverPathList.prepend(albumItem->data.at(ALBUM::coverpath).toString());
          
          //!WARNING limite de l'affichage à 5 cover max
          if(coverPathList.size() >=6) break;
        }
        artistItem->data[ARTIST::coverpathlist] = coverPathList;
        coverPathList.clear();        

        if(Column < (nbItem-1)) {
          Column++;
        }
        else {
          Column = 0;
          artistRow++;
          idx=0;
        }

    } // end for artist loop
    
    if(_infoSize==0) {
      CategorieGraphicItem *cat = new CategorieGraphicItem();
      cat->_catName = tr("Artists");
      cat->_width   = _parentWidget->width()-20;
      cat->setPos( 0 ,10 + categorieRow*50);

      addItem(cat);
      categorieRow++;
      
      InfoGraphicItem *item = new InfoGraphicItem();
      item->_text = tr("No entry found");
      item->_width   = _parentWidget->width();
      item->setPos( 0 , 10 + categorieRow*50);
      addItem(item);
    }       
}



void CollectionScene::populateTrackScene() 
{
    int artistRow     = 0;
    int albumRow      = 0;
    int trackRow      = 0;
    int trackPerAlbum = 0;
    int offset        = 0;
    _infoSize         = 0;
    
    //! track list for virtual playlist
    _listTrack.clear();

    //! categories list (for up/down navigation)
    _listCategorie.clear();
  
    //! artist loop
    for ( int i = 0; i < _collectionModel->rootItem()->childCount(); i++ ) 
    {
      MediaItemPtr artistItem = _collectionModel->rootItem()->child(i);
    
      if(!_collectionModel->isArtistFiltered(artistItem) ) continue;
    
      CategorieGraphicItem *_artist = new CategorieGraphicItem();
      _artist->_catName = artistItem->data.at(ARTIST::name).toString();
      _artist->_width   = _parentWidget->width() - 20;
      _artist->setPos( 0 ,artistRow*50 + trackRow*20 + albumRow*30 + offset );
      addItem(_artist);
      _listCategorie.append(_artist);
    
      artistRow++;
    
      //! album loop
      for (int j = 0; j < artistItem->childCount(); j++) 
      {
        MediaItemPtr albumItem = artistItem->child(j);
      
        if(!_collectionModel->isAlbumFiltered(albumItem)) continue;
      
        AlbumGraphicItem_v2 *_album = new AlbumGraphicItem_v2();
        _album->media = albumItem;
        _album->setPos(4, artistRow*50 + trackRow*20 + albumRow*30 + offset );
        addItem(_album);
      
        trackPerAlbum = 0;
      
        //! track loop
        for (int k = 0; k < albumItem->childCount(); k++) 
        {
          MediaItemPtr trackItem = albumItem->child(k);
          if(!_collectionModel->isTrackFiltered(trackItem)) continue;

          TrackGraphicItem_v2 *_track = new TrackGraphicItem_v2();
          _track->media = trackItem;
          _track->setPos(155, artistRow*50 + trackRow*20 + albumRow*30 + offset);
        
          //PATCH (-20 => fix alignement of the scene)
          _track->_width = _parentWidget->width()-155-20;
          addItem(_track); 
          _listTrack.append(_track);

          trackRow++;
          trackPerAlbum++;
          _infoSize++;
        }
        albumRow++;
      
        if( trackPerAlbum < 8 ) offset = offset + (8 - trackPerAlbum)*20;
      }
    }
  
    if(_infoSize==0) {
      CategorieGraphicItem *cat = new CategorieGraphicItem();
      cat->_catName = tr("Tracks");
      cat->_width   = _parentWidget->width()-20;
      cat->setPos( 0 ,10 + artistRow*50);

      addItem(cat);
      artistRow++;
      
      InfoGraphicItem *item = new InfoGraphicItem();
      item->_text = tr("No entry found");
      item->_width   = _parentWidget->width();
      item->setPos( 0 , 10 + artistRow*50);
      addItem(item);
    }     
}



void CollectionScene::populateGenreScene()
{
    int categorieRow = 0;
    int albumRow     = 0;
    int Column       = 0;
    _infoSize        = 0;
    int nbItem = (_parentWidget->width()/160 > 2) ? _parentWidget->width()/160 : 2;

    //! categories list (for up/down navigation)
    _listCategorie.clear();
  
    QString album     = "";
    QString genre     = "";

    QList<MediaItemPtr> list_track_by_genre = _collectionModel->trackItemHash.values();
    
    
    //! Sort Media Track Item list By Genre
    qSort(list_track_by_genre.begin(), list_track_by_genre.end(),MEDIA::compareTrackItemGenre);    
    
    foreach (MediaItemPtr mi, list_track_by_genre) 
    {
      //! add filtre
      //! if(!_collectionModel->isTrackGenreFiltered(mi,mi->data[TRACK::genre].toString())) continue;
      if(genre == mi->data.at(TRACK::genre).toString() && album == mi->data.at(TRACK::album).toString()) continue;
      if(!_collectionModel->isTrackFiltered(mi)) continue;

      //if (!QRegExp(album, Qt::CaseSensitive).exactMatch(mi->data[TRACK::album].toString())) {
      if(album != mi->data.at(TRACK::album).toString()) 
      {
        if(genre != mi->data.at(TRACK::genre).toString()) 
        {
          //! new genre
          genre = mi->data.at(TRACK::genre).toString();
  
          if(Column>0) albumRow++;
          CategorieGraphicItem *_cat = new CategorieGraphicItem();
          _cat->_catName = genre;
          _cat->_width   = _parentWidget->width()-20;

          _cat->setPos( 0 , 10 + categorieRow*50 + albumRow*170);
          addItem(_cat);
      
          _listCategorie.append(_cat);
          _infoSize++; // on compte les categorie = genre	  
          categorieRow++;
          Column     = 0;
        }
      
        //! new album
        album = mi->data.at(TRACK::album).toString();

        AlbumGenreGraphicItem *_album = new AlbumGenreGraphicItem();
        _album->media    = mi->parent();
        _album->_genre   = genre;
        _album->setPos(4+160*Column, categorieRow*50 + albumRow*170);
        addItem(_album);
  
        if(Column < (nbItem-1)) {
          Column++;
        }
        else {
          Column = 0;
          albumRow++;
        }
      }
    } // fin foreach track
  
    if(_infoSize==0) {
      CategorieGraphicItem *cat = new CategorieGraphicItem();
      cat->_catName = tr("Albums");
      cat->_width   = _parentWidget->width()-20;
      cat->setPos( 0 ,10 + categorieRow*50);

      addItem(cat);
      categorieRow++;
      
      InfoGraphicItem *item = new InfoGraphicItem();
      item->_text = tr("No entry found");
      item->_width   = _parentWidget->width();
      item->setPos( 0 , 10 + categorieRow*50);
      addItem(item);
    }     
}                  


void CollectionScene::populateYearScene()
{
    int categorieRow = 0;
    int albumRow     = 0;
    int Column       = 0;
    _infoSize        = 0;
    int nbItem = (_parentWidget->width()/160 > 2) ? _parentWidget->width()/160 : 2;

    //! categories list (for up/down navigation)
    _listCategorie.clear();
  
    QString album     = "";
    QString year      = "";
    int idx           = 0;
  
  
    QList<MediaItemPtr> list_album_by_year = _collectionModel->albumItemList;
   
    //! Sort Media Album Item list By Year
    qSort(list_album_by_year.begin(), list_album_by_year.end(),MEDIA::compareAlbumItemYear);

    foreach (MediaItemPtr mi, list_album_by_year) 
    {
      //! add filtre
      if(!_collectionModel->isAlbumFiltered(mi)) continue;
    
      if(year != mi->data.at(ALBUM::year).toString()) {
        //! new genre
        year = mi->data.at(ALBUM::year).toString();
      
        if(idx>0) albumRow++;
        CategorieGraphicItem *_cat = new CategorieGraphicItem();
        _cat->_catName = year;
        _cat->_width   = _parentWidget->width() -20;
        _cat->setPos( 0 , 10 + categorieRow*50 + albumRow*170);
        addItem(_cat);
          
        _listCategorie.append(_cat);
        _infoSize++; // on compte les categorie = genre	  
        categorieRow++;
        Column     = 0;
        idx        = 0;
      } 
    
      AlbumGraphicItem *_album = new AlbumGraphicItem();
      _album->media    = mi;
      _album->setPos(4+160*Column, categorieRow*50 + albumRow*170);
      addItem(_album);
  
      if(Column < (nbItem-1)) {
        Column++;
        idx++; 
      }
      else {
        Column = 0;
        albumRow++;
        idx=0;
      }
    } // fin foreach track
  
  
    if(_infoSize==0) {
      CategorieGraphicItem *cat = new CategorieGraphicItem();
      cat->_catName = tr("Albums");
      cat->_width   = _parentWidget->width()-20;
      cat->setPos( 0 ,10 + categorieRow*50);

      addItem(cat);
      categorieRow++;
      
      InfoGraphicItem *item = new InfoGraphicItem();
      item->_text = tr("No entry found");
      item->_width   = _parentWidget->width();
      item->setPos( 0 , 10 + categorieRow*50);
      addItem(item);
    }   
}





void CollectionScene::populateFavoriteScene()
{
    int categorieRow = 0;
    int artistRow    = 0;
    int albumRow     = 0;
    int Column       = 0;
    _infoSize        = 0;  
    int nbItem = (_parentWidget->width()/160 > 2) ? _parentWidget->width()/160 : 2;

    
    //! categories list (for up/down navigation)
    _listCategorie.clear();
  
    QStringList coverPathList = QStringList();
  
    CategorieGraphicItem *_cat = new CategorieGraphicItem();
    _cat->_catName = tr("Favorite Artists");
    _cat->_width   = _parentWidget->width()-20;
    _cat->setPos( 0 , 10 + categorieRow*50 + artistRow*165);
    addItem(_cat);
      
    _listCategorie.append(_cat);
    categorieRow++;
  
  
    //! ------ Favorite Artists ------
    for ( int i = 0; i < _collectionModel->rootItem()->childCount(); i++ ) {
      
      MediaItemPtr artistItem = _collectionModel->rootItem()->child(i);

      if(! _collectionModel->itemIsFavorite(artistItem) ) continue;
      if(! _collectionModel->isArtistFiltered(artistItem) ) continue;
      
      _infoSize++; // we count artist favorite item 

      //! New Artist Item
      ArtistGraphicItem *_artist = new ArtistGraphicItem();
      _artist->media = artistItem;
      _artist->setPos( 4 + 160*Column , categorieRow*50 + artistRow*165);
      addItem(_artist);
      
      if(Column < (nbItem-1)) {
        Column++;
      }
      else {
        Column = 0;
        artistRow++;
      }
    
      //! ALBUM LOOP -> set Album cover path list
      for(int j = artistItem->childCount()-1 ; j >= 0; j--) {
        MediaItemPtr albumItem = artistItem->child(j);
        if(!_collectionModel->isAlbumFiltered(albumItem) ) continue;
      
        coverPathList.prepend(albumItem->data.at(ALBUM::coverpath).toString());
      
        //!WARNING limite de l'affichage à 5 cover max
        if(coverPathList.size() >=6) break;
        }

        artistItem->data[ARTIST::coverpathlist] = coverPathList;
        coverPathList.clear();      
    } // end artist loop
  
    if(_infoSize==0) {
      InfoGraphicItem *item = new InfoGraphicItem();
      item->_text = tr("No entry found");
      item->_width   = _parentWidget->width();
      item->setPos( 0 , 10 + categorieRow*50 + artistRow*165);
      addItem(item);
      categorieRow++;
    }


    //! ------ Favorite Albums ------
    if (Column > 0) artistRow++;

    CategorieGraphicItem *_cat2 = new CategorieGraphicItem();
    _cat2->_catName = tr("Favorite Albums");
    _cat2->_width   = _parentWidget->width()-20;
    _cat2->setPos( 0 , 10 + categorieRow*50 + artistRow*165);
    addItem(_cat2);
        
    _listCategorie.append(_cat2);
                
    Column = 0;
    categorieRow++;
    int oldInfoSize = _infoSize;
    foreach (MediaItemPtr mi, _collectionModel->albumItemList)
    {
        if(!_collectionModel->itemIsFavorite(mi) )  continue;
        if(!_collectionModel->isAlbumFiltered(mi) ) continue;
      
        _infoSize++; // we count album favorite item 
      
        AlbumGraphicItem *_album = new AlbumGraphicItem();
        _album->media = mi;
        _album->setPos(4+160*Column, categorieRow*50 + artistRow*165 + albumRow*170);
        addItem(_album);
      
        if(Column < (nbItem-1)) {
          Column++;
        } 
        else {
          Column = 0;
          albumRow++;
        }
    } // end album loop
    
    if(oldInfoSize == _infoSize) {
      InfoGraphicItem *item = new InfoGraphicItem();
      item->_text = tr("No entry found");
      item->_width   = _parentWidget->width();
      item->setPos( 0 , 10 + categorieRow*50 + artistRow*165);
      addItem(item);      
    }    
    
}



void CollectionScene::populatePlaylistOverviewScene() 
{
    int categorieRow = 0;
    int playlistRow  = 0;
    int Column       = 0;
    _infoSize        = 0;
    int idx    = 0;
    int nbItem = (_parentWidget->width()/160 > 2) ? _parentWidget->width()/160 : 2;

    
    //! button for new smart playlist creation
    ButtonItem* ui_button             = new ButtonItem();
    ui_button->setText( tr("new playlist") );   
    ui_button->setPixmap(QPixmap(":/images/document-new-48x48.png"));
    addItem(ui_button);
    connect(ui_button, SIGNAL(clicked()), this, SLOT(slot_create_new_playlist())); 
    
    //!  button view by track
    ButtonItem* ui_button_2           = new ButtonItem();
    ui_button_2->setText( tr("view tracks") );
    ui_button_2->setData(QVariant(int(MAIN::playlist_by_tracks)));
    addItem(ui_button_2);
    connect(ui_button_2, SIGNAL(clicked()), this, SLOT(slot_change_playlist_view()));     
    
    int xPos = ui_button->width() + ui_button_2->width() +20+30;
    xPos = (xPos > _parentWidget->width()) ? 30 : (_parentWidget->width() - xPos)/2;

    ui_button->setPos(xPos, 25);
    ui_button_2->setPos(xPos+ ui_button->width() + 20, 25);
    
    categorieRow++;
    
    
    //! categories list (for up/down navigation)
    _listCategorie.clear();
  
  
    CategorieGraphicItem *_cat = new CategorieGraphicItem();
    _cat->_catName = tr("Playlist files from internal database");
    _cat->_width   = _parentWidget->width()-20;
    _cat->setPos( 0 ,10 + categorieRow*50 + playlistRow*150);
    addItem(_cat);
    _listCategorie.append(_cat);
    categorieRow++;  
  
    //! playlist loop for T_DATABASE playlist
    int i = 0;
    for ( ; i < _plsFileModel->rootItem()->childCount(); i++ ) 
    {
      MediaItemPtr playlistItem = _plsFileModel->rootItem()->child(i);
    
      //WARNING drop smart playlist for this view
      if(playlistItem->data.at(PLAYLIST::type).toInt() != T_DATABASE) break;

      if(!_plsFileModel->isPlaylistFiltered(playlistItem) ) continue;
    
      _infoSize++;
      idx++; // nouvelle ligne
      
      PlaylistGraphicItem *_playlist = new PlaylistGraphicItem();
      _playlist->media = playlistItem;
      _playlist->setPos(4+160*Column, categorieRow*50 + playlistRow*150);
      addItem(_playlist);    
      
      if(Column < (nbItem-1)) {
        Column++;
      } 
      else {
        Column = 0;
        playlistRow++;
        idx = 0;
      }          
    }
 
    if(_infoSize == 0) {
      InfoGraphicItem *item = new InfoGraphicItem();
      item->_text = tr("No entry found");
      item->_width   = _parentWidget->width();
      item->setPos( 0 , 10 + categorieRow*50 + playlistRow*150);
      addItem(item);      
      categorieRow++;
    }    
    
    int oldInfoSize = _infoSize;
 
    if(idx>0) playlistRow++;
    Column = 0;

    CategorieGraphicItem *_cat2 = new CategorieGraphicItem();
    _cat2->_catName = tr("Playlist files from user collection");
    _cat2->_width   =  _parentWidget->width() -20;
    _cat2->setPos( 0 ,10 + categorieRow*50 + playlistRow*150);
    addItem(_cat2);
    _listCategorie.append(_cat2);
    categorieRow++;

    //! playlist loop for T_DATABASE playlist
    for ( ; i < _plsFileModel->rootItem()->childCount(); i++ ) 
    {
      MediaItemPtr playlistItem = _plsFileModel->rootItem()->child(i);
    
      //WARNING drop smart playlist for this view
      if(playlistItem->data.at(PLAYLIST::type).toInt() != T_FILE) break;

      if(!_plsFileModel->isPlaylistFiltered(playlistItem) ) continue;
    
      _infoSize++;
      idx++; // nouvelle ligne
      
      PlaylistGraphicItem *_playlist = new PlaylistGraphicItem();
      _playlist->media = playlistItem;
      _playlist->setPos(4+160*Column, categorieRow*50 + playlistRow*150);
      addItem(_playlist);    
      
      if(Column < (nbItem-1)) {
        Column++;
      } 
      else {
        Column = 0;
        playlistRow++;
        idx = 0;
      }          
    }
    
    if(_infoSize == oldInfoSize) {
      InfoGraphicItem *item = new InfoGraphicItem();
      item->_text = tr("No entry found");
      item->_width   = _parentWidget->width();
      item->setPos( 0 , 10 + categorieRow*50 + playlistRow*150);
      addItem(item); 
    }       
}



    
    

void CollectionScene::populatePlaylistByTrackScene() 
{
    int categorieRow  = 0;
    int playlistRow   = 0;
    int trackRow      = 0;
    int trackPerAlbum = 0;
    int offset        = 0;
    _infoSize         = 0;
  
    //! track list for virtual playlist
    _listTrack.clear();

    //! categories list (for up/down navigation)
    _listCategorie.clear();

  
    //! button for new smart playlist creation
    ButtonItem* ui_button             = new ButtonItem();
    ui_button->setText(tr("new playlist"));   
    ui_button->setPixmap(QPixmap(":/images/document-new-48x48.png"));
    addItem(ui_button);
    connect(ui_button, SIGNAL(clicked()), this, SLOT(slot_create_new_playlist())); 
    
    //!  button view by icon grid
    ButtonItem* ui_button_2           = new ButtonItem();
    ui_button_2->setText(tr("view grid"));   
    ui_button_2->setData(QVariant(int(MAIN::playlist_overview)));
    addItem(ui_button_2);
    connect(ui_button_2, SIGNAL(clicked()), this, SLOT(slot_change_playlist_view()));     

    int xPos = ui_button->width() + ui_button_2->width() +20+30;
    xPos = (xPos > _parentWidget->width()) ? 30 : (_parentWidget->width() - xPos)/2;

    ui_button->setPos(xPos, 25);
    ui_button_2->setPos(xPos+ ui_button->width() + 20, 25);    
    
    categorieRow++;
  
  
    //! playlist loop
    for ( int i = 0; i < _plsFileModel->rootItem()->childCount(); i++ ) 
    {
      MediaItemPtr playlistItem = _plsFileModel->rootItem()->child(i);
    
      //WARNING drop smart playlist for this view
      if(playlistItem->data.at(PLAYLIST::type).toInt() == T_SMART) break;
      
      if(!_plsFileModel->isPlaylistFiltered(playlistItem) ) continue;
    
      CategorieGraphicItem *_playlistCat = new CategorieGraphicItem();
      _playlistCat->_catName = playlistItem->data.at(PLAYLIST::name).toString();
      _playlistCat->_width   = _parentWidget->width() -20;
      _playlistCat->setPos( 0 , 10 + categorieRow*50 + playlistRow*30 + trackRow*20 + offset );
      addItem(_playlistCat);
      _listCategorie.append(_playlistCat);
    
      categorieRow++;
    
      PlaylistGraphicItem *_playlist = new PlaylistGraphicItem();
      _playlist->media = playlistItem;
      _playlist->setPos(10, categorieRow*50 + playlistRow*30 + trackRow*20 + offset );
      addItem(_playlist);

      //! track loop
      trackPerAlbum = 0;
      for (int k = 0; k < playlistItem->childCount(); k++) 
      {
        MediaItemPtr trackItem = playlistItem->child(k);
        if(!_plsFileModel->isUrlFiltered(trackItem)) continue;

        TrackGraphicItem *_track = new TrackGraphicItem();
        _track->media = trackItem;
        _track->setPos(155, categorieRow*50 + playlistRow*30 + trackRow*20 + offset);
        _track->_width = _parentWidget->width()-155 -20;

        addItem(_track); 
        _listTrack.append(_track);

        trackRow++;
        trackPerAlbum++;
        _infoSize++;
      }
      playlistRow++;
      if( trackPerAlbum < 6 ) offset = offset + (6 - trackPerAlbum)*25;
    }  
  
  
    if(_infoSize==0) {
      CategorieGraphicItem *_cat = new CategorieGraphicItem();
      _cat->_catName = tr("All playlist");
      _cat->_width   = _parentWidget->width()-20;
      _cat->setPos( 0 ,10 + categorieRow*50 + playlistRow*150);
      addItem(_cat);
      _listCategorie.append(_cat);
      categorieRow++;
      
      InfoGraphicItem *item = new InfoGraphicItem();
      item->_text = tr("No entry found");
      item->_width   = _parentWidget->width();
      item->setPos( 0 , 10 + categorieRow*50 + playlistRow*150);
      addItem(item);
    }
}



void CollectionScene::populatePlaylistSmartScene()
{
    int categorieRow = 0;
    int playlistRow  = 0;
    int Column       = 0;
    _infoSize        = 0;
    int idx = 0;
    int nbItem = (_parentWidget->width()/160 > 2) ? _parentWidget->width()/160 : 2;

  
    //! button for new smart playlist creation
    ButtonItem* addSmartButton             = new ButtonItem();
    addSmartButton->setText(tr("new smart playlist"));   
    addSmartButton->setPixmap(QPixmap(":/images/document-new-48x48.png"));
    addItem(addSmartButton);
    connect(addSmartButton, SIGNAL(clicked()), this, SLOT(slot_create_new_smart_playlist()));    

    int xPos = addSmartButton->width() + 30;
    xPos = (xPos > _parentWidget->width()) ? 30 : (_parentWidget->width() - xPos)/2;
    addSmartButton->setPos(xPos, 25);

    
    categorieRow++;
    
    //! categories list (for up/down navigation)
    _listCategorie.clear();
  
  
    CategorieGraphicItem *_cat2 = new CategorieGraphicItem();
    _cat2->_catName = tr("Smart Playlists");
    _cat2->_width   = _parentWidget->width() -20;
    _cat2->setPos( 0 ,10 + categorieRow*50 + playlistRow*150);
    addItem(_cat2);
    _listCategorie.append(_cat2);
    categorieRow++;
      
      
    //! playlist loop
    for ( int i = 0; i < _plsFileModel->rootItem()->childCount(); i++ ) 
    {
      MediaItemPtr playlistItem = _plsFileModel->rootItem()->child(i);
      if(playlistItem->data.at(PLAYLIST::type).toInt() != T_SMART)
        continue;
    
      _infoSize++;
      idx++; // nouvelle ligne
      
      PlaylistGraphicItem *_playlist = new PlaylistGraphicItem();
      _playlist->media = playlistItem;
      _playlist->setPos(4+160*Column, categorieRow*50 + playlistRow*150);
      addItem(_playlist);
    
      if(Column < (nbItem-1)) {
        Column++;
      } 
      else {
        Column = 0;
        playlistRow++;
        idx = 0;
      }
    }
}




void CollectionScene::populateHistoScene()
{
    int categorieRow  = 0;
    int trackRow      = 0;

    _listTrack.clear();
  
    _histoModel->updateModel();
    
    //! button
    ButtonItem* ui_button    = new ButtonItem();
    ui_button->setText(tr("Clear History"));
    addItem(ui_button);
    connect(ui_button, SIGNAL(clicked()), this, SLOT(slot_clear_history()));
    
    ButtonItem* ui_button_2  = new ButtonItem();
    ui_button_2->setText(tr("Reload History"));
    ui_button_2->setPixmap(QPixmap(":/images/rebuild.png"));
    addItem(ui_button_2);
    connect(ui_button_2, SIGNAL(clicked()), this, SLOT(populateScene()));    

    int xPos = ui_button->width() + ui_button_2->width() +20+30;
    xPos = (xPos > _parentWidget->width()) ? 30 : (_parentWidget->width() - xPos)/2;

    ui_button->setPos(xPos, 25);
    ui_button_2->setPos(xPos+ ui_button->width() + 20, 25);

    
    categorieRow++;
  
    //! categories list (for up/down navigation)
    _listCategorie.clear();

    //! current date
    QDate oldDate, newDate;
    
    if (_histoModel->itemCount() == 0) {
      CategorieGraphicItem *cat = new CategorieGraphicItem();
      cat->_catName = tr("Playing history");
      cat->_width   = _parentWidget->width() -20;
      cat->setPos( 0 , 10 + categorieRow*50);
      categorieRow++;
      addItem(cat); 

      InfoGraphicItem *item = new InfoGraphicItem();
      item->_text = tr("No entry found");
      item->_width   = _parentWidget->width();
      item->setPos( 10 ,10 + categorieRow*50);
      addItem(item);   
      return;
    }   
      
    //! track loop from history model
    for ( int i = 0; i < _histoModel->itemCount(); i++ ) 
    {
      if(!_histoModel->isItemFiltered(i)) continue;
    
      MediaItemPtr mi = _histoModel->getItem(i);
    
      QDateTime lastPlayed;
      if(mi->type == TYPE_TRACK)
        lastPlayed = QDateTime::fromString(mi->data.at(TRACK::lastPlayed).toString(), Qt::ISODate);
      else
        lastPlayed = QDateTime::fromString(mi->data.at(STREAM::lastPlayed).toString(), Qt::ISODate);

      newDate = lastPlayed.date(); 

    
      if(oldDate != newDate)
      {
        oldDate = newDate;
     
        CategorieGraphicItem *_cat = new CategorieGraphicItem();
        _cat->_catName = newDate.toString(Qt::ISODate);
        _cat->_width   = _parentWidget->width()-20;
        _cat->setPos(0 , 10 + categorieRow*50 + trackRow*20);
        addItem(_cat);
        _listCategorie.append(_cat);
        
        categorieRow++;     
      }
      
      TrackGraphicItem_v3 *_track = new TrackGraphicItem_v3();
      _track->media = mi;
      _track->setPos(30, categorieRow*50 + trackRow*20);
      _track->_width = _parentWidget->width()-50 -20;
      
      if(mi->type == TYPE_STREAM)
        _track->setToolTip(_track->media->data.at(STREAM::url).toString());

      addItem(_track); 
      _listTrack.append(_track);

      trackRow++;
    }
    
    
    if(_listTrack.isEmpty()) {
      CategorieGraphicItem *cat = new CategorieGraphicItem();
      cat->_catName = tr("History");
      cat->_width   = _parentWidget->width()-20;
      cat->setPos( 0 ,10 + categorieRow*50);

      addItem(cat);
      categorieRow++;
      
      InfoGraphicItem *item = new InfoGraphicItem();
      item->_text = tr("No entry found");
      item->_width   = _parentWidget->width();
      item->setPos( 0 , 10 + categorieRow*50);
      addItem(item);
    }       
}


//! --------------------- contextMenuEvent ------------------------------------- 
void CollectionScene::contextMenuEvent(QGraphicsSceneContextMenuEvent *event) 
{
    //Debug::debug() << " ---- CollectionScene::contextMenuEvent _mouseGrabbedItem" << _mouseGrabbedItem;
    _mouseGrabbedItem = this->mouseGrabberItem();
    if(!_mouseGrabbedItem) return;
  
    m_graphic_item_menu->setGraphicItem(_mouseGrabbedItem);
    m_graphic_item_menu->exec(event->screenPos());
}

void CollectionScene::slot_contextmenu_triggered(ENUM_ACTION_ITEM_MENU id)
{
    switch(id)
    {
      case ALBUM_PLAY         : playAlbum();              break;
      case ALBUM_LOAD_COVER   : loadCoverArtFromFile();   break;
      case ALBUM_LASTFM_COVER : loadLastFmCover();        break;
      case ALBUM_REMOVE_COVER : removeCoverArt();         break;
      case ALBUM_QUEUE_END    : enqueueAlbum(false);      break;
      case ALBUM_QUEUE_NEW    : enqueueAlbum(true);       break;
      case ALBUM_FAVORITE     : updateAlbumFavorite();    break;
      case GENRE_PLAY         : playAlbumGenre();         break;
      case GENRE_QUEUE_END    : enqueueAlbumGenre(false); break;
      case GENRE_QUEUE_NEW    : enqueueAlbumGenre(true);  break;
      case ARTIST_PLAY        : playArtist();             break;
      case ARTIST_QUEUE_END   : enqueueArtist(false);     break;
      case ARTIST_QUEUE_NEW   : enqueueArtist(true);      break;
      case ARTIST_FAVORITE    : updateArtistFavorite();   break;
      case PLAYLIST_PLAY      : playPlaylist();           break;
      case PLAYLIST_EDIT      : edit_playlist_dialog();   break; 
      case PLAYLIST_QUEUE_END : enqueuePlaylist(false);   break;
      case PLAYLIST_QUEUE_NEW : enqueuePlaylist(true);    break;
      case PLAYLIST_FAVORITE  : updatePlaylistFavorite(); break;
      case PLAYLIST_REMOVE    : removePlaylistFromDisk(); break;
      case TRACK_PLAY         : playTrack();              break;
      case TRACK_QUEUE_END    : enqueueTrack(false);      break;
      case TRACK_QUEUE_NEW    : enqueueTrack(true);       break;

      case ALBUM_EDIT         :
      case GENRE_EDIT         :
      case ARTIST_EDIT        :
      case TRACK_EDIT         : edit_media_dialog();      break;
      default:break;
    }
}
   
//! --------------------- slot_item_doubleclick --------------------------------
//! récupération du double click sur un item par une action globale
void CollectionScene::slot_item_doubleclick()
{
   QList<QGraphicsView *> view = this->views();
   if(view.isEmpty()) {
     //Debug::debug() << "CollectionScene::View Not used";   
     return;
   }
   
   //Debug::debug() << "CollectionScene::slot_item_doubleclick";
   QMap<ENUM_ACTION, QAction*> *actions = ACTIONS();
  
   QGraphicsItem *gItem = qvariant_cast<QGraphicsItem *>( (actions->value(BROWSER_ITEM_DOUBLE_CLICK))->data() );

   _mouseGrabbedItem = gItem;
   if(!_mouseGrabbedItem) return;
   
   if(Qt::ControlModifier == QApplication::keyboardModifiers()) 
   {
     switch(gItem->type()) {
       case GraphicsItem::AlbumType      : enqueueAlbum(false);break;
       case GraphicsItem::AlbumGenreType : enqueueAlbumGenre(false);break;
       case GraphicsItem::ArtistType     : enqueueArtist(false);break;
       case GraphicsItem::TrackType      : enqueueTrack(false);break;
       case GraphicsItem::PlaylistType   : enqueuePlaylist(false);break;
       default:break;    
     }
   }
   else
   {
     switch(gItem->type()) {
       case GraphicsItem::AlbumType      : playAlbum();break;
       case GraphicsItem::AlbumGenreType : playAlbumGenre();break;
       case GraphicsItem::ArtistType     : playArtist();break;
       case GraphicsItem::TrackType      : playTrack();break;
       case GraphicsItem::PlaylistType   : playPlaylist();break;
       default:break;    
     }
   }
}

/*******************************************************************************
  CollectionScene::get_item_position
*******************************************************************************/
QPoint CollectionScene::get_item_position(MediaItemPtr media) 
{
    Debug::debug() << "CollectionScene::get_item_position"; 
    QPoint point = QPoint(0,0);
    if(media->type == TYPE_ARTIST && _collectionMode == MAIN::ViewArtist)
    {
       foreach(QGraphicsItem* item, this->items()) {
         ArtistGraphicItem *artist_item = dynamic_cast<ArtistGraphicItem*>(item);
         if(!artist_item) continue;

         if(artist_item->media->data.at(ARTIST::id).toInt() == media->data.at(ARTIST::id).toInt()) {
             point = artist_item->scenePos().toPoint();
             break;
         }
       }
    }
    else if(media->type == TYPE_ALBUM && _collectionMode == MAIN::ViewAlbum)
    {
       foreach(QGraphicsItem* item, this->items()) {
         AlbumGraphicItem *album_item = dynamic_cast<AlbumGraphicItem*>(item);
         if(!album_item) continue;

         if(album_item->media->data.at(ALBUM::id).toInt() == media->data.at(ALBUM::id).toInt()) {
             point = album_item->scenePos().toPoint();
             break;
         }
       }      
    }
    else if(media->type == TYPE_TRACK && _collectionMode == MAIN::ViewTrack)
    {
      QList<MediaItemPtr> list;
      foreach(TrackGraphicItem* item, _listTrack) {
          if(item->media->data.at(TRACK::id).toInt() == media->data.at(TRACK::id).toInt()) {
            point = item->scenePos().toPoint();
            break;
	  }
      }
    }
    

    
    return point;
}


/*******************************************************************************
    album cover stuff
*******************************************************************************/
void CollectionScene::loadCoverArtFromFile() 
{
    //Debug::debug() << "CollectionScene::loadCoverArtFromFile";
    if(!_mouseGrabbedItem) return;
    AlbumGraphicItem *item = static_cast<AlbumGraphicItem*>(_mouseGrabbedItem);

    //! open file dialog  
    QString filename = QFileDialog::getOpenFileName(new QWidget(), tr("Select cover file..."),tr("Files (*.png *.jpg)"));
    if( filename.isEmpty() ) return;

    const QString cover_path = UTIL::CONFIGDIR + "/albums/" + item->media->data.at(ALBUM::coverpath).toString();

    //! get new cover file
    QImage image = QImage(filename);
    image = image.scaled(QSize(110, 110), Qt::KeepAspectRatio, Qt::SmoothTransformation);

    //! remove existing cover file
    if(QFile::exists(cover_path)) 
    {
      QFile::remove(cover_path);
      QPixmapCache::remove ( item->media->data.at(ALBUM::coverpath).toString() );
    }
    //! save new cover file
    image.save(cover_path, "png", -1);

    //! scene update
    item->update();
}

void CollectionScene::loadLastFmCover() 
{
    if(!_mouseGrabbedItem) return;
    AlbumGraphicItem *item = static_cast<AlbumGraphicItem*>(_mouseGrabbedItem);

    //! Get album Name and coverPath
    const QString album      =  item->media->data.at(ALBUM::name).toString();
    const QString artist     =  item->media->parent()->data.at(ARTIST::name).toString();

    //! start search cover process
    ThreadManager::instance()->startCoverSearch(artist, album);
}

void CollectionScene::removeCoverArt()
{
    //Debug::debug() << "CollectionScene::RemoveCoverArtForAlbum";
    if(!_mouseGrabbedItem) return;
    AlbumGraphicItem *item = static_cast<AlbumGraphicItem*>(_mouseGrabbedItem);

    const QString cover_path =  UTIL::CONFIGDIR + "/albums/" + item->media->data.at(ALBUM::coverpath).toString();

    //! remove existing cover file
    if(QFile::exists(cover_path)) 
    {
      QFile::remove(cover_path);
      QPixmapCache::remove ( item->media->data.at(ALBUM::coverpath).toString() );
      item->update();
    }
} 


/*******************************************************************************
    Edit media dialog
*******************************************************************************/
void CollectionScene::edit_media_dialog()
{
    //Debug::debug() << "CollectionScene::edit_media_dialog ";
    MediaItem_Edit_Dialog *dialog = 0;
    
    if(!_mouseGrabbedItem) return;
    
    if(_mouseGrabbedItem->type()== GraphicsItem::ArtistType)
    {
      ArtistGraphicItem *item = static_cast<ArtistGraphicItem*>(_mouseGrabbedItem);
      dialog = new MediaItem_Edit_Dialog(DIALOG_ARTIST);
      dialog->setMediaItem(item->media);    
    }
    else if(_mouseGrabbedItem->type()== GraphicsItem::AlbumType)
    {
      AlbumGraphicItem *item = static_cast<AlbumGraphicItem*>(_mouseGrabbedItem);
      dialog = new MediaItem_Edit_Dialog(DIALOG_ALBUM);
      dialog->setMediaItem(item->media);
    }
    else if(_mouseGrabbedItem->type()== GraphicsItem::AlbumGenreType)
    {
      AlbumGenreGraphicItem *item = static_cast<AlbumGenreGraphicItem*>(_mouseGrabbedItem);
      dialog = new MediaItem_Edit_Dialog(DIALOG_GENRE);
      dialog->setGenre(item->_genre);
      dialog->setMediaItem(item->media);
    } 
    else if(_mouseGrabbedItem->type()== GraphicsItem::TrackType)
    {      
      TrackGraphicItem *item = static_cast<TrackGraphicItem*>(_mouseGrabbedItem);
      dialog = new MediaItem_Edit_Dialog(DIALOG_TRACK);
      dialog->setMediaItem(item->media);    
    }    

    if(dialog) {
      //! blocking call (modal dialog)
      dialog->exec();    

      if( (dialog->result() == QDialog::Accepted) && dialog->isChanged()) {
        emit userDatabaseUpdate(1);      
      }
      dialog->deleteLater();
    }
}


/*******************************************************************************
    Update favorite status
*******************************************************************************/
void CollectionScene::updateAlbumFavorite()
{
    if(!_mouseGrabbedItem) return;
    AlbumGraphicItem *item = static_cast<AlbumGraphicItem*>(_mouseGrabbedItem);

    int _albumBdId    = item->media->data.at(ALBUM::id).toInt();
    bool isFavorite   = _collectionModel->itemIsFavorite(item->media);
  
    _collectionModel->setItemIsFavorite(item->media, !isFavorite);
 
    Database db;
    if (!db.connect("MainThread")) return;
   
    QSqlQuery query("UPDATE `albums` "       \
                    "SET `favorite`="+QString::number(!isFavorite)+" " \
                    "WHERE `id`="+QString::number(_albumBdId)+";",*db.sqlDb());
    query.exec();
   
    //! item update
    item->update();
}


void CollectionScene::updateArtistFavorite()
{
    if(!_mouseGrabbedItem) return;
    ArtistGraphicItem *item = static_cast<ArtistGraphicItem*>(_mouseGrabbedItem);

    int _artistBdId = item->media->data.at(ARTIST::id).toInt();
    bool isFavorite = _collectionModel->itemIsFavorite(item->media);
  
    _collectionModel->setItemIsFavorite(item->media, !isFavorite);
 
    Database db;
    if (!db.connect("MainThread")) return;
   
    QSqlQuery query("UPDATE `interprets` "       \
                     "SET `favorite`="+QString::number(!isFavorite)+" " \
                     "WHERE `id`="+QString::number(_artistBdId)+";",*db.sqlDb());
    query.exec();
   
    //! item update
    item->update();
}

/*******************************************************************************
    Playlist stuff
*******************************************************************************/
void CollectionScene::removePlaylistFromDisk()
{
    if(!_mouseGrabbedItem) return;
    PlaylistGraphicItem *item = static_cast<PlaylistGraphicItem*>(_mouseGrabbedItem);

    //! get data 
    const int dbId       = item->media->data.at(PLAYLIST::id).toInt();
    const QString fname  = item->media->data.at(PLAYLIST::url).toString();
    const int type       = item->media->data.at(PLAYLIST::type).toInt(); 
         
    //Debug::debug() << "CollectionScene:: remove playlist item id" << dbId;

    //! playlist is a local file on disk --> remove file
    if(type == T_FILE) { 
      QFile playlistFile(fname);
      if(playlistFile.exists())
        playlistFile.remove();
    }
     
    //! remove from database
    Database db;
    if (!db.connect("MainThread")) return;
   
    QSqlField data("col",QVariant::String);
    data.setValue(dbId);
    QString id = db.sqlDb()->driver()->formatValue(data,false);
   
    if(type == T_DATABASE) 
    { 
      QSqlQuery query("DELETE FROM `playlists` WHERE `id`="+id+";",*db.sqlDb());
      query.exec();
    
      QSqlQuery query2("DELETE FROM `playlist_items` WHERE `playlist_id` NOT IN (SELECT `id` FROM `playlists`);", *db.sqlDb());
      query2.exec();
    }
    else //T_SMART
    {
      QSqlQuery query("DELETE FROM `smart_playlists` WHERE `id`="+id+";",*db.sqlDb());
      query.exec();
    }
    
    emit userDatabaseUpdate(2);
}


void CollectionScene::edit_playlist_dialog()
{
    if(!_mouseGrabbedItem) return;
    PlaylistGraphicItem *item = static_cast<PlaylistGraphicItem*>(_mouseGrabbedItem);

    const int type       = item->media->data.at(PLAYLIST::type).toInt(); 
    
    if(type == T_DATABASE || type == T_FILE)
    {
      PlaylistEditor *editor = new PlaylistEditor(_parentWidget);
      editor->setPlaylist(item->media);
      editor->show();
      
      QSignalMapper* mapper = new QSignalMapper();
      mapper->setMapping(editor, 2);

      connect(editor, SIGNAL(playlistSaved()), mapper, SLOT(map()));
      connect(mapper, SIGNAL(mapped(int)), this, SIGNAL(userDatabaseUpdate(int))); 

      connect(editor, SIGNAL(closed()), this, SLOT(slot_delete_playlist_editor()));      
    }
    else //T_SMART
    {
      // open smart playlist editor with current smart playlist
      if( SmartPlaylist::edit_dialog( item->media ) )
        emit userDatabaseUpdate(2);      
    }    
}

/*******************************************************************************
    Playlist creation
*******************************************************************************/
void CollectionScene::slot_create_new_smart_playlist()
{
   // open smart playlist editor for a new one
   if( SmartPlaylist::create_dialog() )
     emit userDatabaseUpdate(2);
}

void CollectionScene::slot_create_new_playlist()
{
    PlaylistEditor *editor = new PlaylistEditor(_parentWidget);
    editor->show();
      
    QSignalMapper* mapper = new QSignalMapper();
    mapper->setMapping(editor, 2);

    connect(editor, SIGNAL(playlistSaved()), mapper, SLOT(map()));
    connect(mapper, SIGNAL(mapped(int)), this, SIGNAL(userDatabaseUpdate(int)));

    connect(editor, SIGNAL(closed()), this, SLOT(slot_delete_playlist_editor()));      
}


void CollectionScene::slot_delete_playlist_editor()
{
    PlaylistEditor *editor = qobject_cast<PlaylistEditor *>(sender());
    if(editor)
      editor->deleteLater();  
}


void CollectionScene::updatePlaylistFavorite()
{
    if(!_mouseGrabbedItem) return;
    PlaylistGraphicItem *item = static_cast<PlaylistGraphicItem*>(_mouseGrabbedItem);

    //! get data 
    const int dbId       = item->media->data.at(PLAYLIST::id).toInt();
    const int type       = item->media->data.at(PLAYLIST::type).toInt(); 
    bool isFavorite      = _plsFileModel->itemIsFavorite(item->media);
    
    _plsFileModel->setItemIsFavorite(item->media, !isFavorite);

    Database db;
    if (!db.connect("MainThread")) return;

    if(type == T_DATABASE || type == T_FILE)
    {
      QSqlQuery query("UPDATE `playlists` "       \
                      "SET `favorite`="+QString::number(!isFavorite)+" " \
                      "WHERE `id`="+QString::number(dbId)+";",*db.sqlDb());
      query.exec();     
    }
    else //T_SMART
    {
      QSqlQuery query("UPDATE `smart_playlists` "       \
                      "SET `favorite`="+QString::number(!isFavorite)+" " \
                      "WHERE `id`="+QString::number(dbId)+";",*db.sqlDb());
      query.exec();     
    }   
    
    //! item update
    item->update();
}

/*******************************************************************************
    History model clear
*******************************************************************************/
void CollectionScene::slot_clear_history() 
{
    HistoManager::instance()->clearHistory();
    populateScene();
}
