/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef _ENGINE_PLAYER_H_
#define _ENGINE_PLAYER_H_

// Qt
#include <QObject>
#include <QString>
#include <QTimer>
#include <QPointer>
#include <QPixmap>

// Phonon
#include <phonon/mediasource.h>
#include <phonon/mediaobject.h>
#include <phonon/audiooutput.h>
#include <phonon/path.h>
#include <phonon/effect.h>
#include <phonon/volumefadereffect.h>

#include "MediaItem/mediaitem.h"


namespace ENGINE {
  enum E_ENGINE_STATE {PLAYING, PAUSED, STOPPED, ERROR};
}

/*
********************************************************************************
*                                                                              *
*    Class EnginePlayer                                                        *
*                                                                              *
********************************************************************************
*/
class EnginePlayer : public QObject
{ 
Q_OBJECT
    static EnginePlayer         *INSTANCE;

  public:
    EnginePlayer();
    ~EnginePlayer();
    static EnginePlayer* instance() { return INSTANCE; }
    
    // engine player state
    ENGINE::E_ENGINE_STATE engineState() { return _engineState; }

    // engine error string
    const QString engineError() { return _mediaObject->errorString(); }

    // current playing Item info
    MediaItemPtr enginePlayingMediaItem() {return m_currentMediaItem;}
    QString enginePlayingUrl() {return m_playing_url;}
    const QPixmap enginePlayingPixmap() {return m_playing_pixmap;}
    qint64 enginePlayingTrackLength();
    
    
    // phonon objects
    Phonon::MediaObject* mediaObject() { return _mediaObject; }  // use for SeekSlider
    Phonon::AudioOutput* audioOutput() { return _audioOutput; }  // use for VolumeSlider
    
    // equalizer method 
    bool isEqualizerAvailable();
    void addEqualizer();
    void removeEqualizer();
    void applyEqualizer(QList<int>);
    void loadEqualizerSettings();

    Phonon::Effect* equalizer() const {return _audioEqualizer;}
    
    // volume method
    int volume() const {return _playerVolume;}
    bool isMuted() const {return _audioOutput->isMuted();}
    
  public slots:
    void play();
    void pause();
    void stop();
    void setMediaItem(MediaItemPtr mi);
    void setNextMediaItem(MediaItemPtr mi);

    int  setVolume( int percent );
    void setMuted( bool mute );
  
  private:
    QString currentMediaSource();
    void    setCover();
    QString stateToString(ENGINE::E_ENGINE_STATE state);

  private slots:
    void slot_queue_finished();
    void slot_media_aboutToFinish();
    void slotTrackChanged( const Phonon::MediaSource & );
    void slotTrackTotalTimeChanged(qint64);
    void slotMetaDataChanged();        
    void slotStateChanged(Phonon::State newState, Phonon::State oldState);
    void slotVolumeChanged( qreal );
    void slotMutedChanged( bool );
    
  private:
    ENGINE::E_ENGINE_STATE  _engineState;
    ENGINE::E_ENGINE_STATE  _oldState;
    Phonon::MediaObject     *_mediaObject;
    Phonon::AudioOutput     *_audioOutput;
    Phonon::Path            _phononPath;
    Phonon::Effect          *_audioEqualizer;
    QPointer<Phonon::VolumeFaderEffect> _preamp;
    
    int                     _playerVolume;

    MediaItemPtr            m_currentMediaItem;
    MediaItemPtr            m_nextMediaItem;
    QString                 m_playing_url;
    QPixmap                 m_playing_pixmap;
    
  signals:
    void trackChanged(const QString&);
    void playingInfoChanged();
    void streamInfoChanged();    
    void trackAboutToFinish();           // 2 seconds before and of playing track
    void queueFinished();                // emit only if all track in queue have been played
    
    void player_stop_requested();
    
    void engineStateChanged();
    void engineLoadingError(const QString&);
    
    void trackPositionChanged(qint64 newPos);
    void trackTotalTimeChanged(qint64 newTotalTime);
    void volumeChanged( int percent );
    void muteStateChanged( bool mute );
};

#endif // _ENGINE_PLAYER_H_
