/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "GraphicViewWidget/graphicviewwidget.h"

#include "CollectionModel/collectionmodel.h"
#include "CollectionModel/plsfilemodel.h"
#include "CollectionModel/histomodel.h"
#include "StreamModel/streammodel.h"

// scene
#include "CollectionView/collectionscene.h"
#include "StreamView/stream_scene.h"
#include "ContextView/contextscene.h"
#include "Settings/settings_scene.h"


#include "CentralWidget/centraltoolbar.h"
#include "Widget/statuswidget.h"

#include "yarocksettings.h"         // 0.0.53

#include "global_actions.h"
#include "debug.h"

/*
********************************************************************************
*                                                                              *
*    GraphicViewWidget                                                         *
*                                                                              *
********************************************************************************
*/
GraphicViewWidget::GraphicViewWidget(QWidget *parent) : QGraphicsView(parent)
{
    QPalette palette;
    palette.setColor(QPalette::Background, Qt::white);
    //palette.setColor(QPalette::Window , Qt::white);
    //palette.setColor(QPalette::Foreground, Qt::white);
    palette.setColor(QPalette::Base, Qt::white);
    //palette.setColor(QPalette::AlternateBase, Qt::white);
    palette.setColor(QPalette::Button,  QColor(240,240,240));
  
    //! widget setup
    this->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
    this->setAutoFillBackground(false);
  
    //! GraphicView Setup  
    this->setFrameShape(QFrame::NoFrame);
    this->setFrameShadow(QFrame::Plain);
    this->setPalette(palette);
      
    this->setCacheMode(QGraphicsView::CacheNone);
    this->setDragMode(QGraphicsView::NoDrag); 
    this->setRenderHint(QPainter::Antialiasing);
    //this->setAttribute(Qt::WA_OpaquePaintEvent);// a small optimization 0.0.45c
    this->setOptimizationFlags(QGraphicsView::DontAdjustForAntialiasing
                                   | QGraphicsView::DontClipPainter
                                   | QGraphicsView::DontSavePainterState);
    this->setResizeAnchor(QGraphicsView::NoAnchor);
    this->setViewportUpdateMode(QGraphicsView::MinimalViewportUpdate); 
    this->setAlignment(Qt::AlignLeft | Qt::AlignVCenter); 
    
    //! scrolbar setup
    connect (this->verticalScrollBar(), SIGNAL(actionTriggered(int)), this, SLOT(checkSlider(int)));

    this->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    this->verticalScrollBar()->setSingleStep(10);
    this->verticalScrollBar()->setPageStep(30); 
    this->verticalScrollBar()->setContextMenuPolicy(Qt::NoContextMenu);
    
    //! default scroll position
    m_scrollPosition[MAIN::ViewSettings]       = 0;
    m_scrollPosition[MAIN::ViewAlbum]          = 0;
    m_scrollPosition[MAIN::ViewArtist]         = 0;
    m_scrollPosition[MAIN::ViewTrack]          = 0;
    m_scrollPosition[MAIN::ViewGenre]          = 0;
    m_scrollPosition[MAIN::ViewFavorite]       = 0;
    m_scrollPosition[MAIN::ViewPlaylist]       = 0;
    m_scrollPosition[MAIN::ViewSmartPlaylist]  = 0;
    m_scrollPosition[MAIN::ViewIceCast]        = 0;
    m_scrollPosition[MAIN::ViewShoutCast]      = 0;
    m_scrollPosition[MAIN::ViewCustomRadio]    = 0;
    m_scrollPosition[MAIN::ViewFavoriteRadio]  = 0;
    m_scrollPosition[MAIN::ViewDashBoard]      = 0;    
    m_scrollPosition[MAIN::ViewContext]        = 0;
    m_scrollPosition[MAIN::ViewHistory]        = 0;
    
    //! action to change view
    QActionGroup* group = new QActionGroup(this);
    
    group->addAction(ACTIONS()->value(BROWSER_VIEW_SETTINGS));
    group->addAction(ACTIONS()->value(BROWSER_VIEW_ALBUM));
    group->addAction(ACTIONS()->value(BROWSER_VIEW_ARTIST));
    group->addAction(ACTIONS()->value(BROWSER_VIEW_TRACK));
    group->addAction(ACTIONS()->value(BROWSER_VIEW_GENRE));
    group->addAction(ACTIONS()->value(BROWSER_VIEW_YEAR));
    group->addAction(ACTIONS()->value(BROWSER_VIEW_FAVORITE));
    group->addAction(ACTIONS()->value(BROWSER_VIEW_PLAYLIST));
    group->addAction(ACTIONS()->value(BROWSER_VIEW_SMART_PLAYLIST));
    group->addAction(ACTIONS()->value(BROWSER_VIEW_ICECAST));
    group->addAction(ACTIONS()->value(BROWSER_VIEW_SHOUTCAST));
    group->addAction(ACTIONS()->value(BROWSER_VIEW_RADIO));
    group->addAction(ACTIONS()->value(BROWSER_VIEW_FAVORITE_RADIO));
    group->addAction(ACTIONS()->value(BROWSER_VIEW_DASHBOARD));
    group->addAction(ACTIONS()->value(BROWSER_VIEW_CONTEXT));
    group->addAction(ACTIONS()->value(BROWSER_VIEW_HISTO));

    connect(group, SIGNAL(triggered(QAction*)), this, SLOT(switchView(QAction*)), Qt::DirectConnection);
    
    //! connect toolbar signals
    connect(CentralToolBar::instance(), SIGNAL(searchActivated(const QString)),this, SLOT(updateSearch(const QString)));

    //! restore last state
    _viewMode = (MAIN::ViewMode) SETTINGS()->_viewMode;
}


/*******************************************************************************
    set scene method
*******************************************************************************/
void GraphicViewWidget::setCollectionScene(CollectionScene *scene)
{
    _collectionScene  = scene;
}


void GraphicViewWidget::setStreamScene(StreamScene *scene)
{
    _streamScene  = scene;
    connect(_streamScene, SIGNAL(repopulateStreamScene()),this, SLOT(switchView()));
}

void GraphicViewWidget::setContextScene(ContextScene *scene)
{
    _contextScene  = scene;
}

void GraphicViewWidget::setSettingsScene(SettingsScene *scene)
{
    _settingsScene  = scene;
}

    
    
/*******************************************************************************
    updateView
*******************************************************************************/
//! only use after database rebuild (MainWindow)
void GraphicViewWidget::updateView() 
{
    //Debug::debug() << "GraphicViewWidget --> updateView " << _viewMode;
    switchView();
}

/*******************************************************************************
    resizeEvent
*******************************************************************************/
void GraphicViewWidget::resizeEvent( QResizeEvent * event)
{
    switchView();

    event->accept();
}


/*******************************************************************************
    switchView
*******************************************************************************/
void GraphicViewWidget::switchView(QAction* action)
{
    bool resize_event = false;
    
    MAIN::ViewMode old_mode = _viewMode;
    
    m_scrollPosition[old_mode]  = this->verticalScrollBar()->sliderPosition();
   
    if(action)
      _viewMode = MAIN::ViewMode(action->data().toInt());
    else
      resize_event = true;
    
    Debug::debug() << "GraphicViewWidget::switchView : " << _viewMode;
 
    _contextScene->setActivate(false);
    
    switch(_viewMode)
    {
      case MAIN::ViewSettings        :
        this->setScene(_settingsScene);
        _settingsScene->populate();
        if(!resize_event)
          _settingsScene->restore_settings();
        break;
      
      case MAIN::ViewContext        :
        _contextScene->setActivate(true);
        _contextScene->startContextScene();
        this->setScene(_contextScene);
        break;
            
      case MAIN::ViewAlbum          :
      case MAIN::ViewArtist         :
      case MAIN::ViewTrack          :
      case MAIN::ViewGenre          :
      case MAIN::ViewYear           :
      case MAIN::ViewFavorite       :
      case MAIN::ViewDashBoard      :
      case MAIN::ViewHistory        :
      case MAIN::ViewPlaylist       :
      case MAIN::ViewSmartPlaylist  :      
        _collectionScene->_collectionMode = _viewMode;
        _collectionScene->populateScene();
        this->setScene(_collectionScene);            
        break;
        
      case MAIN::ViewIceCast        :
      case MAIN::ViewShoutCast      :
      case MAIN::ViewCustomRadio    :
      case MAIN::ViewFavoriteRadio  :
        _streamScene->_view_mode           = _viewMode;
        _streamScene->populateScene();    
        this->setScene(_streamScene);  
            
      default:return;break;
    }

    //!restore scroll position
   /* if(this->scene()->sceneRect().toRect().height() < m_scrollPosition[_viewMode])
      m_scrollPosition[_viewMode] = 1;*/

    this->verticalScrollBar()->setSliderPosition(m_scrollPosition[_viewMode]);
    
    updateStatusWidget();
}


/*******************************************************************************
    Search management
*******************************************************************************/
void GraphicViewWidget::updateSearch(const QString& filter) 
{
    //Debug::debug() << "GraphicViewWidget --> updateSearch  " << filter;

    CollectionModel::instance()->setFilter(filter);
    PlsFileModel::instance()->setFilter(filter);
    HistoModel::instance()->setFilter(filter);
    StreamModel::instance()->setFilter(filter);

    switchView();
}

void GraphicViewWidget::setSearch(const QString& search) 
{
    CollectionModel::instance()->setFilter(search);
    PlsFileModel::instance()->setFilter(search);
    HistoModel::instance()->setFilter(search);
    StreamModel::instance()->setFilter(search);
}


/*******************************************************************************
    updateStatusWidget
*******************************************************************************/
void GraphicViewWidget::updateStatusWidget() 
{
    const int collectionSize = _collectionScene->getSceneInfoSize();
    const int radioSize      = _streamScene->getSceneInfoSize();
  
    QString     text;
  
    switch( _viewMode )
    {
      case (MAIN::ViewContext)  : text = tr("Context view");break;
      case (MAIN::ViewHistory)  : text = tr("History view");break;      
      case (MAIN::ViewAlbum)    : text = QString(tr("Collection : <b>%1</b> albums")).arg(QString::number(collectionSize)); break;
      case (MAIN::ViewArtist)   : text = QString(tr("Collection : <b>%1</b> artist")).arg(QString::number(collectionSize));break;
      case (MAIN::ViewTrack)    : text = QString(tr("Collection : <b>%1</b> tracks")).arg(QString::number(collectionSize));break;
      case (MAIN::ViewGenre)    : text = QString(tr("Collection : <b>%1</b> styles")).arg(QString::number(collectionSize));break;
      case (MAIN::ViewYear)     : text = QString(tr("Collection : <b>%1</b> years")).arg(QString::number(collectionSize));break;
      case (MAIN::ViewFavorite) : text = QString(tr("Collection : <b>%1</b> favorite item")).arg(QString::number(collectionSize));break;
      case (MAIN::ViewPlaylist) : text = QString(tr("Playlist : <b>%1</b> playlist files")).arg(QString::number(collectionSize));break;
      case (MAIN::ViewSmartPlaylist)  : text = QString(tr("Playlist : <b>%1</b> smart playlist")).arg(QString::number(collectionSize));break;
      case (MAIN::ViewIceCast)        : text = QString(tr("Radio : <b>%1</b> icecast streams")).arg(QString::number(radioSize));break;
      case (MAIN::ViewShoutCast)      : text = QString(tr("Radio : <b>%1</b> shoutcast streams")).arg(QString::number(radioSize));break;
      case (MAIN::ViewCustomRadio)    : text = QString(tr("Radio : <b>%1</b> streams")).arg(QString::number(radioSize));break;       
      case (MAIN::ViewFavoriteRadio)  : text = QString(tr("Radio : <b>%1</b> streams")).arg(QString::number(radioSize));break;
      default: text = "";break;
    }
    
    if(!text.isEmpty())
      StatusWidget::instance()->startShortMessage(text, STATUS::TYPE_INFO, 2500);
}


/*******************************************************************************
    checkSlider
*******************************************************************************/
void GraphicViewWidget::checkSlider(int action) 
{
    //Debug::debug() << "GraphicViewWidget::checkSlider ACTION" << action;
    switch(action)
    {
      case (QAbstractSlider::SliderPageStepAdd):
        navigateGoDown();
        break;
      case (QAbstractSlider::SliderPageStepSub):
        navigateGoUp();
        break;
    };
}

/*******************************************************************************
    navigateGoUp / navigateGoDown
*******************************************************************************/
void GraphicViewWidget::navigateGoUp() 
{
    QList<CategorieGraphicItem*> listCategorie;
  
    if(dynamic_cast<CollectionScene*>(this->scene()))
       listCategorie      = dynamic_cast<CollectionScene*>(this->scene())->_listCategorie;
    else if(dynamic_cast<StreamScene*>(this->scene()))
       listCategorie      = dynamic_cast<StreamScene*>(this->scene())->_listCategorie;
    else
      return;
    
    int top     = this->verticalScrollBar()->sliderPosition();

    for(int i=listCategorie.size()-1; i>=0;i--)
    {
      QPoint itemPoint = listCategorie.at(i)->scenePos().toPoint();

      if(top > itemPoint.y()) {
        this->verticalScrollBar()->setSliderPosition(itemPoint.y());
        break;
      }
    }
}

void GraphicViewWidget::navigateGoDown() 
{
    QList<CategorieGraphicItem*> listCategorie;
  
    if(dynamic_cast<CollectionScene*>(this->scene()))
       listCategorie      = dynamic_cast<CollectionScene*>(this->scene())->_listCategorie;
    else if(dynamic_cast<StreamScene*>(this->scene()))
       listCategorie      = dynamic_cast<StreamScene*>(this->scene())->_listCategorie;
    else
      return;
    
    int top     = this->verticalScrollBar()->sliderPosition();
  
    foreach (QGraphicsItem *item, listCategorie)
    {
      QPoint itemPoint = (item->scenePos()).toPoint();

      if(top < itemPoint.y()) {
        this->verticalScrollBar()->setSliderPosition(itemPoint.y());
      break;
      }
    }
}

/*******************************************************************************
    set_scroll_to
*******************************************************************************/
void GraphicViewWidget::set_scroll_to(QPoint p)
{
    if(p.isNull()) return;
    
    this->verticalScrollBar()->setSliderPosition( p.y()-40 );
}


