/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "MediaSearch/media_search_engine.h"
#include "CollectionModel/collectionmodel.h"
#include "debug.h"


#include <algorithm>
#include <boost/bind/bind.hpp>

/*
********************************************************************************
*                                                                              *
*    Class SearchEngine                                                        *
*                                                                              *
********************************************************************************
*/
SearchEngine::SearchEngine()
{
    model_                    = CollectionModel::instance();
    
    search_.search_type_      = MediaSearch::Type_And;
    search_.query_list_.clear();        
    search_.sort_type_        = MediaSearch::Sort_No;
    search_.sort_field_       = SearchQuery::field_track_trackname;
    search_.limit_            = -1;
}
 

void SearchEngine::init_search_engine(const MediaSearch& search)
{
    search_ = MediaSearch(search);
#ifdef TEST_FLAG
    Debug::debug() << "SearchEngine::init_search_engine search_type_ :" << (int)search_.search_type_;
    
    foreach(SearchQuery query, search_.query_list_) {
      Debug::debug() << "SearchEngine::init_search_engine query_list_  query.field_ :" << SearchQuery::FieldName(query.field_);
      Debug::debug() << "SearchEngine::init_search_engine query_list_  query.operator_ :" << query.operator_;
      Debug::debug() << "SearchEngine::init_search_engine query_list_  query.value_ :" << query.value_;
    }
    
    Debug::debug() << "SearchEngine::init_search_engine sort_type_ :" << (int)search_.sort_type_;
    Debug::debug() << "SearchEngine::init_search_engine sort_field_ :" << SearchQuery::FieldName(search_.sort_field_);
    Debug::debug() << "SearchEngine::init_search_engine limit_ :" << (int)search_.limit_;
#endif  
}
     
     

     

void SearchEngine::doSearch() 
{
    Debug::debug() << "SearchEngine::doSearch";

    list_result_media_.clear();
    // un-ordered media track 
    QList<MediaItemPtr> list_media = model_->trackItemHash.values();
    
    QList<MediaItemPtr> list_media2; 
    
    //! sort random
    if(search_.sort_type_ == MediaSearch::Sort_Random) {
      std::random_shuffle( list_media.begin(), list_media.end() ); 
    }
    //! sort by field
    else if(search_.sort_type_ == MediaSearch::Sort_FieldAsc || search_.sort_type_ == MediaSearch::Sort_FieldDesc ) 
    { 
        if(search_.sort_field_ == SearchQuery::field_track_lastPlayed) 
        {
          for (int index = 0; index < list_media.size(); index++) {
            if( list_media.at(index)->data.at(TRACK::lastPlayed).toDate() != QDate())
              list_media2 << list_media.at(index);
            }
            list_media = list_media2;
        }
      
        qSort(list_media.begin(), list_media.end(), 
              boost::bind( &SearchEngine::sortMediaItem, this, _1, _2)
        );
    }    
   
    foreach (MediaItemPtr media, list_media) 
    {
        bool skip_track = false;
        
        if(search_.search_type_ == MediaSearch::Type_All) 
        {
            list_result_media_ << media;
        }
        else 
        {      
            for(int i =0; i< search_.query_list_.size();i++)
            {
              SearchQuery::Search_Field     field     = search_.query_list_.at(i).field_;
              SearchQuery::Search_Operator  op        = search_.query_list_.at(i).operator_;
              QVariant                      pattern   = search_.query_list_.at(i).value_;
    
              QVariant value = QVariant::fromValue(fieldData(field,media));

              bool result =  match(op, SearchQuery::TypeOf(field), value, pattern);

              if(result == true && search_.search_type_ == MediaSearch::Type_Or) 
              {
                list_result_media_ << media;
                skip_track = true;
              }
              else if(result == false && search_.search_type_ == MediaSearch::Type_And) {
                skip_track = true;   
              }      
              else if(result == true && search_.search_type_ == MediaSearch::Type_And &&  (i == search_.query_list_.size()-1))
              {
                list_result_media_ << media;            
              }

              if(skip_track) break;
            } // end for

            if(skip_track) continue;
      }

      //! limit result    
      if(search_.limit_ != -1) 
      {
         if(list_result_media_.size() >= search_.limit_)
            break;
      }    
    } // end foreach
    
}


QVariant SearchEngine::fieldData(SearchQuery::Search_Field field, MediaItemPtr media)
{
    switch(field) 
    {
      case SearchQuery::field_track_filename  : return media->data.at(TRACK::url);break;
      case SearchQuery::field_track_trackname : return media->data.at(TRACK::title);break;
      case SearchQuery::field_track_number    : return media->data.at(TRACK::num);break;
      case SearchQuery::field_track_length    : return media->data.at(TRACK::duration);break;
      case SearchQuery::field_artist_name     : return media->data.at(TRACK::artist);break;
      case SearchQuery::field_album_name      : return media->data.at(TRACK::album);break;
      case SearchQuery::field_genre_name      : return media->data.at(TRACK::genre);break;

      //FIXME [TRACK::year] field is not used 
      case SearchQuery::field_track_year      : 
      case SearchQuery::field_album_year      : 
        return QVariant( QDate(media->parent()->data.at(ALBUM::year).toInt(), 1, 1) );
        break;

      case SearchQuery::field_artist_playcount: return media->parent()->parent()->data.at(ARTIST::playcount);break;
      case SearchQuery::field_album_playcount : return media->parent()->data.at(ALBUM::playcount);break;
      case SearchQuery::field_track_playcount : return media->data.at(TRACK::playcount);break;

      case SearchQuery::field_artist_rating   : return media->parent()->parent()->data.at(ARTIST::rating);break;
      case SearchQuery::field_album_rating    : return media->parent()->data.at(ALBUM::rating);break;
      case SearchQuery::field_track_rating    : return media->data.at(TRACK::rating);break;

      //! in this case lastPlayed is a QDate
      case SearchQuery::field_track_lastPlayed: return media->data.at(TRACK::lastPlayed); break;

      default : return QVariant();break;
    }
    return QVariant();
}




bool  SearchEngine::match(SearchQuery::Search_Operator op, SearchQuery::Field_Type type, QVariant data_field, QVariant pattern)
{
    switch (type) 
    {
      case SearchQuery::type_Text   : return match_string(op, data_field.toString(), pattern.toString());break;
      case SearchQuery::type_Number : return match_number(op, data_field.toInt(), pattern.toInt());break;      
      case SearchQuery::type_Time   : return match_time(op, data_field.toTime(), pattern.toTime());break;
      case SearchQuery::type_Date   : return match_date(op, data_field.toDate(), pattern.toDate());break;
      case SearchQuery::type_Rating : return match_rating(op, data_field.toFloat(), pattern.toFloat());break;
      default : return false;
    }
}



bool SearchEngine::match_string (SearchQuery::Search_Operator op, QString field_string, QString pattern)
{
    switch (op) {
      case SearchQuery::op_StartsWith:
        return field_string.startsWith ( pattern, Qt::CaseInsensitive );
      case SearchQuery::op_EndsWith:
        return field_string.endsWith ( pattern, Qt::CaseInsensitive ); 
      case SearchQuery::op_Contains:
        return field_string.contains(pattern, Qt::CaseInsensitive);
      case SearchQuery::op_NotContains:
        return !field_string.contains(pattern, Qt::CaseInsensitive);
      case SearchQuery::op_Equals:
        return field_string == pattern;
      //! only use for sorting not matching
      case SearchQuery::op_GreaterThan: 
        return field_string > pattern;
      case SearchQuery::op_LessThan:  
        return field_string < pattern;

      default : return false;  
    }
  
    return false;
}

bool SearchEngine::match_number(SearchQuery::Search_Operator op, int field_number, int pattern) 
{
    switch (op) {
      case SearchQuery::op_GreaterThan  : return field_number > pattern;
      case SearchQuery::op_LessThan     : return field_number < pattern;
      case SearchQuery::op_Equals       : return field_number == pattern;
      case SearchQuery::op_NotEquals    : return field_number != pattern;
      default : return false;
    }
    return false;
}

bool SearchEngine::match_time(SearchQuery::Search_Operator op, QTime field_time, QTime pattern)
{
    switch (op) {
      case SearchQuery::op_GreaterThan:
        return QTime(0,0).secsTo(field_time) > QTime(0,0).secsTo(pattern);
      case SearchQuery::op_LessThan:
        return QTime(0,0).secsTo(field_time) < QTime(0,0).secsTo(pattern);
      case SearchQuery::op_Equals:
        return QTime(0,0).secsTo(field_time) == QTime(0,0).secsTo(pattern);
      case SearchQuery::op_NotEquals:
        return QTime(0,0).secsTo(field_time) != QTime(0,0).secsTo(pattern);
      default : return false;      
    }
    return false;
} 



bool SearchEngine::match_date(SearchQuery::Search_Operator op, QDate field_date, QDate pattern)
{
    switch (op) {
      case SearchQuery::op_GreaterThan : return field_date > pattern;  // after
      case SearchQuery::op_LessThan    : return field_date < pattern;  // before
      case SearchQuery::op_Equals      : return field_date == pattern; // date is on
      case SearchQuery::op_NotEquals   : return field_date != pattern; // date is not on    
      default : return false;      
    }
    return false;
} 

bool SearchEngine::match_rating(SearchQuery::Search_Operator op, float field_rating, float pattern)
{
    //Debug::debug() << "SearchEngine::match_rating field_rating " << field_rating << " pattern " << pattern;

    switch (op) {
      case SearchQuery::op_GreaterThan : return field_rating > pattern;
      case SearchQuery::op_LessThan    : return field_rating < pattern;
      case SearchQuery::op_Equals      : return field_rating == pattern;    
      case SearchQuery::op_NotEquals   : return field_rating != pattern;    
      default : return false;    
    }
    return false;
}
    
    
    
bool SearchEngine::sortMediaItem(const MediaItemPtr mi1,const MediaItemPtr mi2)
{
    SearchQuery::Search_Field field = search_.sort_field_;
    
    SearchQuery::Search_Operator ope;
    
    if(search_.sort_type_ == MediaSearch::Sort_FieldAsc) 
      ope = SearchQuery::op_LessThan;
    else
      ope = SearchQuery::op_GreaterThan;
      
    return match(ope, SearchQuery::TypeOf(field), fieldData(field, mi1), fieldData(field, mi2));
}
