/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Playlist/playlistview.h" 
#include "Playlist/playlistmodel.h" 
#include "MediaItem/mediaitem.h" 

#include "utilities.h"       // 0.0.48
#include "yarocksettings.h"  // 0.0.53
#include "global_actions.h" 
#include "debug.h"


#include <QFileInfo>
#include <QString>




bool SortByRowUptoDown(const QModelIndex& a, const QModelIndex& b) {
  return a.row() < b.row();
}

bool SortByRowDowntoUp(const QModelIndex& a, const QModelIndex& b) {
  return b.row() < a.row();
}


/*
********************************************************************************
*                                                                              *
*    Class PlaylistView                                                        *
*                                                                              *
********************************************************************************
*/
PlaylistView::PlaylistView(QWidget *parent, PlaylistModel* model) : QListView(parent) 
{
    Debug::debug() << "--- PlaylistView creation";
  
    //! Gui
    this->setFrameShape(QFrame::NoFrame);
  
    QPalette palette;
    palette.setColor(QPalette::Background, Qt::white);
    palette.setColor(QPalette::Window , Qt::white);
    
    
    //palette.setColor(QPalette::Foreground, Qt::white);
    
    palette.setColor(QPalette::Base, Qt::white);
    //palette.setColor(QPalette::AlternateBase, Qt::white);
    //palette.setColor(QPalette::Button, QColor(240,240,240));
  
    //palette.setColor(QPalette::Text, QColor(0x315996));
    //palette.setColor(QPalette::WindowText, QColor(0x315996));
    //palette.setColor(QPalette::HighlightedText, Qt::white);
  
    this->setPalette(palette);
    this->setAutoFillBackground(false);
  
    //! widget
    this->setEnabled(true);
    this->setAlternatingRowColors(false);
    this->setEditTriggers(QAbstractItemView::NoEditTriggers);
    this->setSelectionMode(QAbstractItemView::ExtendedSelection); 
    this->setUniformItemSizes(true); //!optim
    this->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
  
    //! dragndrop
    this->setDragEnabled(true);
    this->setAcceptDrops(true);
    this->setDropIndicatorShown(true);
    this->setDragDropMode(QAbstractItemView::DragDrop);  
    this->setMouseTracking(true);
  
    //! Delegate
    _playlistDelegate = new PlaylistDelegate(this);
    this->setItemDelegate(_playlistDelegate);
    
    //! vertical scrolbar setup
    this->verticalScrollBar()->setContextMenuPolicy(Qt::NoContextMenu);
    
    //! connection
    connect(ACTIONS()->value(LASTFM_LOVE), SIGNAL(triggered()), this, SLOT(sendLastFmLove()));
    connect(ACTIONS()->value(PLAYQUEUE_REMOVE_ITEM), SIGNAL(triggered()), this, SLOT(removeSelected()));
    
    connect(this, SIGNAL(activated(const QModelIndex &)), SLOT(slot_itemActivated(const QModelIndex &)));
       
    //! model
    _playlistModel = model;
    this->setModel(_playlistModel);
    connect(_playlistModel, SIGNAL(needSelectionAfterMove(QList<MediaItemPtr>)), SLOT(selectItems(QList<MediaItemPtr>)));
    
    drop_indicator_row_ = -1;
    drag_over_          = false;
}


//!---------- PlaylistView::sendLastFmLove -------------------------------------
void PlaylistView::sendLastFmLove()
{
    Debug::debug() << "--- PlaylistView-->sendLastFmLove";
    if(_selection.isEmpty()) 
      return;
    
    foreach (const QModelIndex& selected_model_index, _selection) 
    {
        if (_playlistModel->rowExists(selected_model_index.row())) 
        {
          MediaItemPtr mi = _playlistModel->MediaItemAt( selected_model_index.row() );
          if(mi && mi->type == TYPE_TRACK)
            emit signal_lastFmLove(mi->data.at(TRACK::artist).toString(), mi->data.at(TRACK::title).toString() );
        }      
    }
}

//!---------- PlaylistView::selectionChanged -----------------------------------
void PlaylistView::selectionChanged(const QItemSelection &selected, const QItemSelection &deselected) 
{
    _selection     = this->selectionModel()->selectedRows();

    QListView::selectionChanged(selected, deselected);
    
    emit selectionChanged();
}


//!---------- PlaylistView::selectItems ----------------------------------------
void PlaylistView::selectItems(QList<MediaItemPtr> list)
{
    this->selectionModel()->clear();
  
    foreach(MediaItemPtr mi, list) {
       int row = _playlistModel->rowForMediaItem(mi);
       this->selectionModel()->select(_playlistModel->index(row,0), QItemSelectionModel::Select);
    }  
}




//!---------- PlaylistView::getSelectedMediaItem -------------------------------
const MediaItemPtr PlaylistView::getSelectedMediaItem()
{
    // use when play/pause action from mainwindow
    // we get first the selected item and play it (so we have to activate it)
    
    if(!_selection.isEmpty()) {
      qSort(_selection.begin(), _selection.end(), SortByRowUptoDown);
      
      QModelIndex selected_model_index = _selection.first();      
      if (_playlistModel->rowExists(selected_model_index.row())) 
      {
        _playlistModel->setRequestedMediaItem(selected_model_index.row());

        return  _playlistModel->MediaItemAt(selected_model_index.row());
      }
    }
   
  return MediaItemPtr(); 
}

//!---------- PlaylistView::removeSelected -------------------------------------
void PlaylistView::removeSelected() 
{
    if(_selection.isEmpty()) 
      return;
    
    qSort(_selection.begin(), _selection.end(), SortByRowDowntoUp);
    foreach (const QModelIndex& selected_model_index, _selection)
    {
        if (_playlistModel->rowExists(selected_model_index.row()))
           _playlistModel->removeRows(selected_model_index.row(), 1, QModelIndex());
    }
    
    this->selectionModel()->clear();   
}



//!---------- PlaylistView::slot_itemActivated --------------------------------
// gestion du doucle click sur un item
void PlaylistView::slot_itemActivated(const QModelIndex &index)
{
    //Debug::debug() << "--- PlaylistView::slot_itemActivated" << index.row();

    if (_playlistModel->rowExists(index.row()))
    {
      _playlistModel->setRequestedMediaItem(index.row());

      emit signal_playlist_itemDoubleClicked();
    }
}

//!---------- PlaylistView::clearPlaylist --------------------------------------
void PlaylistView::clearPlaylist()
{
    this->selectionModel()->clear();
    _playlistModel->clear();
}

//!---------- PlaylistView::getNextMediaItem -----------------------------------
const MediaItemPtr PlaylistView::getNextMediaItem()
{
    int activerow = _playlistModel->getPlayingRow();
  
    RepeatMode  repeat_m  = (RepeatMode)SETTINGS()->_repeatMode;
    ShuffleMode shuffle_m = (ShuffleMode)SETTINGS()->_shuffleMode;
  
    //! normal mode get next track
    if(repeat_m == RepeatOff && shuffle_m == ShuffleOff) {
      if( (activerow+1) < _playlistModel->rowCount(QModelIndex())) {
        _playlistModel->skipForward();
      
        return _playlistModel->getRequestedMediaItem();
      }
    }
  
    //! repeat track mode
    if(repeat_m == RepeatTrack) {
      if( activerow < _playlistModel->rowCount(QModelIndex())) {
        return _playlistModel->getRequestedMediaItem();
      }
    }
  
    //! repeat all mode
    if(repeat_m == RepeatAll && shuffle_m == ShuffleOff) {
      if( activerow == _playlistModel->rowCount(QModelIndex())-1) {
        _playlistModel->setRequestedMediaItem(0);
      
        return _playlistModel->getRequestedMediaItem();
      }
      else if( (activerow+1) < _playlistModel->rowCount(QModelIndex())) {
        _playlistModel->skipForward();
        return _playlistModel->getRequestedMediaItem();
      }
    }
  
    //! shuffle ==> random next until all is played
    if(shuffle_m == ShuffleOn) {
      QList<MediaItemPtr> list = _playlistModel->unplayedMediaItem();
      if(list.isEmpty()) 
        return MediaItemPtr();
    
      int index = UTIL::randomInt(0, list.size() -1);
      MediaItemPtr mi = list.at(index);
      _playlistModel->setRequestedMediaItem(_playlistModel->rowForMediaItem(mi));
    
      return _playlistModel->getRequestedMediaItem();
    }
  
    return MediaItemPtr();
}


//!---------- PlaylistView::getPrevMediaItem -----------------------------------
const MediaItemPtr PlaylistView::getPrevMediaItem()
{
    int activerow = _playlistModel->getPlayingRow();
  
    RepeatMode  repeat_m  = (RepeatMode)SETTINGS()->_repeatMode;
    ShuffleMode shuffle_m = (ShuffleMode)SETTINGS()->_shuffleMode;
  
    //! normal mode get prev track
    if(repeat_m == RepeatOff && shuffle_m == ShuffleOff) {
      if( activerow > 0) {
        _playlistModel->skipBackward();
        return _playlistModel->getRequestedMediaItem();
      }
    }
      
    //! repeat track mode
    if(repeat_m == RepeatTrack) {
      if( activerow < _playlistModel->rowCount(QModelIndex())) {
      return _playlistModel->getRequestedMediaItem();
      }
    }
  
    //! repeat all mode
    if(repeat_m == RepeatAll && shuffle_m == ShuffleOff) {
      if( activerow == 0) {
        _playlistModel->setRequestedMediaItem(_playlistModel->rowCount(QModelIndex())-1);
      
        return _playlistModel->getRequestedMediaItem();
      }
      else if( activerow > 0) {
        _playlistModel->skipBackward();
        return _playlistModel->getRequestedMediaItem();
      }
    }

    //! shuffle ==> random prev until all is played
    if(shuffle_m == ShuffleOn) {
      QList<MediaItemPtr> list = _playlistModel->unplayedMediaItem();
      if(list.isEmpty()) 
        return MediaItemPtr();
    
      int index = UTIL::randomInt(0, list.size() -1);
      MediaItemPtr mi = list.at(index);
      _playlistModel->setRequestedMediaItem(_playlistModel->rowForMediaItem(mi));
    
      return _playlistModel->getRequestedMediaItem();
    }
  
    return MediaItemPtr();
}

//!---------- PlaylistView::getActiveMediaItem ---------------------------------
const MediaItemPtr PlaylistView::getActiveMediaItem() 
{
    return _playlistModel->getRequestedMediaItem(); 
}


//!---------- PlaylistView::getMediaItemAt -------------------------------------
const MediaItemPtr PlaylistView::getMediaItemAt(int i)
{
    if (_playlistModel->rowExists(i)) {
      return _playlistModel->MediaItemAt(i);
    }
   
    return MediaItemPtr();    
}
    



//!---------- PlaylistView::stopPlaying ----------------------------------------
void PlaylistView::stopPlaying() 
{
    //stop all item activation from the model
    _playlistModel->stopPlayingMediaItem();
}




//!---------- PlaylistView::paintEvent -----------------------------------------
void PlaylistView::paintEvent(QPaintEvent *event)
{ 
    QPainter painter(viewport());
    if(_playlistModel->rowCount() < 1) 
    {
      const QString message = tr("Add file to play or Drag and Drop File from Collection");
      painter.setFont(QFont("Segoe UI", 10, QFont::Normal));
      
      const QRect textBox = rect().adjusted(10, 0, -10, 0);
      
      painter.setPen(QColor(0x555555));    
      
      painter.drawText(textBox.adjusted(0, 70, 0, 0), Qt::AlignHCenter | Qt::TextWordWrap, message);
      
      const QPixmap pix = QPixmap(":/images/info-message.png") ;
      painter.drawPixmap(textBox.width()/2 - pix.width()/2, 10 ,pix);
    }

    if (drop_indicator_row_ != -1)
    {
      // Find the y position of the drop indicator
      QModelIndex drop_index = _playlistModel->index(drop_indicator_row_, 0);
      
      int drop_pos = -1;
      switch (this->dropIndicatorPosition()) {
        case QAbstractItemView::OnItem: break;
      
        case QAbstractItemView::AboveItem: drop_pos = visualRect(drop_index).top(); break;
      
        case QAbstractItemView::BelowItem: drop_pos = visualRect(drop_index).bottom() + 1;break;

        case QAbstractItemView::OnViewport:
          if (_playlistModel->rowCount() == 0)
            drop_pos = 1;
          else
            drop_pos = visualRect(_playlistModel->index(_playlistModel->rowCount() - 1, 0)).bottom() + 1;
        break;
      }

      // Draw a nice gradient first
      QColor line_color(QApplication::palette().color(QPalette::Highlight));
      QColor shadow_color(line_color.lighter(140));
      QColor shadow_fadeout_color(shadow_color);
      shadow_color.setAlpha(255);
      shadow_fadeout_color.setAlpha(0);

      QLinearGradient gradient(QPoint(0, drop_pos - 5),QPoint(0, drop_pos + 5));
      gradient.setColorAt(0.0, shadow_fadeout_color);
      gradient.setColorAt(0.5, shadow_color);
      gradient.setColorAt(1.0, shadow_fadeout_color);
      QPen gradient_pen(QBrush(gradient), 5 * 2);
      painter.setPen(gradient_pen);
      painter.drawLine(QPoint(0, drop_pos),QPoint(width(), drop_pos));
        
      // Now draw the line on top
      QPen line_pen(line_color, 2);
      painter.setPen(line_pen);
      painter.drawLine(QPoint(0, drop_pos),QPoint(width(), drop_pos));
    } // end if drop_indicator_row_ != -1
  
    QListView::paintEvent(event);
}  


void PlaylistView::dragMoveEvent(QDragMoveEvent *event)
{
  
    QListView::dragMoveEvent(event);
    QModelIndex index(indexAt(event->pos()));
    drop_indicator_row_ = index.isValid() ? index.row() : 0;
}

void PlaylistView::dragEnterEvent(QDragEnterEvent *event)
{
    QListView::dragEnterEvent(event);
    drag_over_ = true;
}

void PlaylistView::dragLeaveEvent(QDragLeaveEvent *event)
{
    QListView::dragLeaveEvent(event);
    drag_over_ = false;
    drop_indicator_row_ = -1;
}

void PlaylistView::dropEvent(QDropEvent *event) 
{
    QListView::dropEvent(event);
    drop_indicator_row_ = -1;
    drag_over_ = false;
}

void PlaylistView::mouseMoveEvent(QMouseEvent* event)
{
  if (!drag_over_)
    QListView::mouseMoveEvent(event);
}

void PlaylistView::keyPressEvent(QKeyEvent* event) 
{
    if (event->matches(QKeySequence::Delete)) 
    {
      removeSelected();
      event->accept();
    } 
    else if (event->key() == Qt::Key_Enter || event->key() == Qt::Key_Return) 
    {
      QModelIndex selected_model_index; 
      if(!_selection.isEmpty()){
        qSort(_selection.begin(), _selection.end(), SortByRowUptoDown);
        selected_model_index = _selection.first();
        if (_playlistModel->rowExists(selected_model_index.row()))
          slot_itemActivated(selected_model_index);
      }      
      
      event->accept();
    }
    else 
    {
    event->ignore();
    }
}


/*
********************************************************************************
*                                                                              *
*    Class PlaylistDelegate                                                    *
*                                                                              *
********************************************************************************
*/
PlaylistDelegate::PlaylistDelegate(QObject *parent) : QItemDelegate(parent)
{
    icon_media_playing  = QIcon(":/images/media-playing.png");
    icon_media_broken   = QIcon(":/images/media-broken-18x18.png");
    icon_media_track    = QIcon(":/images/track-48x48.png");
    icon_media_stream   = QIcon(":/images/media-url-48x48.png");
    
    font_normal = QApplication::font();
    font_normal.setBold(false);
    font_bold   = QApplication::font();
    font_bold.setBold(true);
    
    _mode = PLAYQUEUE::MODE_EXTENDED;    
}



void PlaylistDelegate::paint ( QPainter * painter, const QStyleOptionViewItem & option, const QModelIndex & index ) const
{
    //! get the data object
    PlaylistModel * model = (PlaylistModel *)index.model();
    MediaItemPtr    mi    = model->MediaItemAt(index.row());

    bool isTrack     = (mi->type == TYPE_TRACK) ? true : false;
    bool isBroken    = isTrack ? mi->data.at(TRACK::isBroken).toBool() :  mi->data.at(STREAM::isBroken).toBool();
    bool isPlaying   = isTrack ? mi->data.at(TRACK::isPlaying).toBool() : mi->data.at(STREAM::isPlaying).toBool();

    const int left   = option.rect.left();
    const int top    = option.rect.top();
    const int width  = option.rect.width();
    const int height = _mode == PLAYQUEUE::MODE_EXTENDED ? 40 : 20;
    
    //! Create base pixmap
    QPixmap pixmap(width, height);
    {
      pixmap.fill(Qt::transparent);
      QPainter p(&pixmap);
      p.translate(-option.rect.topLeft());

      //! draw background
      QStyleOptionViewItemV4 opt(option);
      opt.state |= QStyle::State_Active;
      opt.state |= QStyle::State_HasFocus;
            
      QStyle *style = opt.widget ? opt.widget->style() : QApplication::style();
      style->drawPrimitive(QStyle::PE_PanelItemViewItem, &opt, &p, opt.widget);
      
      //! set painter font & color
      p.setFont(font_normal);
      if(option.state & QStyle::State_Selected)
        p.setPen( QColor("white"));
      else if (isPlaying)
        p.setPen( QColor(0x940606));
      else
        p.setPen( QColor(0x2e2b2b));

      QFontMetrics fm( font_normal );    

      /*---------------------------------------------*/
      /* TITLE/ALBUM/ARTIST view                     */
      /* --------------------------------------------*/
      if(_mode != PLAYQUEUE::MODE_EXTENDED) 
      {
          //! Paint Icon
          QIcon icon = this->getIcon(mi);
          icon.paint(&p, left + 3, top + 2, 16, 16, Qt::AlignCenter, QIcon::Normal);

          //! Paint duration
          QRect rectDuree = QRect();
          if(isTrack && !isBroken)
          {
            QString durationText = MEDIA::durationTextFromInt(mi->data.at(TRACK::duration).toInt());
            p.drawText(left + width - 50,top, 49, height,Qt::AlignVCenter | Qt::AlignRight, durationText);
            rectDuree = fm.boundingRect ( durationText );
          }

          //! Paint track info
          QString trackinfo = this->getTrackInfo(mi);
          trackinfo = fm.elidedText ( trackinfo, Qt::ElideRight, width-51 );
          p.drawText(left + 25,top,width-rectDuree.width()-5, 20,Qt::AlignVCenter | Qt::AlignLeft, trackinfo);
      }
      /*---------------------------------------------*/
      /* EXTENDED view                               */
      /* --------------------------------------------*/
      else 
      {
          //! Paint Icon
          QIcon icon = this->getIcon(mi);
          icon.paint(&p, left + 3, top+(height-16)/2, 16, 16, Qt::AlignVCenter | Qt::AlignCenter, QIcon::Normal);

          // Prepare rectangle for painting
          QRect rectDuree = QRect();

          if(!isBroken) 
          {
            if(isTrack) 
            {
              //! Paint duration
              QString durationText = MEDIA::durationTextFromInt(mi->data.at(TRACK::duration).toInt());
              p.drawText(left + width - 50,top, 49, height,Qt::AlignVCenter | Qt::AlignRight, durationText);
              rectDuree = fm.boundingRect ( durationText );
            }

            //! Paint track info
            QString info_2  = isTrack ? 
            mi->data.at(TRACK::artist).toString() + " - " + mi->data.at(TRACK::album).toString() : mi->data.at(STREAM::url).toString();
            info_2 = fm.elidedText ( info_2, Qt::ElideRight, width-51 );
            p.drawText(left + 25,option.rect.bottom()-fm.height()-7,width-rectDuree.width()-5, 20,Qt::AlignVCenter | Qt::AlignLeft, info_2);

            p.setFont(font_bold);
            fm = QFontMetrics(font_bold);

            QString track_title  = isTrack ? mi->data.at(TRACK::title).toString() : mi->data.at(STREAM::name).toString();
            track_title = fm.elidedText ( track_title, Qt::ElideRight, width-51 );
            p.drawText(left + 25,top+3,width-rectDuree.width()-5, 20,Qt::AlignVCenter | Qt::AlignLeft, track_title);
          }
          else
          {
            //! Paint track info
            QString track_info = isTrack ? mi->data.at(TRACK::url).toString() :  mi->data.at(STREAM::url).toString();
            track_info = fm.elidedText ( track_info, Qt::ElideRight, width-51 );
            p.drawText(left + 25,option.rect.bottom()-fm.height()-7,width-rectDuree.width()-5, 20,Qt::AlignVCenter | Qt::AlignLeft, track_info);

            p.setFont(font_bold);
            fm = QFontMetrics(font_bold);

            QString info_1  = tr("media reading error");
            info_1 = fm.elidedText ( info_1, Qt::ElideRight, width-51 );
            p.drawText(left + 25,top+3,width-rectDuree.width()-5, 20,Qt::AlignVCenter | Qt::AlignLeft, info_1);
          }
      }
      
      //! draw flag for "stop after" action
      if(mi->data.at(TRACK::isStopAfter).toBool()) {
        int y = _mode == PLAYQUEUE::MODE_EXTENDED ? 12 : 2;
        drawStop(&p, QRect(left + width - 41,top+y, 40, 16) );
      }
      
      p.end();
    }
    //Draw finished pixmap
    painter->drawPixmap(option.rect.topLeft(), pixmap);
}

    
QSize PlaylistDelegate::sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const
{
Q_UNUSED(index)
  if(_mode == PLAYQUEUE::MODE_EXTENDED)
    return QSize(option.rect.width(), 40);
  else 
    return QSize(option.rect.width(), 20);
}


QString PlaylistDelegate::getTrackInfo(const MediaItemPtr mi) const
{
  bool isTrack  = (mi->type == TYPE_TRACK) ? true : false;
  bool isBroken = isTrack ? mi->data.at(TRACK::isBroken).toBool() :  mi->data.at(STREAM::isBroken).toBool();
  QString s;
  
  if(isBroken)
  {
    s = isTrack ? mi->data.at(TRACK::url).toString() :  mi->data.at(STREAM::url).toString();
  }
  else 
  {
    if(isTrack) 
    {
        switch( _mode ) {
          case PLAYQUEUE::MODE_TITLE  : 
             s = mi->data.at(TRACK::title).toString();break;
          case PLAYQUEUE::MODE_ALBUM  : 
             s = mi->data.at(TRACK::album).toString() + " - " + mi->data.at(TRACK::title).toString();break;
          case PLAYQUEUE::MODE_ARTIST : 
             s = mi->data.at(TRACK::artist).toString() + " - " + mi->data.at(TRACK::album).toString() + " - " + mi->data.at(TRACK::title).toString();break;
          default : 
             s = mi->data.at(TRACK::title).toString();break;
        }
    } 
    else 
    {
      s = mi->data.at(STREAM::name).toString();
    }
  }
  
  return s;
}


QIcon PlaylistDelegate::getIcon(const MediaItemPtr mi) const
{ 
    bool isTrack     = (mi->type == TYPE_TRACK) ? true : false;
    bool isPlaying   = isTrack ? mi->data.at(TRACK::isPlaying).toBool() : mi->data.at(STREAM::isPlaying).toBool();
    bool isBroken    = isTrack ? mi->data.at(TRACK::isBroken).toBool()  : mi->data.at(STREAM::isBroken).toBool();

    if(isPlaying) 
      return icon_media_playing;
    else if (isBroken)
      return icon_media_broken;
    else if (isTrack)
      return icon_media_track;
    else 
      return icon_media_stream;
}

void PlaylistDelegate::drawStop(QPainter * painter, QRect rect) const
{
    painter->setFont(QFont("Segoe UI", 7, QFont::Bold));
  
    QLinearGradient gradient(rect.topLeft(), rect.bottomLeft());
    gradient.setColorAt(0.0,  qRgb(102, 150, 227) );
    gradient.setColorAt(1.0, qRgb(77, 121, 200));
  
    painter->setRenderHint(QPainter::Antialiasing);
  
    // Draw the box
    painter->setPen(QPen(Qt::white, 1));
    painter->setBrush(gradient);
    painter->drawRoundedRect(rect, 3, 3);

    // Draw the text
    painter->drawText(rect, Qt::AlignCenter, "stop");
}





