/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Playlist/playlistwidget.h" 
#include "Widget/statuswidget.h" 
#include "yarocksettings.h"

#include "global_actions.h"
//#include "debug.h"


PlaylistWidget* PlaylistWidget::INSTANCE = 0;

/*
********************************************************************************
*                                                                              *
*    Class PlaylistWidget                                                      *
*                                                                              *
********************************************************************************
*/
PlaylistWidget::PlaylistWidget(QWidget *parent,PlaylistView *view) : QWidget(parent)
{
    //! init
    INSTANCE          = this;
    _menu             = 0;
    _playlistview     = view;
    _playlistmodel    = PlaylistModel::instance();
    
    //! create Gui
    QVBoxLayout *vl = new QVBoxLayout(this);
    vl->setSpacing(0);
    vl->setContentsMargins(0, 0, 0, 0);
    vl->addWidget(_playlistview);
  
    //! connect
    connect(_playlistmodel, SIGNAL(layoutChanged()), this, SLOT(updatePlaylistWidget()));
    connect(_playlistview, SIGNAL(selectionChanged()), this, SLOT(updatePlaylistActions()));
    
    //! restore last mode for display
    _playlistview->setMode(SETTINGS()->_playqueueMode);   
    
    
    updatePlaylistActions();
}

    
void PlaylistWidget::updatePlaylistWidget() 
{
    updatePlaylistSummary();
    updatePlaylistActions();
}




//! ---- List Summary ----------------------------------------------------------
void PlaylistWidget::updatePlaylistSummary()
{
    const int rowCount = _playlistmodel->rowCount(QModelIndex());
    QString duree      = _playlistmodel->getListDuration(); 

    QString playlistInfo;
  
    if (rowCount < 1)
      playlistInfo = tr("Empty Playlist");
    else 
      playlistInfo = QString(tr("Playlist : <b>%1</b> tracks - %2")).arg(QString::number(rowCount), duree);

    StatusWidget::instance()->startShortMessage(playlistInfo, STATUS::TYPE_INFO, 2500);    
}



void PlaylistWidget::updatePlaylistActions()
{
    //Debug::debug() << "--- PlaylistWidget-->updatePlaylistActions";
    const int rowCount  = _playlistmodel->rowCount(QModelIndex());
    const bool isPlaylistEmpty = rowCount < 1;
    
    ACTIONS()->value(PLAYQUEUE_CLEAR)->setEnabled(!isPlaylistEmpty);
    ACTIONS()->value(PLAYQUEUE_SAVE)->setEnabled(!isPlaylistEmpty);
    ACTIONS()->value(PLAYQUEUE_AUTOSAVE)->setEnabled(!isPlaylistEmpty);
    ACTIONS()->value(PLAYQUEUE_REMOVE_ITEM)->setEnabled((!isPlaylistEmpty) && _playlistview->isMediaItemSelected());
}


void PlaylistWidget::contextMenuEvent(QContextMenuEvent* e) 
{
    //Debug::debug() << "--- PlaylistWidget-->contextMenuEvent";  
    QMap<ENUM_ACTION, QAction*> *actions = ACTIONS();
    //! handle stop after track action 
    const int right_click_row = _playlistview->indexAt(e->pos()).row();
    
    if( right_click_row != -1) {
      actions->value(PLAYQUEUE_STOP_AFTER)->setEnabled(true);    

      MediaItemPtr right_click_media = _playlistmodel->MediaItemAt(right_click_row);
            
      QVariant v;
      v.setValue(static_cast<MediaItemPtr>(right_click_media));
      (actions->value(PLAYQUEUE_STOP_AFTER))->setData(v); 

      MediaItemPtr current_stop_after_media = _playlistmodel->get_stop_after_media();

      actions->value(PLAYQUEUE_STOP_AFTER)->setChecked( current_stop_after_media == right_click_media );
    }
    else {
      actions->value(PLAYQUEUE_STOP_AFTER)->setEnabled(false);   
      actions->value(PLAYQUEUE_STOP_AFTER)->setChecked(false);
    }
    
    //! build menu 
    if (!_menu) {
      QMap<ENUM_ACTION, QAction*> *actions = ACTIONS();
      _menu = new QMenu(this);

      _menu->addAction(actions->value(PLAYQUEUE_ADD_FILE));
      _menu->addAction(actions->value(PLAYQUEUE_ADD_DIR));
      _menu->addAction(actions->value(PLAYQUEUE_ADD_URL));    
      _menu->addAction(actions->value(PLAYQUEUE_CLEAR));
      _menu->addAction(actions->value(PLAYQUEUE_SAVE));
      _menu->addAction(actions->value(PLAYQUEUE_AUTOSAVE));
      _menu->addAction(actions->value(PLAYQUEUE_REMOVE_ITEM));
      _menu->addAction(actions->value(PLAYQUEUE_STOP_AFTER));    
    
      _menu->addSeparator();
    
      QMenu *m = _menu->addMenu(tr("Choose playlist mode"));
    
      QSignalMapper* mapper = new QSignalMapper(this);
      QActionGroup* group = new QActionGroup(this);
      addMenuItem(mapper, group, actions->value(PLAYQUEUE_MODE_TITLE),  PLAYQUEUE::MODE_TITLE);
      addMenuItem(mapper, group, actions->value(PLAYQUEUE_MODE_ALBUM),  PLAYQUEUE::MODE_ALBUM);
      addMenuItem(mapper, group, actions->value(PLAYQUEUE_MODE_ARTIST), PLAYQUEUE::MODE_ARTIST);
      addMenuItem(mapper, group, actions->value(PLAYQUEUE_MODE_EXTENDED), PLAYQUEUE::MODE_EXTENDED);

      connect(mapper, SIGNAL(mapped(int)), SLOT(playQueueModeChange(int)));
      m->addActions(group->actions());
    
      _menu->addSeparator();
      _menu->addAction(actions->value(LASTFM_LOVE));
      _menu->addSeparator();
      _menu->addAction(actions->value(APP_SHOW_PLAYQUEUE));
    }

    // show the menu
    _menu->popup(e->globalPos());
    e->accept();
}



void PlaylistWidget::addMenuItem(QSignalMapper* mapper, QActionGroup* group,
                                 QAction* action, PLAYQUEUE::Q_MODE mode) 
{
    group->addAction(action);
    mapper->setMapping(action, mode);
    connect(action, SIGNAL(triggered()), mapper, SLOT(map()));
    connect(action, SIGNAL(triggered()), _playlistmodel, SLOT(needUpdate()));

    // restore action state with current mode
    if (mode == SETTINGS()->_playqueueMode)
        action->setChecked(true);
}

/*
playQueueModeChange(int mode) 
public slots (ré-utiliser par le menu du centraltoolbar) 

*/

void PlaylistWidget::playQueueModeChange(int mode) 
{
    // il faut forcer le repaint de la playlistview
    _playlistview->setMode(mode);
    _playlistview->update();
}
