/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _SCROBBLER_LASTFM_H_
#define _SCROBBLER_LASTFM_H_

#include <QObject>
#include <QNetworkReply>
#include <QTimer>

/* 
 * Base on the lastFm API version 2.0. implementation done by Dan Vratil for 
 * TepSonic player with adaptation done for Yarock 
 * Copyright 2010 Dan Vratil <vratil@progdansoft.com>
*/
 
 
namespace LastFm {

// Forward declarations
class Track;
class Cache;
class Scrobbler;

// Global configuration 
namespace Global
{
  extern QString api_key;
  extern QString secret_key;
  extern QString username;
  extern QString auth_token;
  extern QString session_key;
};


/*
********************************************************************************
*                                                                              *
*    Class Scrobbler                                                           *
*                                                                              *
********************************************************************************
*/
class Scrobbler: public QObject
{
Q_OBJECT
  public:
    Scrobbler();
    ~Scrobbler();
 
    // Last.fm specific stuff
    bool isAuthenticated() const;
    void signIn(const QString& username, const QString& password);
    void signOut();
    QString username() {return LastFm::Global::username;}
    
    
    // Submits the cache scrobbling all it's content.
    // This method should be used instead of LastFm::Track::scrobble()
    void scrobble();
    
    // Add given track to cache and scrobble everything what's in cache
    void scrobble(LastFm::Track *track);

    // Emits error() signal
    void raiseError(int code);

    // Returns pointer to scrobbler's cache 
    LastFm::Cache* cache() { return m_cache; }

    // Scrobble informations about now playing track
    void nowPlaying(LastFm::Track *track);

    // Returns pointer to currently playing track
    LastFm::Track* currentTrack() { return m_currentTrack; }

    // Set new current track
    void setCurrentTrack(LastFm::Track* track) { m_currentTrack = track; }
            
  private slots:
    void slotSignInFinished(QNetworkReply*);
	    
  private:
    LastFm::Cache    *m_cache;
    LastFm::Track    *m_currentTrack;
            
  signals:
    void signInFinished();
    void error(QString message, int code);
};


/*
********************************************************************************
*                                                                              *
*    Class Track                                                               *
*                                                                              *
********************************************************************************
*/
/*
  This class provides API for submitting, updating Now Playing and loving tracks.
*/
class Track: public QObject
{
Q_OBJECT
  public:
    //  Constructs a new Track. Please note, that it is neccessary to pass 
    // pointer to a valid LastFm::Scrobbler object
    Track(LastFm::Scrobbler *scobbler,
          QString artist = QString(),
          QString trackTitle = QString(),
          QString album = QString(),
          int trackLength = 0,
          QString genre = QString(),
          int trackNumber = 0,
          uint playbackStart = 0);

    // setter 
    void setArtist(QString artist) { m_artist = artist; }
    void setTrackTitle(QString trackTitle) { m_trackTitle = trackTitle; }
    void setAlbum(QString album) { m_album = album; }
    void setTrackLength(int trackLength) { m_trackLength = trackLength; }
    void setGenre(QString genre) { m_genre = genre; }
    void setTrackNumber(int trackNumber) { m_trackNumber = trackNumber; }
    void setPlaybackStart(uint playbackStart) { m_playbackStart = playbackStart; }
            
    // accessor 
    QString artist() { return m_artist; }
    QString trackTitle() { return m_trackTitle; }
    QString album() { return m_album; }
    int trackLength() { return m_trackLength; }
    QString genre() { return m_genre; }
    int trackNumber() { return m_trackNumber; }
    uint playbackStart() { return m_playbackStart; }

  public slots:

    // Scrobble the track to Last.fm. When scrobbling fails, the item is held in cache
    // and is submitted after some time.
    // For internal use only. To scrobble the track to Last.fm, use LastFm::Scrobbler::scrobble()
    // method, which will submit whole cache.
    void scrobble();

    // Update "Now playing" status. The track is also stored in cache, so when the playback is
    // finished, submitting cache will update it to "recently played".
    void nowPlaying();

    // Set the track as loved.
    // When loving a track, all the informations above must be set.
    // The method works simillary to scrobble() or nowPlaying() methods and
    // it is not necessarry to call any of them to submit the love.
    void love();

    // Pause (or unpause) the track
    // This is used to count real time for which the track is playing
    void pause(bool pause);

  signals:
    // This signal is emitted when the track is scrobbled 
    void scrobbled();
    
    // This signal is emitted when the track is loved 
    void loved();

  private:
    LastFm::Scrobbler *m_scrobbler;

    // Track data 
    QString        m_artist;
    QString        m_trackTitle;
    QString        m_album;
    int            m_trackLength;
    QString        m_genre;
    int            m_trackNumber;
    uint           m_playbackStart;

    // Helper time counter
    int            m_playbackLength;
    uint           m_unpauseTime;

  private slots:
    void scrobbled(QNetworkReply*);
};

/*
********************************************************************************
*                                                                              *
*    Class Cache                                                               *
*                                                                              *
********************************************************************************
*/
/*
Cache provides an internal mechanism for saving unscrobbled items.
      * The cache is saved and loaded automatically, so you don't need to take
      * care of anything.
      * Do not use the cache directly as it is not neccessary and will probably not
      * work for you.    
*/
class Cache: public QObject
{
Q_OBJECT
  public:
    Cache(LastFm::Scrobbler *scrobbler);
    ~Cache();

  public slots:
    // Add a track to the cache
    void add(LastFm::Track *track);

    // Try to submit all tracks in cache
    void submit();

    // Called when track are submitted
    void submitted(QNetworkReply*);

  private:
    void loadCache();

    QList<LastFm::Track*> m_cache;
    LastFm::Scrobbler    *m_scrobbler;
    QTimer                *m_resubmitTimer;

    // Let's have access to track's private items
    friend class LastFm::Track;
  };

} // end lastfm namespace 

#endif // _SCROBBLER_LASTFM_H_
