/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "settings_widget.h"
#include "ContextView/widget/context_graphicitem.h" 

#include "utilities.h"
#include "yarocksettings.h"
#include "debug.h"

/*
********************************************************************************
*                                                                              *
*    Class PageGeneral                                                         *
*                                                                              *
********************************************************************************
*/
PageGeneral::PageGeneral(QWidget* parentView) : QGraphicsWidget(0)
{
    m_parent       = parentView;

    //! create Gui
    createGui();
}


//! ----------- createGui ------------------------------------------------------
void PageGeneral::createGui()
{
    m_title   = new TitleGraphicItem();
    m_title->m_name   = tr("General");
    m_title->m_width  = m_parent->width();
    
    // main widget
    QWidget* main_widget = new QWidget();
    
    QPalette palette;
    palette.setColor(QPalette::Background, Qt::white);
    palette.setColor(QPalette::Window , Qt::white);
    palette.setColor(QPalette::WindowText, Qt::black);
    palette.setColor(QPalette::Base, Qt::white);
    palette.setColor ( QPalette::Disabled, QPalette::WindowText, Qt::darkGray);
    palette.setColor ( QPalette::Disabled, QPalette::Base, Qt::darkGray);
    palette.setColor ( QPalette::Disabled, QPalette::Text, Qt::darkGray);    

    main_widget->setAttribute(Qt::WA_NoBackground, true);
    main_widget->setPalette(palette);
    main_widget->setAutoFillBackground(true);

    QVBoxLayout* vl0 = new QVBoxLayout(main_widget);
    /*-------------------------------------------------*/
    /* General settings                                */
    /* ------------------------------------------------*/      
    //! dbus & mpris parameter
    ui_check_enableDbus = new QCheckBox(main_widget);
    ui_check_enableDbus->setText(tr("Enable Now playing Notification"));

    ui_check_enableMpris = new QCheckBox(main_widget);
    ui_check_enableMpris->setText(tr("Enable Mpris interface"));

    //! systray parameter
    ui_check_systray = new QCheckBox(main_widget);
    ui_check_systray->setText(tr("Minimize application to systray"));
    
    ui_check_hideAtStartup = new QCheckBox(main_widget);
    ui_check_hideAtStartup->setText(tr("Hide window at startup"));

    /*-------------------------------------------------*/
    /* Graphical Interface settings                    */
    /* ------------------------------------------------*/    
    QLabel *lbl2 = new QLabel(tr("Graphical Interface"), main_widget);
    lbl2->setFont(QFont("Arial",10,QFont::Bold));    
    
    ui_check_enable_mouseover = new QCheckBox(main_widget);
    ui_check_enable_mouseover->setText(tr("Show menu on mouse over"));

    //! layout
    vl0->addWidget(ui_check_enableDbus);
    vl0->addWidget(ui_check_enableMpris);
    vl0->addWidget(ui_check_systray);
    vl0->addWidget(ui_check_hideAtStartup);
    vl0->addItem( new QSpacerItem(20, 30, QSizePolicy::Minimum, QSizePolicy::Expanding) );    
    vl0->addWidget(lbl2);
    vl0->addWidget(ui_check_enable_mouseover);
    
    // proxy widget
    proxy_widget = new QGraphicsProxyWidget( this );
    proxy_widget->setWidget( main_widget );
    proxy_widget->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
    

    m_title->setParentItem(this);
    m_title->setPos(5,0);
}

void PageGeneral::resizeEvent( QGraphicsSceneResizeEvent *event )
{
Q_UNUSED(event)  
    //Debug::debug() << "## PageGeneral::resizeEvent";
    doLayout();
}

void PageGeneral::doLayout()
{
    //Debug::debug() << "## PageGeneral::doLayout";
    m_title->m_width =  m_parent->width();
    int Xpos = m_parent->width() - 350 > 0 ? (m_parent->width() - 350)/2 : 50;
    proxy_widget->setPos(Xpos,50);
}

//! ----------- saveSettings ---------------------------------------------------
void PageGeneral::saveSettings()
{
    Debug::debug() << "PageGeneral::saveSettings";
    SETTINGS()->_useTrayIcon         = this->ui_check_systray->isChecked();
    SETTINGS()->_hideAtStartup       = this->ui_check_hideAtStartup->isChecked();
    SETTINGS()->_useDbusNotification = this->ui_check_enableDbus->isChecked();
    SETTINGS()->_useMpris            = this->ui_check_enableMpris->isChecked();
    SETTINGS()->_popup_on_mouseover  = this->ui_check_enable_mouseover->isChecked();
}

//! ----------- restoreSettings ------------------------------------------------
void PageGeneral::restoreSettings()
{
    this->ui_check_systray->setChecked( SETTINGS()->_useTrayIcon );
    this->ui_check_hideAtStartup->setChecked( SETTINGS()->_hideAtStartup );
    this->ui_check_enableDbus->setChecked( SETTINGS()->_useDbusNotification );
    this->ui_check_enableMpris->setChecked( SETTINGS()->_useMpris );
    this->ui_check_enable_mouseover->setChecked( SETTINGS()->_popup_on_mouseover );    
}


bool PageGeneral::isSystrayChanged()
{
    return SETTINGS()->_useTrayIcon != ui_check_systray->isChecked();
}

bool PageGeneral::isDbusChanged()
{
    return SETTINGS()->_useDbusNotification != ui_check_enableDbus->isChecked();
}

bool PageGeneral::isMprisChanged()
{
    return SETTINGS()->_useMpris != ui_check_enableMpris->isChecked();
}






/*
********************************************************************************
*                                                                              *
*    Class PagePlayer                                                          *
*                                                                              *
********************************************************************************
*/
PagePlayer::PagePlayer(QWidget* parentView) : QGraphicsWidget(0)
{
    m_parent       = parentView;
    
    //! create Gui
    createGui();
}

//! ----------- createGui ------------------------------------------------------
void PagePlayer::createGui()
{
    m_title           = new TitleGraphicItem();
    m_title->m_name   = tr("Player settings");
    m_title->m_width  = m_parent->width();

    // main widget
    QWidget* main_widget = new QWidget();
    
    QPalette palette;
    palette.setColor(QPalette::Background, Qt::white);
    palette.setColor(QPalette::Window , Qt::white);
    palette.setColor(QPalette::WindowText, Qt::black);
    palette.setColor(QPalette::Base, Qt::white);
    palette.setColor ( QPalette::Disabled, QPalette::WindowText, Qt::darkGray);
    palette.setColor ( QPalette::Disabled, QPalette::Base, Qt::darkGray);
    palette.setColor ( QPalette::Disabled, QPalette::Text, Qt::darkGray);

    main_widget->setAttribute(Qt::WA_NoBackground, true);
    main_widget->setPalette(palette);
    main_widget->setAutoFillBackground(true);

    
    /*-------------------------------------------------*/
    /* Player settings                                 */
    /* ------------------------------------------------*/ 
    ui_stopOnPlayqueueClear = new QCheckBox(main_widget);
    ui_stopOnPlayqueueClear->setText(tr("Stop playing on playqueue clear"));
    
    ui_restartPlayingAtStartup = new QCheckBox(main_widget);
    ui_restartPlayingAtStartup->setText(tr("Restart playing at startup"));    
    
    ui_groupBox = new QGroupBox(tr("Use ReplayGain track metadata"));
    ui_groupBox->setCheckable(true);
    ui_groupBox->setChecked(false);
    ui_groupBox->setFlat(true);

    ui_mode_track = new QRadioButton(tr("mode radio"));
    ui_mode_album = new QRadioButton(tr("mode album"));
    
    QVBoxLayout *vbox = new QVBoxLayout;
    vbox->addWidget(new QLabel(tr("Choose ReplayGain mode"), main_widget));
    vbox->addWidget(ui_mode_track);
    vbox->addWidget(ui_mode_album);
    ui_groupBox->setLayout(vbox);    
   
    QVBoxLayout* vl0 = new QVBoxLayout(main_widget);
    vl0->addWidget(ui_stopOnPlayqueueClear);
    vl0->addWidget(ui_restartPlayingAtStartup);
    vl0->addItem( new QSpacerItem(20, 30, QSizePolicy::Minimum, QSizePolicy::Expanding) );        
    vl0->addWidget(ui_groupBox);
    
    // proxy widget
    proxy_widget = new QGraphicsProxyWidget( this );
    proxy_widget->setWidget( main_widget );
    proxy_widget->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
    

    m_title->setParentItem(this);
    m_title->setPos(5,0);
}

void PagePlayer::resizeEvent( QGraphicsSceneResizeEvent *event )
{
Q_UNUSED(event)
    //Debug::debug() << "## PageGeneral::resizeEvent";
    doLayout();
}

void PagePlayer::doLayout()
{
    //Debug::debug() << "## PagePlayer::doLayout";
    m_title->m_width =  m_parent->width();
    int Xpos = m_parent->width() - 350 > 0 ? (m_parent->width() - 350)/2 : 50;
    proxy_widget->setPos(Xpos,50);
}


void PagePlayer::saveSettings()
{
    Debug::debug() << "PagePlayer::saveSettings";
  
    SETTINGS()->_stopOnPlayqueueClear = this->ui_stopOnPlayqueueClear->isChecked();
    SETTINGS()->_restartPlayingAtStartup = this->ui_restartPlayingAtStartup->isChecked();

    if(!ui_groupBox->isChecked())
      SETTINGS()->_replaygain = 0;
    else if(ui_mode_track->isChecked())
      SETTINGS()->_replaygain = 1;
    else 
      SETTINGS()->_replaygain = 2;
}

void PagePlayer::restoreSettings()
{
    this->ui_stopOnPlayqueueClear->setChecked( SETTINGS()->_stopOnPlayqueueClear );
    this->ui_restartPlayingAtStartup->setChecked( SETTINGS()->_restartPlayingAtStartup );
  
    this->ui_groupBox->setChecked( SETTINGS()->_replaygain != 0 );
    this->ui_mode_track->setChecked( SETTINGS()->_replaygain == 1 );
    this->ui_mode_album->setChecked( SETTINGS()->_replaygain == 2 );
}


/*
********************************************************************************
*                                                                              *
*    Class PageLibrary                                                         *
*                                                                              *
********************************************************************************
*/
PageLibrary::PageLibrary(QWidget* parentView) : QGraphicsWidget(0)
{
    m_parent       = parentView;
    
    //! create Gui
    createGui();
        
    //! connection
    connect(this->addPathButton, SIGNAL(clicked()), this, SLOT(on_addPathButton_clicked()));
    connect(this->removePathButton, SIGNAL(clicked()), this, SLOT(on_removePathButton_clicked()));
    connect(this->removeAllPathsButton, SIGNAL(clicked()), this, SLOT(on_removeAllPathsButton_clicked()));
    
    connect(this->autoupdateCollectionsCheckbox, SIGNAL(stateChanged (int)), this, SLOT(updateDatabaseParam1()));
    connect(this->searchCoverFromDir, SIGNAL(stateChanged (int)), this, SLOT(updateDatabaseParam2()));

    connect(this->_comboBoxDb, SIGNAL(currentIndexChanged(QString)), SLOT(loadDatabaseParam(QString)));
    connect(this->_dbNewButton, SIGNAL(clicked()), this, SLOT(newDatabaseParam()));
    connect(this->_dbDelButton, SIGNAL(clicked()), this, SLOT(delDatabaseParam()));
    connect(this->_dbRenameButton, SIGNAL(clicked()), this, SLOT(renameDatabaseParam()));
    
    connect(this->enableMultiDb, SIGNAL(clicked()), this, SLOT(enableMultiDatabase()));
    
    
    enableMultiDatabase();
}

//! ----------- createGui ------------------------------------------------------  
void PageLibrary::createGui()
{
    m_title           = new TitleGraphicItem();
    m_title->m_name   = tr("Library settings");
    m_title->m_width  = m_parent->width();

    // main widget
    QWidget* main_widget = new QWidget();
    
    QPalette palette;
    palette.setColor(QPalette::Background, Qt::blue);
    palette.setColor(QPalette::Window , Qt::white);
    palette.setColor(QPalette::WindowText, Qt::black);
    palette.setColor(QPalette::Base, Qt::white);
    palette.setColor ( QPalette::Disabled, QPalette::WindowText, Qt::darkGray);
    palette.setColor ( QPalette::Disabled, QPalette::Base, Qt::darkGray);
    palette.setColor ( QPalette::Disabled, QPalette::Text, Qt::darkGray);

    main_widget->setAttribute(Qt::WA_NoBackground, true);
    main_widget->setPalette(palette);
    main_widget->setAutoFillBackground(true);
    
    /*-------------------------------------------------*/
    /* Libray settings                                 */
    /* ------------------------------------------------*/     
    QVBoxLayout* verticalLayout = new QVBoxLayout(main_widget);

    //! label
    QLabel *lbl1 = new QLabel(tr("Database Name"), main_widget);
    lbl1->setFont(QFont("Arial",10,QFont::Bold));

    //! Check box --> enable Multi Databases
    enableMultiDb = new QCheckBox(main_widget);
    enableMultiDb->setText(tr("Enable Multi Database support"));
     
    //! horizontal Layout (database list + save + delete)
    _comboBoxDb = new QComboBox(main_widget);
    QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
    sizePolicy.setHorizontalStretch(0);
    sizePolicy.setVerticalStretch(0);
    sizePolicy.setHeightForWidth(_comboBoxDb->sizePolicy().hasHeightForWidth());
    _comboBoxDb->setSizePolicy(sizePolicy);

    _dbNewButton = new QToolButton(main_widget);
    _dbNewButton->setIcon(QIcon(":/images/document-new-48x48.png"));
    _dbNewButton->setToolTip(tr("New database"));
    
    _dbDelButton = new QToolButton(main_widget);
    _dbDelButton->setIcon(QIcon::fromTheme("edit-delete"));
    _dbDelButton->setToolTip(tr("Delete database"));

    _dbRenameButton = new QToolButton(main_widget);
    _dbRenameButton->setIcon(QIcon(":/images/rename-48x48.png"));
    _dbRenameButton->setToolTip(tr("Rename database"));
    
    QHBoxLayout *hl1 = new QHBoxLayout();
    hl1->addWidget(_comboBoxDb);
    hl1->addWidget(_dbNewButton);
    hl1->addWidget(_dbDelButton);
    hl1->addWidget(_dbRenameButton);

    //! label
    QLabel *lbl2 = new QLabel(tr("Database Properties"), main_widget);
    lbl2->setFont(QFont("Arial",10,QFont::Bold));
    
    //! horizontal Layout (source path list + add + delete + delete all)
    collectionsPathsList = new QListWidget(main_widget);
    collectionsPathsList->setMinimumHeight(220);
    
    QHBoxLayout* hl2 = new QHBoxLayout();
    hl2->addWidget(collectionsPathsList);
     
    //! vertical Layout (button Remove/Remove All/Add)
    addPathButton = new QPushButton(main_widget);
    addPathButton->setText(tr("Add ..."));
    
    removePathButton = new QPushButton(main_widget);
    removePathButton->setText(tr("Remove"));
  
    removeAllPathsButton = new QPushButton(main_widget);
    removeAllPathsButton->setText(tr("Remove all"));
  
    QVBoxLayout *vl2 = new QVBoxLayout();
    vl2->addWidget(addPathButton);
    vl2->addWidget(removePathButton);
    vl2->addWidget(removeAllPathsButton);
    vl2->addItem(new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding));
    hl2->addLayout(vl2);

    //! Check box auto check database
    autoupdateCollectionsCheckbox = new QCheckBox(main_widget);
    autoupdateCollectionsCheckbox->setText(tr("Update collections automatically after start"));
  
    //! Check box --> search cover into directory
    searchCoverFromDir = new QCheckBox(main_widget);
    searchCoverFromDir->setText(tr("Search cover art from file directory"));
    
    verticalLayout->addWidget(lbl1);
    verticalLayout->addWidget(enableMultiDb);
    verticalLayout->addLayout(hl1);
    verticalLayout->addItem( new QSpacerItem(20, 30, QSizePolicy::Minimum, QSizePolicy::Fixed) );        
    verticalLayout->addWidget(lbl2);
    verticalLayout->addLayout(hl2);
    verticalLayout->addWidget(autoupdateCollectionsCheckbox);
    verticalLayout->addWidget(searchCoverFromDir);
    
    
    // proxy widget
    proxy_widget = new QGraphicsProxyWidget( this );
    proxy_widget->setWidget( main_widget );
    proxy_widget->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
    

    m_title->setParentItem(this);
    m_title->setPos(5,0);
}

void PageLibrary::resizeEvent( QGraphicsSceneResizeEvent *event )
{
Q_UNUSED(event)
    //Debug::debug() << "## PageGeneral::resizeEvent";
    doLayout();
}

void PageLibrary::doLayout()
{
    //Debug::debug() << "## PageLibrary::doLayout";
    m_title->m_width =  m_parent->width();
    int Xpos = m_parent->width() - 350 > 0 ? (m_parent->width() - 350)/2 : 50;
    proxy_widget->setPos(Xpos,50);
}

//! ---- restoreSettings -------------------------------------------------------
void PageLibrary::restoreSettings()
{
    //! read settings from DatabaseManager 
    DatabaseManager *dbManager = DatabaseManager::instance();
    Debug::debug() << "PageLibrary::restoreSettings dbManager->multiDb" << dbManager->multiDb;
    
    _dbParamList.clear();
    _comboBoxDb->clear();
  
    enableMultiDb->setChecked(dbManager->multiDb);
    enableMultiDatabase();
    //Debug::debug() << "SettingCollectionPage::restoreSettings dbManager->DB_NAME" << dbManager->DB_NAME;
    
    foreach (const QString& name, dbManager->DB_NAME_LIST()) {
      //Debug::debug() << "SettingCollectionPage::restoreSettings dbManager->_dbParamList.keys()" << name;
      addDatabaseParam(name, dbManager->DB_PARAM(name));
    }

    int selectedIdx = _comboBoxDb->findText(dbManager->DB_NAME);
    if (selectedIdx != -1)
      _comboBoxDb->setCurrentIndex(selectedIdx);
    
    _isChanged = false;
}

//! ---- saveSettings -----------------------------------------------------------
void PageLibrary::saveSettings()
{
    Debug::debug() << "PageLibrary::saveSettings";
  
    //! save settings file
    DatabaseManager *dbManager = DatabaseManager::instance();
    
    dbManager->multiDb = enableMultiDb->isChecked();
    dbManager->DB_NAME = _comboBoxDb->currentText();
    dbManager->CLEAR_PARAM();
    
    foreach (const QString& name, _dbParamList.keys()) {
      //Debug::debug() << "SettingCollectionPage::saveSettings _dbParamList[name].autoRebuil" << _dbParamList[name].autoRebuild;
      //Debug::debug() << "SettingCollectionPage::saveSettings _dbParamList[name].checkCover" << _dbParamList[name].checkCover;
      dbManager->SET_PARAM(name, _dbParamList[name]);
      
      //dbManager->_dbParamList[name].autoRebuild = _dbParamList[name].autoRebuild;
      //dbManager->_dbParamList[name].checkCover  = _dbParamList[name].checkCover;
      //dbManager->_dbParamList[name].sourcePathList = QStringList() << _dbParamList[name].sourcePathList;
      //dbManager->_dbParamList[name].useGrid     = _dbParamList[name].useGrid;
    }

    dbManager->saveSettings();
}


//! ---- database instance management ------------------------------------------
void PageLibrary::enableMultiDatabase() 
{
    //Debug::debug() << "SettingCollectionPage::enableMultiDatabase";
    _comboBoxDb->setEnabled(enableMultiDb->isChecked());
    _dbNewButton->setEnabled(enableMultiDb->isChecked());
    _dbDelButton->setEnabled(enableMultiDb->isChecked());
    _dbRenameButton->setEnabled(enableMultiDb->isChecked());
}


void PageLibrary::updateDatabaseParam1()
{
    QString dbName = _comboBoxDb->currentText();
  
    if(_dbParamList.contains(dbName)) {
      _dbParamList[dbName].autoRebuild  = autoupdateCollectionsCheckbox->isChecked();
    }  
}


void PageLibrary::updateDatabaseParam2()
{
    QString dbName = _comboBoxDb->currentText();
  
    if(_dbParamList.contains(dbName)) {
      _dbParamList[dbName].checkCover   = searchCoverFromDir->isChecked();    
    }  
}


void PageLibrary::loadDatabaseParam(QString dbName)
{
   if(dbName.isEmpty()) return;

   if(_dbParamList.contains(dbName)) {
     //Debug::debug() << "SettingCollectionPage::loadDatabaseParam   NAME = " << dbName;
     //Debug::debug() << "SettingCollectionPage::loadDatabaseParam   _dbParamList[dbName].autoRebuild = " << _dbParamList[dbName].autoRebuild;
     //Debug::debug() << "SettingCollectionPage::loadDatabaseParam   _dbParamList[dbName].checkCover = " << _dbParamList[dbName].checkCover;
     
     autoupdateCollectionsCheckbox->setChecked(_dbParamList[dbName].autoRebuild);
     searchCoverFromDir->setChecked(_dbParamList[dbName].checkCover);
      
     collectionsPathsList->clear();
     foreach (QString path, _dbParamList[dbName].sourcePathList) {
       collectionsPathsList->addItem(path);
     }  
      _isChanged = true;
    }
}


void PageLibrary::newDatabaseParam()
{
    QString currentDbName = _comboBoxDb->currentText();
    
    QString name = QInputDialog::getText(m_parent, tr("New database"), tr("Name"),QLineEdit::Normal, currentDbName);
    
    if (name.isEmpty()) return;
    
    if (_dbParamList.contains(name)) {
      QMessageBox::warning(
        m_parent, tr("New database"),
        tr("The database  \"%1\" already exist, please try another name").arg(name),
        QMessageBox::Ok);
      
      return;
    }
    
    DB::S_dbParam dbParam;
    dbParam.autoRebuild    = false;
    dbParam.checkCover     = true;
    dbParam.sourcePathList = QStringList();

    addDatabaseParam(name, dbParam);
    
    _comboBoxDb->setCurrentIndex(_comboBoxDb->findText(name));
    
    //! TODO ? create cover folder, playlist folder
}


void PageLibrary::addDatabaseParam(const QString& name, const DB::S_dbParam& dbParam) 
{
    _dbParamList[name].autoRebuild    = dbParam.autoRebuild;
    _dbParamList[name].checkCover     = dbParam.checkCover;
    _dbParamList[name].sourcePathList.clear();
    _dbParamList[name].sourcePathList << dbParam.sourcePathList;
    
    //Debug::debug() << "SettingCollectionPage::addDatabaseParam  dbParam.sourcePathList" << dbParam.sourcePathList;
    //Debug::debug() << "SettingCollectionPage::addDatabaseParam  dbParam.autoRebuild" << dbParam.autoRebuild;
    //Debug::debug() << "SettingCollectionPage::addDatabaseParam  dbParam.checkCover" << dbParam.checkCover;

    if (_comboBoxDb->findText(name) == -1)
      _comboBoxDb->addItem(name);
}


void PageLibrary::delDatabaseParam()
{
    QString name = _comboBoxDb->currentText();
    
    if (!_dbParamList.contains(name) || name.isEmpty()) return;
    
    int returnCode = QMessageBox::question(
      m_parent, tr("Delete database properties"),
      tr("Are you sure you want to delete the \"%1\" database ?").arg(name),
      QMessageBox::Yes, QMessageBox::No);

    if (returnCode == QMessageBox::No) return;
    
    _dbParamList.remove(name);
    _comboBoxDb->removeItem(_comboBoxDb->currentIndex());
    
    //! TODO delete database file + playlist folder + album folder
}

void PageLibrary::renameDatabaseParam()
{
    //! rename an existing database
    Debug::debug() << "SettingCollectionPage::renameDatabaseParam";
    
    QString oldName = _comboBoxDb->currentText();
    
    QString newName = QInputDialog::getText(m_parent, tr("Rename database"), tr("Name"),QLineEdit::Normal, oldName);
    if (newName.isEmpty() || (newName == oldName) )
      return;
      
    if (_dbParamList.contains(oldName))
    {    
      Debug::debug() << "SettingCollectionPage::renameDatabaseParam";

      DB::S_dbParam dbParam = _dbParamList.take(oldName);
      addDatabaseParam(newName, dbParam);
      _comboBoxDb->removeItem(_comboBoxDb->findText(oldName));
      //! TODO rename dabatase file, cover folder, playlist folder
     
      QCryptographicHash hash(QCryptographicHash::Sha1);
      hash.addData(oldName.toUtf8().constData());
      QFile oldDb(QString(UTIL::CONFIGDIR + "/" + hash.result().toHex() + ".db"));

      QCryptographicHash hash2(QCryptographicHash::Sha1);
      hash2.addData(newName.toUtf8().constData());
      
      if(oldDb.exists()) {
           oldDb.rename(QString(UTIL::CONFIGDIR + "/" + hash2.result().toHex() + ".db"));	
      }
    }
}


//! ----------- on_addPathButton_clicked ---------------------------------------  
void PageLibrary::on_addPathButton_clicked()
{
    QString dirName = QFileDialog::getExistingDirectory(m_parent,
                      tr("Add directory"),
                      QString(),
                      QFileDialog::ShowDirsOnly);
                      
    if (!dirName.isEmpty()) {
      collectionsPathsList->addItem(dirName);

      QString dbName = _comboBoxDb->currentText();
      if (!_dbParamList.contains(dbName) || dbName.isEmpty()) 
        return;

      _dbParamList[dbName].sourcePathList.clear();
      for (int i = 0; i < collectionsPathsList->count(); i++)
        _dbParamList[dbName].sourcePathList << collectionsPathsList->item(i)->text();
    
      _isChanged = true;
    }
    
}

//! ----------- on_removePathButton_clicked ------------------------------------  
void PageLibrary::on_removePathButton_clicked()
{
    foreach (QListWidgetItem *item, collectionsPathsList->selectedItems()) {
      delete collectionsPathsList->takeItem(collectionsPathsList->row(item));
    }
    
    QString dbName = _comboBoxDb->currentText();
    if (!_dbParamList.contains(dbName) || dbName.isEmpty()) return;
   
    _dbParamList[dbName].sourcePathList.clear();
    for (int i = 0; i < collectionsPathsList->count(); i++)
      _dbParamList[dbName].sourcePathList << collectionsPathsList->item(i)->text();

    _isChanged = true;
}

//! ----------- on_removeAllPathsButton_clicked --------------------------------  
void PageLibrary::on_removeAllPathsButton_clicked()
{
    collectionsPathsList->clear();
    
    QString dbName = _comboBoxDb->currentText();
    if (!_dbParamList.contains(dbName) || dbName.isEmpty()) return;
    _dbParamList[dbName].sourcePathList.clear();
   
    _isChanged = true;
}


/*
********************************************************************************
*                                                                              *
*    Class PageRadio                                                           *
*                                                                              *
********************************************************************************
*/
PageRadio::PageRadio(QWidget* parentView) : QGraphicsWidget(0)
{
    m_parent       = parentView;
    
    //! create Gui
    createGui();

    //! connection
    connect(this->addFileButton, SIGNAL(clicked()), this, SLOT(on_addFileButton_clicked()));
    connect(this->removeFileButton, SIGNAL(clicked()), this, SLOT(on_removeFileButton_clicked()));
    connect(this->removeAllFileButton, SIGNAL(clicked()), this, SLOT(on_removeAllFilesButton_clicked()));
}


//! ----------- createGui ------------------------------------------------------  
void PageRadio::createGui()
{
    m_title           = new TitleGraphicItem();
    m_title->m_name   = tr("Radio settings");
    m_title->m_width  = m_parent->width();

    // main widget
    QWidget* main_widget = new QWidget();
    
    QPalette palette;
    palette.setColor(QPalette::Background, Qt::blue);
    palette.setColor(QPalette::Window , Qt::white);
    palette.setColor(QPalette::WindowText, Qt::black);
    palette.setColor(QPalette::Base, Qt::white);
    palette.setColor ( QPalette::Disabled, QPalette::WindowText, Qt::darkGray);
    palette.setColor ( QPalette::Disabled, QPalette::Base, Qt::darkGray);
    palette.setColor ( QPalette::Disabled, QPalette::Text, Qt::darkGray);

    main_widget->setAttribute(Qt::WA_NoBackground, true);
    main_widget->setPalette(palette);
    main_widget->setAutoFillBackground(true);
    
    /*-------------------------------------------------*/
    /* Radio settings widget                           */
    /* ------------------------------------------------*/     
    QVBoxLayout* verticalLayout = new QVBoxLayout(main_widget);
     
    RadioFileList = new QListWidget(main_widget);

    QHBoxLayout* hl1 = new QHBoxLayout();
     
    //! Horizontal Layout
    QLabel *lbl1 = new QLabel(tr("Radio files"), main_widget);
    lbl1->setFont(QFont("Arial",10,QFont::Bold));    
    
    //! Vertical layout for button Remove/Remove All/Add
    addFileButton = new QPushButton(main_widget);
    addFileButton->setText(tr("Add ..."));
  
    removeFileButton = new QPushButton(main_widget);
    removeFileButton->setText(tr("Remove"));
  
    removeAllFileButton = new QPushButton(main_widget);
    removeAllFileButton->setText(tr("Remove all"));
  
    QSpacerItem* verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

    QVBoxLayout *vl2 = new QVBoxLayout();
    vl2->addWidget(addFileButton);
    vl2->addWidget(removeFileButton);
    vl2->addWidget(removeAllFileButton);
    vl2->addItem(verticalSpacer_2);

    hl1->addWidget(RadioFileList);
    hl1->addLayout(vl2);
    
    verticalLayout->addWidget(lbl1);
    verticalLayout->addLayout(hl1);
    
    
    // proxy widget
    proxy_widget = new QGraphicsProxyWidget( this );
    proxy_widget->setWidget( main_widget );
    proxy_widget->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
    

    m_title->setParentItem(this);
    m_title->setPos(5,0);
}

void PageRadio::resizeEvent( QGraphicsSceneResizeEvent *event )
{
Q_UNUSED(event)
    doLayout();
}

void PageRadio::doLayout()
{
    //Debug::debug() << "## PageRadio::doLayout";
    m_title->m_width =  m_parent->width();
    int Xpos = m_parent->width() - 350 > 0 ? (m_parent->width() - 350)/2 : 50;
    proxy_widget->setPos(Xpos,50);
}


//! ----------- saveSettings ---------------------------------------------------
void PageRadio::saveSettings()
{
    Debug::debug() << "PageRadio::saveSettings";
  
    //! nothing done yet
    // a voir si il faut définir la liste des radios utilisées (par db)
}

//! ----------- restoreSettings ------------------------------------------------
void PageRadio::restoreSettings()
{
    const QStringList filters = QStringList() << "*.xspf";
    QDir sourceDir(UTIL::CONFIGDIR + "/radio");
    sourceDir.setNameFilters(filters);  

    QStringList entryList = sourceDir.entryList(filters, QDir::Files);
    RadioFileList->clear();
    RadioFileList->addItems(entryList);
    
    _isChanged = false;    
}

//! ----------- on_addPathButton_clicked ---------------------------------------  
void PageRadio::on_addFileButton_clicked()
{
    QStringList files = QFileDialog::getOpenFileNames(
       m_parent, 
       tr("Add xspf radio files"), 
       QDesktopServices::storageLocation(QDesktopServices::MusicLocation), 
       tr("Radio files(*.xspf);;All files(*.*)"));

    if(files.isEmpty()) return; 
    QString fileName = files.takeFirst();  
    if (!fileName.isEmpty()) 
    {
      QFile file(fileName);
      if( file.copy(UTIL::CONFIGDIR + "/radio/" + QFileInfo(fileName).fileName()) ) {
        //! File copy is OK
        RadioFileList->addItem(QFileInfo(fileName).fileName());	
        _isChanged = true;
      }
    }
    
}

//! ----------- on_removePathButton_clicked ------------------------------------  
void PageRadio::on_removeFileButton_clicked()
{
    foreach (QListWidgetItem *item, RadioFileList->selectedItems()) {
      QString filename =  item->data(Qt::DisplayRole).toString();
      Debug::debug() << "SettingRadioPage -  on_removeFileButton_clicked " << filename;
      QFile file(UTIL::CONFIGDIR + "/radio/" + filename);
      
      if(file.exists()) {
        file.remove();
        delete RadioFileList->takeItem(RadioFileList->row(item));
        _isChanged = true;
      }
    }
}

//! ----------- on_removeAllPathsButton_clicked --------------------------------  
void PageRadio::on_removeAllFilesButton_clicked()
{
    RadioFileList->clear();
    _isChanged = true;
}


/*
********************************************************************************
*                                                                              *
*    Class PageShortcut                                                        *
*                                                                              *
********************************************************************************
*/
PageShortcut::PageShortcut(QWidget* parentView) : QGraphicsWidget(0)
{
    m_parent       = parentView;
    
    createGui();

}

PageShortcut::~PageShortcut()
{
   
}

void PageShortcut::createGui()
{
    //Debug::debug() << "###### PageShortcut createGui";

    m_title           = new TitleGraphicItem();
    m_title->m_name   = tr("Shortcut settings");
    m_title->m_width  = m_parent->width();

    m_items["play"]        = new ShortcutGraphicItem(tr("Play/Pause"), SETTINGS()->_shortcutsKey["play"], QPixmap(":/images/media-play.png"));
    m_items["stop"]        = new ShortcutGraphicItem(tr("Stop"), SETTINGS()->_shortcutsKey["stop"], QPixmap(":/images/media-stop.png"));
    m_items["prev_track"]  = new ShortcutGraphicItem(tr("Previous track"), SETTINGS()->_shortcutsKey["prev_track"], QPixmap(":/images/media-prev.png"));
    m_items["next_track"]  = new ShortcutGraphicItem(tr("Next track"), SETTINGS()->_shortcutsKey["next_track"], QPixmap(":/images/media-next.png"));
    m_items["inc_volume"]  = new ShortcutGraphicItem(tr("Increase volume"), SETTINGS()->_shortcutsKey["inc_volume"], QPixmap(":/images/volume-icon.png"));
    m_items["dec_volume"]  = new ShortcutGraphicItem(tr("Decrease volume"), SETTINGS()->_shortcutsKey["dec_volume"], QPixmap(":/images/volume-icon.png"));
    m_items["mute_volume"] = new ShortcutGraphicItem(tr("Mute/Unmute volume"), SETTINGS()->_shortcutsKey["mute_volume"], QPixmap(":/images/volume-muted.png"));

    connect(m_items.value("play"), SIGNAL(clicked()),this, SLOT(shorcutItemClicked()));
    connect(m_items.value("stop"), SIGNAL(clicked()),this, SLOT(shorcutItemClicked()));
    connect(m_items.value("prev_track"), SIGNAL(clicked()),this, SLOT(shorcutItemClicked()));
    connect(m_items.value("next_track"), SIGNAL(clicked()),this, SLOT(shorcutItemClicked()));
    connect(m_items.value("inc_volume"), SIGNAL(clicked()),this, SLOT(shorcutItemClicked()));
    connect(m_items.value("dec_volume"), SIGNAL(clicked()),this, SLOT(shorcutItemClicked()));
    connect(m_items.value("mute_volume"), SIGNAL(clicked()),this, SLOT(shorcutItemClicked()));
    

    m_items.value("play")->setParentItem(this);
    m_items.value("stop")->setParentItem(this);
    m_items.value("prev_track")->setParentItem(this);
    m_items.value("next_track")->setParentItem(this);
    m_items.value("inc_volume")->setParentItem(this);
    m_items.value("dec_volume")->setParentItem(this);
    m_items.value("mute_volume")->setParentItem(this);
    m_title->setParentItem(this);
    m_title->setPos(5,0);
}

void PageShortcut::resizeEvent( QGraphicsSceneResizeEvent *event )
{
Q_UNUSED(event)
    //Debug::debug() << "## PageGeneral::resizeEvent";
    doLayout();
}

void PageShortcut::doLayout()
{
    //Debug::debug() << "## PageShortcut::doLayout";
    m_title->m_width =  m_parent->width();
    
    int Xpos = m_parent->width() - 350 > 0 ? (m_parent->width() - 350)/2 : 50;
    int Ypos = 50;
    
    m_items.value("play")->setPos(Xpos,Ypos);
    Ypos += 50;
    m_items.value("stop")->setPos(Xpos,Ypos);
    Ypos += 50;
    m_items.value("prev_track")->setPos(Xpos,Ypos);
    Ypos += 50;
    m_items.value("next_track")->setPos(Xpos,Ypos);
    Ypos += 50;    
    m_items.value("inc_volume")->setPos(Xpos,Ypos);
    Ypos += 50;    
    m_items.value("dec_volume")->setPos(Xpos,Ypos);
    Ypos += 50;    
    m_items.value("mute_volume")->setPos(Xpos,Ypos);  
}

//! ----------- restoreSettings ------------------------------------------------
void PageShortcut::restoreSettings()
{
    //Debug::debug() << "PageShortcut restoreSettings";
    m_items["play"]->m_key        = SETTINGS()->_shortcutsKey["play"];
    m_items["stop"]->m_key        = SETTINGS()->_shortcutsKey["stop"];
    m_items["prev_track"]->m_key  = SETTINGS()->_shortcutsKey["prev_track"];
    m_items["next_track"]->m_key  = SETTINGS()->_shortcutsKey["next_track"];
    m_items["inc_volume"]->m_key  = SETTINGS()->_shortcutsKey["inc_volume"];
    m_items["dec_volume"]->m_key  = SETTINGS()->_shortcutsKey["dec_volume"];
    m_items["mute_volume"]->m_key = SETTINGS()->_shortcutsKey["mute_volume"];
    
    _isChanged = false;    
}

//! ----------- saveSettings ------------------------------------------------
void PageShortcut::saveSettings()
{
    Debug::debug() << "PageShortcut::saveSettings";
  
    SETTINGS()->_shortcutsKey["play"]        = m_items["play"]->m_key;
    SETTINGS()->_shortcutsKey["stop"]        = m_items["stop"]->m_key;
    SETTINGS()->_shortcutsKey["prev_track"]  = m_items["prev_track"]->m_key;
    SETTINGS()->_shortcutsKey["next_track"]  = m_items["next_track"]->m_key;
    SETTINGS()->_shortcutsKey["inc_volume"]  = m_items["inc_volume"]->m_key;
    SETTINGS()->_shortcutsKey["dec_volume"]  = m_items["dec_volume"]->m_key;
    SETTINGS()->_shortcutsKey["mute_volume"] = m_items["mute_volume"]->m_key;
}    

void PageShortcut::shorcutItemClicked()
{
    //Debug::debug() << "PageShortcut shorcutItemClicked";
  
    ShortcutGraphicItem *item = qobject_cast<ShortcutGraphicItem *>(sender());
    if(!item) return;
    
    QString item_name = m_items.key(item);
    QString shortcut  = item->m_key;
    
    ShortcutDialog* dialog = new ShortcutDialog(item_name, shortcut, m_parent);
    
    
    if(dialog->exec() == QDialog::Accepted) 
    {
      item->m_key = dialog->newShortcut();
      item->update();
     _isChanged = true;      
    }
    
    delete dialog;
}

/*
********************************************************************************
*                                                                              *
*    Class ShortcutGraphicItem                                                 *
*                                                                              *
********************************************************************************
*/ 
ShortcutGraphicItem::ShortcutGraphicItem(const QString& name, const QString& key, QPixmap pix)
{
    m_name = name;
    m_key = key;
    m_pixmap = pix.scaled(QSize(24,24), Qt::KeepAspectRatio, Qt::SmoothTransformation);
    
    setAcceptsHoverEvents(true);
    setAcceptDrops(false);
    setFlag(QGraphicsItem::ItemIsSelectable, false);
    setFlag(QGraphicsItem::ItemIsMovable, false);
    setFlag(QGraphicsItem::ItemIgnoresTransformations, true);
    setCursor(Qt::PointingHandCursor);
    
    setGraphicsItem(this);
    setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Preferred );
    
    //! option configuration
    QStyle *style =  QApplication::style();
    opt.widget = 0;
    opt.palette = QApplication::palette();
    opt.font = QApplication::font();
    opt.fontMetrics = QFontMetrics(opt.font);
    
    opt.showDecorationSelected = style->styleHint(QStyle::SH_ItemView_ShowDecorationSelected);
    int pm = style->pixelMetric(QStyle::PM_IconViewIconSize);
    
    opt.decorationSize = QSize(pm, pm);
    opt.decorationPosition = QStyleOptionViewItem::Top;
    opt.displayAlignment = Qt::AlignCenter;
   
    opt.locale.setNumberOptions(QLocale::OmitGroupSeparator);
    opt.state |= QStyle::State_Active;
    opt.state |= QStyle::State_Enabled;
    opt.state &= ~QStyle::State_Selected;    
}

QRectF ShortcutGraphicItem::boundingRect() const
{
    return QRectF(0, 0, 500, 30);
}

// Inherited from QGraphicsLayoutItem
void ShortcutGraphicItem::setGeometry(const QRectF &geom)
{
    setPos(geom.topLeft());
}

// Inherited from QGraphicsLayoutItem
QSizeF ShortcutGraphicItem::sizeHint(Qt::SizeHint which, const QSizeF &constraint) const
{
Q_UNUSED(which);
Q_UNUSED(constraint);
    return boundingRect().size();
}

void ShortcutGraphicItem::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget)
{
Q_UNUSED(option)
   //! Draw frame for State_HasFocus item
   QStyle *style = widget ? widget->style() : QApplication::style();
   opt.rect = boundingRect().toRect(); 
   style->drawControl(QStyle::CE_ItemViewItem, &opt, painter, widget);
   
   //! paint album title
   painter->setPen(QColor("black"));
   painter->setFont(opt.font);
   
   painter->drawPixmap(4, 3, m_pixmap);
   painter->drawText(QRect (32,0,300, 30), Qt::AlignLeft | Qt::AlignVCenter,m_name );
   painter->drawText(QRect (310,0,190, 30), Qt::AlignLeft | Qt::AlignVCenter,m_key );
}

void ShortcutGraphicItem::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
{
Q_UNUSED(event)
  opt.state |= QStyle::State_MouseOver;

  this->update();
}

void ShortcutGraphicItem::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
{
Q_UNUSED(event)
  opt.state &= ~QStyle::State_MouseOver;  

  this->update();
}

void ShortcutGraphicItem::mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * event ) 
{
Q_UNUSED(event)
  emit clicked();  
} 



/*
********************************************************************************
*                                                                              *
*    Class PageScrobbler                                                       *
*                                                                              *
********************************************************************************
*/
PageScrobbler::PageScrobbler(QWidget* parentView) : QGraphicsWidget(0)
{
    m_parent       = parentView;
    
    m_title           = new TitleGraphicItem();
    m_title->m_name   = tr("Scrobbler settings");
    m_title->m_width  = m_parent->width();

    // main widget
    QWidget* main_widget = new QWidget();
    
    QPalette palette;
    palette.setColor(QPalette::Background, Qt::blue);
    palette.setColor(QPalette::Window , Qt::white);
    palette.setColor(QPalette::WindowText, Qt::black);
    palette.setColor(QPalette::Base, Qt::white);
    palette.setColor ( QPalette::Disabled, QPalette::WindowText, Qt::darkGray);
    palette.setColor ( QPalette::Disabled, QPalette::Base, Qt::darkGray);
    palette.setColor ( QPalette::Disabled, QPalette::Text, Qt::darkGray);

    main_widget->setAttribute(Qt::WA_NoBackground, true);
    main_widget->setPalette(palette);
    main_widget->setAutoFillBackground(true);
    
    /*-------------------------------------------------*/
    /* Scrobbler settings widget                       */
    /* ------------------------------------------------*/      
    QVBoxLayout* vl0 = new QVBoxLayout(main_widget);

    //! Check box for lastFm scrobbler enable/disable
    useLastFmScrobbler = new QCheckBox(main_widget);
    useLastFmScrobbler->setText(tr("Use LastFm scrobbler"));
   
    //! lastfm connexion 
    QHBoxLayout* hl0 = new QHBoxLayout();
    statusLabel  = new QLabel(main_widget); 
    statusPixmap = new QLabel(main_widget);
    hl0->addWidget(statusPixmap);
    hl0->addWidget(statusLabel);
    hl0->addItem(new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));
    
    accountGroupB = new QGroupBox(tr("Account details"), main_widget);
    accountGroupB->setFlat(true);
    
    QGridLayout* gl = new QGridLayout(accountGroupB);
    gl->setContentsMargins(0, 0, 0, 0);

    QLabel* label_1 = new QLabel(tr("Last.fm username"), accountGroupB);
    gl->addWidget(label_1, 0, 0, 1, 1);

    QLabel* label_2 = new QLabel(tr("Last.fm password"), accountGroupB);
    gl->addWidget(label_2, 1, 0, 1, 1);

    lineEdit_1 = new QLineEdit(accountGroupB);
    lineEdit_1->setMinimumWidth(QFontMetrics(QFont()).width("WWWWWWWWWWWW"));
    gl->addWidget(lineEdit_1, 0, 1, 1, 1);

    lineEdit_2 = new QLineEdit(accountGroupB);
    lineEdit_2->setEchoMode(QLineEdit::Password); 
    gl->addWidget(lineEdit_2, 1, 1, 1, 1);
 
    //! sign in button
    signInButton = new QPushButton(main_widget);
    signInButton->setText(tr("Sign In"));

    //! connextion status
    vl0->addWidget(useLastFmScrobbler);
    vl0->addLayout(hl0);
    vl0->addWidget(accountGroupB);
    vl0->addWidget(signInButton);
    vl0->addItem(new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding));
    
    // proxy widget
    proxy_widget = new QGraphicsProxyWidget( this );
    proxy_widget->setWidget( main_widget );
    proxy_widget->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );

    m_title->setParentItem(this);
    m_title->setPos(5,0);
    
    
    // scrobbler & slots    
    scrobbler  = new LastFm::Scrobbler();
    
    connect(this->useLastFmScrobbler, SIGNAL(clicked()), this, SLOT(enableChange()));
    connect(this->signInButton, SIGNAL(clicked()), this, SLOT(signInClicked()));
    connect(scrobbler, SIGNAL(signInFinished()), this, SLOT(slotSignInDone()));
}

void PageScrobbler::resizeEvent( QGraphicsSceneResizeEvent *event )
{
Q_UNUSED(event)
    doLayout();
}

void PageScrobbler::doLayout()
{
    //Debug::debug() << "## PageScrobbler::doLayout";
  
    m_title->m_width =  m_parent->width();
    int Xpos = m_parent->width() - 350 > 0 ? (m_parent->width() - 350)/2 : 50;
    proxy_widget->setPos(Xpos,50);
}

//! ----------- saveSettings ---------------------------------------------------
void PageScrobbler::saveSettings()
{
    Debug::debug() << "PageScrobbler::saveSettings";
  
    SETTINGS()->_useLastFmScrobbler = this->useLastFmScrobbler->isChecked();
}

//! ----------- restoreSettings ------------------------------------------------
void PageScrobbler::restoreSettings()
{
    _isEnableOld = SETTINGS()->_useLastFmScrobbler;
    this->useLastFmScrobbler->setChecked(_isEnableOld);
    
    enableChange();
    
    updateSignInStatus();
    
    _isChanged = false;
}


//! ----------- enableChange ---------------------------------------------------
void PageScrobbler::enableChange()
{
    //Debug::debug() << "SettingScrobblerPage::enableChange";
    bool isEnableNew = this->useLastFmScrobbler->isChecked();
    
    if(isEnableNew != _isEnableOld)
       _isChanged = true;
    else
       _isChanged = false;
       
    // update state according to enable status
    statusLabel->setEnabled(isEnableNew);
    signInButton->setEnabled(isEnableNew);
    accountGroupB->setEnabled(isEnableNew);
}


//! ----------- getAuthentificationStatus --------------------------------------
void PageScrobbler::updateSignInStatus()
{
    bool auth = scrobbler->isAuthenticated();
    if(auth) {
      lineEdit_1->setText(scrobbler->username());
      lineEdit_2->clear();    
      statusPixmap->setPixmap(QPixmap(":/images/signal_accepted-48x48.png").scaled(QSize(24,24))); 
      statusLabel->setText(QString(tr("You are log in lastFm service as <b>%1</b>").arg(scrobbler->username())));        
      
      signInButton->setText(tr("Sign Out"));
    }
    else {
      statusPixmap->setPixmap(QPixmap(":/images/warning-48x48.png").scaled(QSize(24,24)));  
      statusLabel->setText(tr("You are not log into"));
      signInButton->setText(tr("Sign In"));
    }
}


//! ----------- signInClicked --------------------------------------------------
void PageScrobbler::signInClicked()
{
    _isChanged = true;
    bool auth = scrobbler->isAuthenticated();
    if(auth) {
       lineEdit_1->clear();
       lineEdit_2->clear();
       scrobbler->signOut();
       updateSignInStatus();
    }
    else {
      scrobbler->signIn(lineEdit_1->text(), lineEdit_2->text());
    }
}



//! ----------- slotSignInDone -------------------------------------------------
void PageScrobbler::slotSignInDone()
{
    bool success = scrobbler->isAuthenticated();
    updateSignInStatus();

    if (!success)
      QMessageBox::warning(m_parent, tr("Authentication failed"), tr("Your Last.fm credentials were incorrect"));
}
