/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "StreamModel/shoutcastmanager.h"
#include "networkaccess.h"
#include "utilities.h"
#include "debug.h"

#include <QtCore>


static const QStringList CONST_SHOUTCAST_GENRE = QStringList()
<< "Alternative"
<< "Blues"
<< "Classical"
<< "Country"
<< "Easy Listening"
<< "Electronic"
<< "Folk"
<< "Themes"
<< "Rap"
<< "Inspirational"
<< "International"
<< "Jazz"
<< "Latin"
<< "Metal"
<< "New Age"
<< "Decades"
<< "Pop"
<< "R&B/Urban"
<< "Reggae"
<< "Rock"
<< "Seasonal/Holiday"
<< "Soundtracks"
<< "Talk"
<< "Misc";

/*
********************************************************************************
*                                                                              *
*    Class ShoutCastManager                                                    *
*                                                                              *
********************************************************************************
*/
ShoutCastManager::ShoutCastManager() 
{
    Debug::debug() << " ---- ShoutCastManager --> creation";
    _isUpdating  = false;
    _isError     = false;
    _downloadTry = 0;
}


QList<MediaItemPtr> ShoutCastManager::getShoutCastStream() const
{
    return readFile();
}


bool ShoutCastManager::isReady()
{
    //! pour savoir si il faut tlcharger ou non
    QFile file(UTIL::CONFIGDIR + "/radio/" + "shoutcast.xml");
    return (file.exists() && !_isUpdating);
}

bool ShoutCastManager::isError()
{
    return _isError && !isReady();
}



//! ------------------ startDownload -------------------------------------------  
void ShoutCastManager::startDownload() 
{
    Debug::debug() << " ---- ShoutCastManager --> startDownload";
    if(_isUpdating) return;
    
    if(_downloadTry++ > 2) {
        _isError = true;
        _isUpdating  = false;
        return;
    }
    _isUpdating = true;
    sendGenreRequest();
}
   
//! ------------------  genre request  -----------------------------------------  
void ShoutCastManager::sendGenreRequest() 
{
    Debug::debug() << " ---- ShoutCastManager -> sendGenreRequest";
  
    //! Genre request to shoutcast directory
    QUrl url("http://api.shoutcast.com/legacy/genrelist?k=fa1669MuiRPorUBw");
    //QUrl url("http://yp.shoutcast.com/sbin/newxml.phtml");

    QObject *reply = HTTP()->get(url);
    connect(reply, SIGNAL(data(QByteArray)), this, SLOT(parseGenreResponse(QByteArray)));
    connect(reply, SIGNAL(error(QNetworkReply*)), this, SLOT(shoutCastStopError()));
}

void ShoutCastManager::parseGenreResponse(QByteArray bytes) 
{
    Debug::debug() << " ---- ShoutCastManager -> parseGenreResponse ";

    QXmlStreamReader xml(bytes);
    _shoutGenreList.clear();

    while(!xml.atEnd() && !xml.hasError()) { 
      xml.readNext();
      if (xml.isStartElement() && xml.name() == "genre") {
        QString genre = xml.attributes().value("name").toString();
        //_shoutGenreList.append(QString(xml.readElementText()));
        _shoutGenreList.append(genre); 
        Debug::debug() << " ---- ShoutCastManager -> read genre " << genre;
      }
    }
    
    //! clear de la liste des stations
    _stationList.clear();
    if(_shoutGenreList.isEmpty()) {
       Debug::debug() << " ---- ShoutCastManager -> take predefined genre ";
      _shoutGenreList = CONST_SHOUTCAST_GENRE;
    }
    foreach (const QString &genre, _shoutGenreList) {
      sendStationRequest(genre);
    }
    writeFile(_stationList);  
    shoutCastStop();
}
//! ------------------  station request ----------------------------------------  
void ShoutCastManager::sendStationRequest(QString genre) 
{
    Debug::debug() << " ---- ShoutCastManager -> sendStationRequest for genre" << genre;

    //QString request = QString("http://yp.shoutcast.com/sbin/newxml.phtml?genre=%1&limit=50").arg(genre);
  
    QString request = QString("http://api.shoutcast.com/legacy/genresearch?k=fa1669MuiRPorUBw&genre=%1&limit=50").arg(genre);
  
    QUrl url(request);
  
    QByteArray bytes = HTTP()->syncGetBytes(url);
    parseStationResponse(bytes, genre);
}

void ShoutCastManager::parseStationResponse(QByteArray bytes, QString &genre) 
{
    QXmlStreamReader xml(bytes);

    while(!xml.atEnd() && !xml.hasError()) { 
      xml.readNext();
      if (xml.isStartElement() && xml.name() == "station") {
        MediaItemPtr mi = MediaItemPtr(new MediaItem(TYPE_STREAM));
        
        QString name  = xml.attributes().value("name").toString();
        QString id    = xml.attributes().value("id").toString();
        QString url   = QString("http://yp.shoutcast.com/sbin/tunein-station.pls?id=%1").arg(id);

        mi->data[STREAM::name]      = name;
        mi->data[STREAM::url]       = url;
        mi->data[STREAM::categorie] = genre;

        Debug::debug() << " ---- ShoutCastManager -> added stream" << url;
        _stationList.append(mi); 
      }
    }
}

//! ------------------  local shoutcast file read/write ------------------------  
void ShoutCastManager::writeFile(const QList<MediaItemPtr>& liste)  
{
    //! open file to write
    QFile file(UTIL::CONFIGDIR + "/radio/" + "shoutcast.xml");
    if(file.exists()) file.remove();
    if(!file.open(QIODevice::WriteOnly)) return;  
  
    QXmlStreamWriter xml(&file);
    xml.writeStartDocument();
    xml.writeStartElement("shoutcast");
    xml.writeStartElement("trackList");
  
    foreach (MediaItemPtr mi, liste) 
    {
      if (!mi) continue;
      if (mi->type != TYPE_STREAM) continue;
    
      xml.writeStartElement("track");
      xml.writeTextElement("location", mi->data.at(STREAM::url).toString());
      xml.writeTextElement("title", mi->data.at(STREAM::name).toString());
      xml.writeStartElement("extension");
      xml.writeAttribute("application", "yarock");
      xml.writeTextElement("category", mi->data.at(STREAM::categorie).toString());
      xml.writeEndElement(); //extension
      xml.writeEndElement(); //track
    }
    xml.writeEndElement(); //trackList
    xml.writeEndElement(); //shoutcast
    xml.writeEndDocument(); 
}

QList<MediaItemPtr> ShoutCastManager::readFile() const
{
    QList<MediaItemPtr> listStream;
      
    QFile file(UTIL::CONFIGDIR + "/radio/" + "shoutcast.xml");

    if(!file.exists()) return listStream;
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) return listStream;

    QXmlStreamReader xml(&file); 
    MediaItemPtr mi = MediaItemPtr(0);

    while(!xml.atEnd() && !xml.hasError()) { 
      xml.readNext();
      
      if (xml.isStartElement()) {
         if(xml.name() == "track") {
            //! new stream entry
            mi = MediaItemPtr(new MediaItem(TYPE_STREAM));
            
         } else if (xml.name() == "title" && mi) {
           mi->data[STREAM::name] = QString(xml.readElementText());
         } else if (xml.name() == "location" && mi) {
           mi->data[STREAM::url]  = QString(xml.readElementText());
         } else if (xml.name() == "category" && mi) {
           mi->data[STREAM::categorie] = QString(xml.readElementText());              
         }
      }
      else if (xml.isEndElement() && xml.name() == "track") {
         //! if stream OK --> add it
         if (!mi->data.at(STREAM::url).toString().isEmpty() &&
             !mi->data.at(STREAM::name).toString().isEmpty() &&
             !MEDIA::isUrlLocal(mi->data.at(STREAM::url).toString())) 
          {
             listStream.append(mi);
          }
          mi = MediaItemPtr(0);
      }
   }
   
   if(!_showDuplicate) 
   {
      qSort(listStream.begin(), listStream.end(), MEDIA::compareStreamName);
      QString _current;
      foreach (MediaItemPtr mi, listStream) {
        QString _new = mi->data.at(STREAM::name).toString();
        if(_current == _new) {
          listStream.removeOne ( mi );
        }
        else {
          _current = _new;
        }
      }
    }
    
    qSort(listStream.begin(), listStream.end(), MEDIA::compareStreamCategorie);
    return listStream;
}


//! ------------------  shoutCastStop ------------------------------------------  
void ShoutCastManager::shoutCastStop()  
{
    Debug::debug() << " ---- ShoutCastManager -> shoutCastStop";
    _isUpdating = false;
    emit shoutCastUpdated();
}

void ShoutCastManager::shoutCastStopError()  
{
    Debug::debug() << " ---- ShoutCastManager -> shoutCastStopError";
    _isUpdating = false;
    emit shoutCastStoppedError();
}
