/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "mainwindow.h"

// player engine
#include "EnginePlayer/engineplayer.h"

// widget 
#include "Playlist/playlistview.h"
#include "Playlist/playlistwidget.h"
#include "NowPlaying/nowplayingview.h"
#include "GraphicViewWidget/graphicviewwidget.h"
#include "CentralWidget/centralwidget.h"
#include "CentralWidget/menumodel.h"
#include "CentralWidget/menuview.h"
#include "Widget/playertoolbar.h"            
#include "Widget/minimalwidget.h"            // 0.0.43
#include "Widget/statuswidget.h"                

// equalizer
#include "Widget/equalizer_dialog.h"

// data model & scene
#include "CollectionModel/collectionmodel.h"
#include "CollectionModel/plsfilemodel.h"
#include "StreamModel/streammodel.h"
#include "CollectionModel/histomodel.h" 
#include "Playlist/playlistmodel.h"

#include "CollectionView/collectionscene.h"
#include "StreamView/stream_scene.h"
#include "ContextView/contextscene.h"         // 0.0.42

#include "Settings/settings_scene.h"

// core 
#include "scrobblermanager.h"                 // 0.0.48
#include "Histo/histomanager.h"               // 0.0.45
#include "threadmanager.h"                    
#include "Database/databasemanager.h"         // 0.0.44
#include "CollectionView/virtualplayqueue.h"
#include "audiocontrols.h"                   
#include "commandlineoptions.h"               // 0.0.53
#include "globalshortcuts.h"                  // 0.0.53 
#include "yarocksettings.h"                   // 0.0.53
#include "database.h"
#include "networkaccess.h"

#include "global_actions.h"
#include "constants.h"
#include "utilities.h"
#include "debug.h"

// Dbus & remote 
#include "Dbus/dbusnotification.h"           // 0.0.50
#include "Dbus/mpris1.h"                     // 0.0.53

// dialog
#include "Dialog/aboutdialog.h"
#include "Dialog/dboperationdialog.h"
#include "Dialog/addstreamdialog.h"
#include "Playlist/playlisteditor.h"         // 0.0.55

#include "Widget/statuswidget.h"


MainWindow* MainWindow::INSTANCE = 0;

/*
********************************************************************************
*                                                                              *
*    Class MainWindow                                                          *
*                                                                              *
********************************************************************************
*/
MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
{
    INSTANCE = this;

    //Debug::debug() << "line :"<< __LINE__ <<  " MainWindow -> start";
    setObjectName(QString::fromUtf8("YaRock Player"));
    setWindowTitle(QString::fromUtf8("YaRock Player"));
    
    _appIcon = new QIcon(":/icon/64x64/yarock.png");
    QApplication::setWindowIcon(*_appIcon);
    
    // Size Policy
    QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    sizePolicy.setHorizontalStretch(0);
    sizePolicy.setVerticalStretch(0);
    sizePolicy.setHeightForWidth(this->sizePolicy().hasHeightForWidth());
    this->setSizePolicy(sizePolicy);
    
    // Menu Policy
    this->setContextMenuPolicy (Qt::NoContextMenu);
    
    //! ###############   Settings   ########################################
    new YarockSettings();
    SETTINGS()->readSettings();
    
    //! ###############   NetWork   #########################################
    new NetworkManager();
    
    //! ###############   Actions    ########################################
    new GlobalActions();
    createActions();
    
    //! ############### init Player #########################################
    _player       = new EnginePlayer();
    
    //! ############### init collection database ############################
    Debug::debug() << "Mainwindow -> init collection database";
    _dbManager = new DatabaseManager();
    
    Debug::debug() << "Mainwindow -> init collection database _dbManager->DB_NAME" << _dbManager->DB_NAME;
    Debug::debug() << "Mainwindow -> init collection database _dbManager->DB_FILE()" << _dbManager->DB_FILE();

    //! ############### init playqueue part  ################################
    _playlistModel   = new PlaylistModel(this);
    _playlistView    = new PlaylistView(this, _playlistModel);
    _nowplayingview  = new NowPlayingView(this);
    _playlistwidget  = new PlaylistWidget(this,_playlistView);

    //! ############### init data model  ####################################
    _collectionModel = new CollectionModel(this);
    _plsFileModel    = new PlsFileModel(this);
    _histoModel      = new HistoModel(this);
    _streamModel     = new StreamModel(this);
    

    //! WARNING menumodel defined browsing views actions
    //! WARNING centraltoolbar defined search and up/down actions
    //Debug::debug() << "MainWindow -> creation menu model";
    _menuModel      = new MenuModel();
    _menuView       = new MenuView(_menuModel, this);         
    _centralToolBar = new CentralToolBar(this);
        
    //! ############### init views  #########################################
    //Debug::debug() << "MainWindow -> creation GraphicViewWidget";
    _graphicViewWidget    = new GraphicViewWidget(this);
    _collectionScene      = new CollectionScene(_graphicViewWidget);
    _graphicViewWidget->setCollectionScene(_collectionScene);
    
    //Debug::debug() << "MainWindow -> creation VirtualPlayQueue";
    _virtualPlayqueue   = new VirtualPlayqueue(this); 
    _collectionScene->setVirtualPlayqueue(_virtualPlayqueue);

    //Debug::debug() << "MainWindow -> creation StreamModel/StreamScene";
    _streamScene      = new StreamScene(_graphicViewWidget);
    _streamScene->setVirtualPlayqueue(_virtualPlayqueue);
    _graphicViewWidget->setStreamScene(_streamScene);

    //Debug::debug() << "MainWindow -> creation ContextScene";
    _graphicViewWidget->setContextScene(new ContextScene(_graphicViewWidget));

    //Debug::debug() << "MainWindow -> creation Settings Scene";
    m_settingsScene = new SettingsScene(_graphicViewWidget);
    _graphicViewWidget->setSettingsScene(m_settingsScene);
    
    //! ############### End Gui Stuff     ###################################
    createGui();
    
    //! ############### _statusWidget            ############################
    _statusWidget = new StatusWidget(_centralWidget);

    //! ############### Thread management ###################################
    _threadManager = new ThreadManager();
    _threadManager->setModel(_playlistModel);

    _playlistModel->setThreadManager(_threadManager);   
    _streamScene->setThreadManager(_threadManager);
    
    //Debug::debug() << "MainWindow -> creation Player tool bar";
    this->addToolBar(Qt::TopToolBarArea, new PlayerToolBar(this) );

    //! Shortcuts & Signals
    //Debug::debug() << "MainWindow -> creation global shortcut";
    _globalShortcuts = new GlobalShortcuts (this);
    
    //! ###############     Connection    ###################################
    //Debug::debug() << "MainWindow -> connectSlots";
    connectSlots();
    
    //! ############### restore playlist last played ########################
    if( QFile::exists(UTIL::CONFIGDIR + "/last.xspf") )
      _threadManager->playlistAddFile(QString(UTIL::CONFIGDIR).append("/last.xspf"));

    //! ############### Scrobbler ###########################################
    // le scrobbler doit être initialisé avant le fisrtStartDialog
    //Debug::debug() << "MainWindow -> Scrobbler";      
    _scrobblerManager =  new ScrobblerManager();
    QObject::connect(_playlistView, SIGNAL(signal_lastFmLove(QString,QString)), _scrobblerManager, SLOT(loveTrackRequest(QString,QString)));    
    

    //! ############### start database ######################################
    /* use Timer to let Maindow show before all database stuff is done */
    QTimer::singleShot(10, this, SLOT(slot_database_start()));
    
    
    //! ############### Systray Icon ########################################
    _canClose = false;
    _trayIcon = 0;        
    reloadSystraySettings();
    
    //! ############### Minimal window ######################################
    _minimalwidget = NULL;
    
    //! ############### History manager #####################################
    _histoManager = new HistoManager();
    
    //! ############### DBUS & MPRIS ########################################
    _dbusNotification = new DbusNotification(this);
    _mpris1 = 0;
    reloadMprisSettings();
    
    //! ############### STATUS WIDGET #######################################
    _statusWidget->moveWidget();
    _statusWidget->hide();
    
    //! ############### Restore Playing   ###################################
    if(SETTINGS()->_restartPlayingAtStartup)
      restorePlayingTrack();    

    //! ############### Restore windows geometry ############################
    Debug::debug() << "MainWindow -> restore geometry";      
    if( !SETTINGS()->_windowsGeometry.isEmpty() )
      restoreGeometry(SETTINGS()->_windowsGeometry);
    else
      resize(800, 750);

    //! ############### Hide window       ###################################
    if(SETTINGS()->_hideAtStartup)
      this->showMinimized();
}


//! --------- ~MainWindow ------------------------------------------------------  
MainWindow::~MainWindow()
{
    //! Save current playling track
    if(SETTINGS()->_restartPlayingAtStartup)
      savePlayingTrack();
      
    _player->stop();

    //! Save current playlist to file
    if(_playlistModel->rowCount() <= 0) 
    {
        if( QFile::exists(UTIL::CONFIGDIR + "/last.xspf") )
          QFile::remove(UTIL::CONFIGDIR + "/last.xspf");
    }
    else 
    {
      _threadManager->playlistSaveToFile(QString(UTIL::CONFIGDIR).append("/last.xspf"));
    }

    _threadManager->stopThread();
    
    //! Save setting
    SETTINGS()->_windowsGeometry = this->saveGeometry();
    SETTINGS()->_windowsState    = this->saveState();
    SETTINGS()->_splitterState   = _centralWidget->splitterState();
    SETTINGS()->_viewMode        = (int)_graphicViewWidget->mode();
    SETTINGS()->_playqueueMode   = (int)_playlistwidget->mode();
 
    //! Write setting
    SETTINGS()->writeSettings();
    
    //! database manager save settings
    _dbManager->saveSettings();
    
    //! delete repertoire temporaire de telechargement
    QDir dir(UTIL::CONFIGDIR + "/download/");

    Q_FOREACH(QFileInfo info, dir.entryInfoList(QDir::NoDotAndDotDot | QDir::System | QDir::Hidden  | QDir::AllDirs | QDir::Files, QDir::DirsFirst)) 
    {
      QFile::remove(info.absoluteFilePath());
    }
  
    //! delete object
    delete _threadManager;
    delete _scrobblerManager;
  
    //!WARNING needed in order to active save method
    delete _centralWidget;
    delete _menuModel;
    delete _collectionModel;
    delete _plsFileModel;
    delete _playlistModel;
    delete _player;

    QPixmapCache::clear();
    Debug::debug() << "Mainwindow -> Bye Bye";
}

//! --------- createGui --------------------------------------------------------  
void MainWindow::createGui() 
{
     Debug::debug() << "Mainwindow -> createGui";
    _centralWidget = new CentralWidget(this,
                                _centralToolBar,
                                _menuView,
                                _nowplayingview,
                                _playlistwidget,
                                _graphicViewWidget
                               );
    this->setCentralWidget(_centralWidget);
}

//! --------- reloadSystraySettings --------------------------------------------
void MainWindow::reloadSystraySettings() 
{
    // systray de-activated      
    if(!SETTINGS()->_useTrayIcon)
    {
      if(_trayIcon != 0) {
          disconnect(_trayIcon, 0,this, 0);
          delete _trayIcon;
         _trayIcon = 0;
      }
    }
    // systray activated
    else 
    {
      if(!_trayIcon) {
        QMenu *_trayIconMenu = new QMenu(this);
        //QAction *focus_action = new QAction(tr("set focus"),this);
        _trayIconMenu->addAction(ACTIONS()->value(ENGINE_PLAY_PREV));
        _trayIconMenu->addAction(ACTIONS()->value(ENGINE_PLAY));
        _trayIconMenu->addAction(ACTIONS()->value(ENGINE_STOP));
        _trayIconMenu->addAction(ACTIONS()->value(ENGINE_PLAY_PREV));
        _trayIconMenu->addSeparator();
        _trayIconMenu->addAction(ACTIONS()->value(LASTFM_LOVE_NOW_PLAYING));
        _trayIconMenu->addSeparator();
        _trayIconMenu->addAction(ACTIONS()->value(APP_QUIT));

        _trayIcon = new QSystemTrayIcon(this);
        
        //! custom tray icon
        if( QFile::exists(UTIL::CONFIGDIR + "/systray_icon.png") )
          _trayIcon->setIcon(QIcon(UTIL::CONFIGDIR + "/systray_icon.png"));
        else
          _trayIcon->setIcon(*_appIcon);          
        
        _trayIcon->setToolTip("Yarock");
        _trayIcon->setContextMenu(_trayIconMenu);
        _trayIcon->show();

        connect(ACTIONS()->value(LASTFM_LOVE_NOW_PLAYING), SIGNAL(triggered()), this, SLOT(slot_LastFmLove()));
        connect(_trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this, SLOT(slot_systray_clicked(QSystemTrayIcon::ActivationReason)));
        connect(_trayIcon, SIGNAL(messageClicked()), this, SLOT(setFocus()));
      }
    }
}

     
//! --------- createActions ----------------------------------------------------
void MainWindow::createActions()
{
    //! yarock global actions
    ACTIONS()->insert(APP_QUIT, new QAction(QIcon::fromTheme("application-exit"),tr("&Quit Yarock"), this));
    ACTIONS()->insert(APP_SHOW_YAROCK_ABOUT, new QAction(QIcon::fromTheme("help-about"),tr("About YaRock"), this));
    ACTIONS()->insert(APP_SHOW_QT_ABOUT, new QAction(QIcon(":/trolltech/qmessagebox/images/qtlogo-64.png"),tr("About Qt"), this));
  
    //! playlist action
    ACTIONS()->insert(PLAYQUEUE_ADD_FILE, new QAction(QIcon(":/images/track-48x48.png"),tr("&Add media to playlist"), this));               
    ACTIONS()->insert(PLAYQUEUE_ADD_DIR, new QAction(QIcon(":/images/folder.png"),tr("&Add directory to playlist"), this));               
    ACTIONS()->insert(PLAYQUEUE_ADD_URL, new QAction(QIcon(":/images/media-url-48x48.png"),tr("&Add Url..."), this));
    ACTIONS()->insert(PLAYQUEUE_CLEAR, new QAction(QIcon::fromTheme("edit-clear-list"), tr("&Clear playlist"), this));     
    ACTIONS()->insert(PLAYQUEUE_SAVE, new QAction(QIcon(":/images/save-as-32x32.png"), tr("&Save playlist to file"), this));  
    ACTIONS()->insert(PLAYQUEUE_AUTOSAVE, new QAction(QIcon(":/images/save-32x32.png"), tr("&Auto save playlist to YaRock database"), this));
    ACTIONS()->insert(PLAYQUEUE_REMOVE_ITEM,new QAction(QIcon::fromTheme("edit-delete"),tr("&Remove media from playlist"), this));
    ACTIONS()->insert(PLAYQUEUE_STOP_AFTER,new QAction(QIcon(),tr("Stop after this track"), this));
    ACTIONS()->value(PLAYQUEUE_STOP_AFTER)->setCheckable(true);

    //! player action
    ACTIONS()->insert(ENGINE_PLAY, new QAction(QIcon(":/images/media-play.png"), tr("Play or Pause media"), this));     
    ACTIONS()->insert(ENGINE_STOP, new QAction(QIcon(":/images/media-stop.png"), tr("Stop playing media"), this));      
    ACTIONS()->insert(ENGINE_PLAY_NEXT, new QAction(QIcon(":/images/media-next.png"), tr("Play next media"), this));     
    ACTIONS()->insert(ENGINE_PLAY_PREV, new QAction(QIcon(":/images/media-prev.png"), tr("Play previous media"), this));     
    ACTIONS()->insert(ENGINE_VOL_MUTE, new QAction(QIcon(":/images/volume-icon.png"),"", this));
    ACTIONS()->insert(ENGINE_AUDIO_EQ, new QAction(QIcon(":/images/equalizer48x48.png"),tr("Audio equalizer"), this));
  
    //! database action
    ACTIONS()->insert(DIALOG_DB_OPERATION, new QAction(QIcon(":/images/rebuild.png"),tr("Database operation"), this));
    ACTIONS()->insert(TASK_COVER_SEARCH, new QAction(QIcon(":/images/lastfm.png"),tr("Find all missing cover art"), this));
  
    //! playlist Mode Affichage
    ACTIONS()->insert(PLAYQUEUE_MODE_TITLE, new QAction(tr("Title"), this));               
    ACTIONS()->insert(PLAYQUEUE_MODE_ALBUM, new QAction(tr("Album - Title"), this));               
    ACTIONS()->insert(PLAYQUEUE_MODE_ARTIST, new QAction(tr("Artist - Album - Title"), this));  
    ACTIONS()->insert(PLAYQUEUE_MODE_EXTENDED, new QAction(tr("Extended"), this));  
    ACTIONS()->value(PLAYQUEUE_MODE_TITLE)->setCheckable(true);
    ACTIONS()->value(PLAYQUEUE_MODE_ALBUM)->setCheckable(true);
    ACTIONS()->value(PLAYQUEUE_MODE_ARTIST)->setCheckable(true);
    ACTIONS()->value(PLAYQUEUE_MODE_EXTENDED)->setCheckable(true);

    //! Show/hide MenuBar / PlayQueue / Menu
    ACTIONS()->insert(APP_SHOW_PLAYQUEUE, new QAction(QIcon(),tr("Show playqueue panel"), this));     
    ACTIONS()->insert(APP_SHOW_MENU, new QAction(QIcon(),tr("Show menu panel"), this));
    ACTIONS()->insert(APP_SHOW_NOW_PLAYING, new QAction(QIcon(),tr("Show now playing"), this));
    ACTIONS()->value(APP_SHOW_PLAYQUEUE)->setCheckable(true);
    ACTIONS()->value(APP_SHOW_MENU)->setCheckable(true);
    ACTIONS()->value(APP_SHOW_NOW_PLAYING)->setCheckable(true);
  
    //! Screen mode Switch actions
    ACTIONS()->insert(APP_MODE_COMPACT, new QAction(QIcon(":/images/screen-minimalmode.png"), tr("Switch to minimal mode"), this));
    ACTIONS()->insert(APP_MODE_NORMAL, new QAction(QIcon(":/images/screen-normalmode.png"), tr("Switch to normal mode"), this));

    //! LastFM Love
    ACTIONS()->insert(LASTFM_LOVE, new QAction(QIcon(":/images/lastfm.png"), tr("Send LastFm love track"), this));
    ACTIONS()->insert(LASTFM_LOVE_NOW_PLAYING, new QAction(QIcon(":/images/lastfm.png"), tr("Send LastFm love track"), this));

    //! restore actions states
    (ACTIONS()->value(APP_SHOW_PLAYQUEUE))->setChecked(SETTINGS()->_showPlayQueuePanel);
    (ACTIONS()->value(APP_SHOW_MENU))->setChecked(SETTINGS()->_showMenuPanel);
    (ACTIONS()->value(APP_SHOW_NOW_PLAYING))->setChecked(SETTINGS()->_showNowPlaying);

    //! playlist editor
    ACTIONS()->insert(DIALOG_PLAYLIST_EDITOR, new QAction(QIcon(":/images/edit_info.png"),tr("Open playlist editor"), this));
    
    //! jump to track
    ACTIONS()->insert(BROWSER_JUMP_TO_ARTIST, new QAction(QIcon(),tr("Jump to artist"), this));
    ACTIONS()->insert(BROWSER_JUMP_TO_ALBUM, new QAction(QIcon(),tr("Jump to album"), this));
    ACTIONS()->insert(BROWSER_JUMP_TO_TRACK, new QAction(QIcon(),tr("Jump to track"), this));
    ACTIONS()->value(BROWSER_JUMP_TO_ARTIST)->setData(QVariant(int(MAIN::ViewArtist)));
    ACTIONS()->value(BROWSER_JUMP_TO_ALBUM)->setData(QVariant(int(MAIN::ViewAlbum)));
    ACTIONS()->value(BROWSER_JUMP_TO_TRACK)->setData(QVariant(int(MAIN::ViewTrack)));
}


//! --------- connectSlots -----------------------------------------------------
void MainWindow::connectSlots()
{
    //! Global Yarock Actions
    QObject::connect(ACTIONS()->value(APP_QUIT), SIGNAL(triggered()), SLOT(slot_on_yarock_quit()));
    QObject::connect(ACTIONS()->value(APP_SHOW_YAROCK_ABOUT), SIGNAL(triggered()), SLOT(slot_on_aboutYarock()));
    QObject::connect(ACTIONS()->value(APP_SHOW_QT_ABOUT), SIGNAL(triggered()), SLOT(slot_on_aboutQt()));
    
    //! Playback option
    QObject::connect(ACTIONS()->value(PLAYQUEUE_STOP_AFTER), SIGNAL(triggered()), SLOT(slot_stop_after_media_triggered()));
    
    //! Connection Actions player
    QObject::connect(_player, SIGNAL(trackChanged(const QString&)), this, SLOT(slot_player_on_track_change(const QString&)));
    QObject::connect(_player, SIGNAL(queueFinished()), this, SLOT(stopPlayer()));  
    QObject::connect(_player, SIGNAL(trackAboutToFinish()), this, SLOT(slot_player_enqueue_next()));  
    QObject::connect(_player, SIGNAL(engineStateChanged()), this, SLOT(slot_player_on_state_change()));
    QObject::connect(_player, SIGNAL(player_stop_requested()), this, SLOT(stopPlayer()));  
    
    QObject::connect(ACTIONS()->value(ENGINE_PLAY), SIGNAL(triggered()), SLOT(playOrPause()));
    QObject::connect(ACTIONS()->value(ENGINE_PLAY_NEXT), SIGNAL(triggered()), SLOT(playNext()));
    QObject::connect(ACTIONS()->value(ENGINE_PLAY_PREV), SIGNAL(triggered()), SLOT(playPrev()));
    QObject::connect(ACTIONS()->value(ENGINE_STOP), SIGNAL(triggered()), SLOT(stopPlayer()));
    QObject::connect(ACTIONS()->value(ENGINE_AUDIO_EQ), SIGNAL(triggered()), SLOT(slot_eq_openDialog()));
    
    //! Connection Actions playlist
    QObject::connect(ACTIONS()->value(PLAYQUEUE_CLEAR), SIGNAL(triggered()), SLOT(slot_playqueue_clear()));
    QObject::connect(ACTIONS()->value(PLAYQUEUE_SAVE), SIGNAL(triggered()), SLOT(slot_playqueue_save()));
    QObject::connect(ACTIONS()->value(PLAYQUEUE_AUTOSAVE), SIGNAL(triggered()), SLOT(slot_playqueue_save_auto()));
    QObject::connect(ACTIONS()->value(PLAYQUEUE_ADD_FILE), SIGNAL(triggered()), SLOT(slot_playqueue_add()));
    QObject::connect(ACTIONS()->value(PLAYQUEUE_ADD_DIR), SIGNAL(triggered()), SLOT(slot_playqueue_add()));
    QObject::connect(ACTIONS()->value(PLAYQUEUE_ADD_URL), SIGNAL(triggered()), SLOT(slot_playqueue_add()));
    QObject::connect(_playlistView, SIGNAL(signal_playlist_itemDoubleClicked()), this, SLOT(slot_play_from_playqueue()));    
  
    //! Connection Actions Database
    QObject::connect(ACTIONS()->value(DIALOG_DB_OPERATION), SIGNAL(triggered()), SLOT(slot_database_dialog()));
    QObject::connect(ACTIONS()->value(TASK_COVER_SEARCH), SIGNAL(triggered()), SLOT(startCoverSearchThread()));

    //! Connection signaux COLLECTION
    QObject::connect(_virtualPlayqueue, SIGNAL(signal_collection_playTrack()), this, SLOT(slot_play_from_collection()));    
    QObject::connect(_collectionScene,SIGNAL(userDatabaseUpdate(int)),this,SLOT(slot_model_populate(int)), Qt::DirectConnection);

    //! Connection signaux USER SETTINGS
    QObject::connect(m_settingsScene, SIGNAL(settings_saved()), SLOT(slot_on_settings_saved()));
  
    //! ThreadManager connection
    QObject::connect(_centralToolBar, SIGNAL(dbNameChanged()), this, SLOT(slot_database_start()));;
    QObject::connect(_threadManager, SIGNAL(dbBuildFinished()), this, SLOT(slot_model_populate()));

    QSignalMapper *sm = new QSignalMapper(this);
    sm->setMapping(_threadManager, 2);
    QObject::connect(_threadManager, SIGNAL(playlistSaved()), sm, SLOT(map()));
    QObject::connect(sm, SIGNAL(mapped(int)),this, SLOT(slot_model_populate(int)));

    QObject::connect(_threadManager, SIGNAL(modelPopulationFinished(E_MODEL_TYPE)), this, SLOT(slot_views_update(E_MODEL_TYPE)));

    //! Screen mode connection
    QObject::connect(ACTIONS()->value(APP_MODE_COMPACT), SIGNAL(triggered()), SLOT(slot_widget_mode_switch()));;
    QObject::connect(ACTIONS()->value(APP_MODE_NORMAL), SIGNAL(triggered()), SLOT(slot_widget_mode_switch()));

    //! playlist editor 
    QObject::connect(ACTIONS()->value(DIALOG_PLAYLIST_EDITOR), SIGNAL(triggered()), SLOT(slot_open_playlist_editor()));
    
    //! now playing actions
    QObject::connect(ACTIONS()->value(BROWSER_JUMP_TO_ARTIST), SIGNAL(triggered()), SLOT(slot_jump_to_now_playing()));
    QObject::connect(ACTIONS()->value(BROWSER_JUMP_TO_ALBUM), SIGNAL(triggered()), SLOT(slot_jump_to_now_playing()));
    QObject::connect(ACTIONS()->value(BROWSER_JUMP_TO_TRACK), SIGNAL(triggered()), SLOT(slot_jump_to_now_playing()));        
}


//! --------- resizeEvent ------------------------------------------------------
void MainWindow::resizeEvent ( QResizeEvent * event )
{
//Q_UNUSED(event)
    //Debug::debug() << "Mainwindow -> moveEvent";
    if(_statusWidget->isVisible())
      _statusWidget->moveWidget();

    event->accept();    
}    



//! --------- Quit Actions -----------------------------------------------------
void MainWindow::slot_on_yarock_quit()
{
    _canClose = true;
    this->close();
}

void MainWindow::closeEvent(QCloseEvent *event)
{
    if(!_trayIcon) {
      this->close();
    }
    else {
      // only hide app is tray is running
      if (_trayIcon->isVisible() && (_canClose == false)) {
        this->hide();
        event->ignore();
      }
    }
} 

//! -------------- Systray methods ---------------------------------------------
void MainWindow::slot_systray_clicked(QSystemTrayIcon::ActivationReason reason)
{
    if (reason == QSystemTrayIcon::Trigger) 
    {
        bool is_visible = !this->isVisible();
        this->setVisible(is_visible);
        if (is_visible) {
          activateWindow();
          showNormal();
        }        
    }
}

//! --------- Help About -------------------------------------------------------
void MainWindow::slot_on_aboutYarock()
{
    AboutDialog about;
    about.exec();
}

void MainWindow::slot_on_aboutQt()
{
    QMessageBox::aboutQt(this,tr("About Qt"));
}

/*******************************************************************************
    Playlist Editor 
*******************************************************************************/
void MainWindow::slot_open_playlist_editor()
{
    PlaylistEditor *editor = new PlaylistEditor(this);
    editor->show();
      
    QSignalMapper* mapper = new QSignalMapper();
    mapper->setMapping(editor, 2);

    connect(editor, SIGNAL(playlistSaved()), mapper, SLOT(map()));
    connect(mapper, SIGNAL(mapped(int)), this, SLOT(slot_model_populate(int))); 

    connect(editor, SIGNAL(closed()), this, SLOT(slot_delete_playlist_editor()));        
}

void MainWindow::slot_delete_playlist_editor()
{
    PlaylistEditor *editor = qobject_cast<PlaylistEditor *>(sender());
    if(editor)
      editor->deleteLater(); 
}


/*******************************************************************************
    File management
*******************************************************************************/
void MainWindow::slot_playqueue_add()
{
    QAction *action = qobject_cast<QAction *>(sender());
    if(!action) return;
    
    /*--------------------------------------------------*/
    /* add file to playqueue                            */
    /* -------------------------------------------------*/    
    if( action == ACTIONS()->value(PLAYQUEUE_ADD_FILE) )
    {    
      // TO DO : repartir du dernier repertoire ouvert 
      QStringList files = QFileDialog::getOpenFileNames(
                  this, 
                  tr("Add music files or playlist"), 
                  QDesktopServices::storageLocation(QDesktopServices::MusicLocation), 
                  tr("Audio files(*.mp3 *.wav *.ogg *.flac *.m4a *.aac);;Playlists files(*.m3u *.pls *.xspf);;All files(*.*)"));

      _threadManager->playlistAddFiles(files);
    }
    /*--------------------------------------------------*/
    /* add dir to playqueue                             */
    /* -------------------------------------------------*/      
    else if ( action == ACTIONS()->value(PLAYQUEUE_ADD_DIR) ) 
    {
      // TO DO : repartir du dernier repertoire ouvert 
      QString dirName = QFileDialog::getExistingDirectory(
                        this,
                        tr("Add music directory"),
                        QDesktopServices::storageLocation(QDesktopServices::MusicLocation), 
                        QFileDialog::ShowDirsOnly);
  
      if (!dirName.isEmpty())
        _threadManager->playlistAddFile(dirName);          
    }
    /*--------------------------------------------------*/
    /* add url to playqueue                             */
    /* -------------------------------------------------*/        
    else if ( action == ACTIONS()->value(PLAYQUEUE_ADD_URL) ) 
    {
      AddStreamDialog stream_dialog(this,false);
      
      if(stream_dialog.exec() == QDialog::Accepted) 
      {
        const QString url   = stream_dialog.url();
        
        if(!QUrl(url).isEmpty() && QUrl(url).isValid()) {
          const QString name  = stream_dialog.name();
        
          MediaItemPtr media = MediaItemPtr(new MediaItem(TYPE_STREAM));
          media->data[STREAM::id]          = -1;
          media->data[STREAM::url]         = url;
          media->data[STREAM::name]        = !name.isEmpty() ? name : url ;
          media->data[STREAM::isFavorite]  = false;
          media->data[STREAM::isPlaying]   = false;
          media->data[STREAM::isBroken]    = false;
          media->data[STREAM::isPlayed]    = false;
          media->data[STREAM::isStopAfter] = false;

         _playlistModel->addMediaItem(media);
          media.reset();
        }
        else {
          _statusWidget->startShortMessage("invalid url can not be added !!", STATUS::TYPE_WARNING, 5000);
        }
      }  
    }
}


void MainWindow::slot_playqueue_save()
{
    QString filename = QFileDialog::getSaveFileName(
                this, 
                tr("Save playlist to file"), 
                QDir::homePath(), 
                tr("m3u playlist (m3u) (*.m3u);;pls playlist (pls) (*.pls);;xspf playlist (xspf) (*.xspf)")
                );

    if (!filename.isEmpty())    
      _threadManager->playlistSaveToFile(filename);
}

void MainWindow::slot_playqueue_save_auto()
{
    QInputDialog dialog(this);
    dialog.setWindowTitle(tr("Save playlist"));
    dialog.setLabelText(tr("Playlist name:"));
    dialog.setTextEchoMode(QLineEdit::Normal);
    dialog.setFixedSize(480,160);

    // to do check id dialog->textValue() is not empty
    if(dialog.exec() == QDialog::Accepted) 
    {
      _threadManager->playlistSaveToDb(dialog.textValue());
    }
}



/*******************************************************************************
    Playlist
*******************************************************************************/
void MainWindow::slot_playqueue_clear()
{
    _playlistView->clearPlaylist();
    _playlistView->stopPlaying();
    
    // on arrete la lecture seulement si il n'y a pas de lecture a partir 
    // de la collection 
    if(!_virtualPlayqueue->isPlaying() && SETTINGS()->_stopOnPlayqueueClear) {
      _player->stop();
    }

}

/*******************************************************************************
    Player management
*******************************************************************************/
void MainWindow::slot_player_on_state_change()
{
   ENGINE::E_ENGINE_STATE state = _player->engineState();
   
   switch (state) {
     /**************** STOPPED **********************/
     case ENGINE::STOPPED:
        _centralToolBar->set_enable_jump_to(false);
        (ACTIONS()->value(ENGINE_STOP))->setEnabled(false); 
        (ACTIONS()->value(ENGINE_PLAY))->setIcon(QIcon(":/images/media-play.png"));
     break;
     /**************** PLAYING ***********************/
     case ENGINE::PLAYING:
        (ACTIONS()->value(ENGINE_STOP))->setEnabled(true); 
        (ACTIONS()->value(ENGINE_PLAY))->setIcon(QIcon(":/images/media-pause.png"));
     break;
     /**************** PAUSE *************************/
     case ENGINE::PAUSED: 
        (ACTIONS()->value(ENGINE_PLAY))->setIcon(QIcon(":/images/media-play.png"));
     break; 
     /**************** ERROR *************************/
     case ENGINE::ERROR:
     {
        _statusWidget->startShortMessage("Phonon backend error", STATUS::TYPE_ERROR, 5000);
        const QString file = _player->enginePlayingUrl();
        if(!file.isEmpty()) {
          if(_playRequestFrom == FromCollection) {
            _streamScene->updateStreamBroken(file);
            _collectionScene->updateTrackBroken(file);
          }
          else {
            _playlistModel->updateMediaItemBroken(file);
          }
        }
        stopPlayer();
      }
      break;
      default:break;
    }
}


void MainWindow::slot_player_on_track_change(const QString& file) 
{
    //Debug::debug() << " ---- MainWindow -> slot_player_on_track_change :" << file;
    MediaItemPtr media = EnginePlayer::instance()->enginePlayingMediaItem();
    if(media->type == TYPE_TRACK)
      _centralToolBar->set_enable_jump_to(media->data.at(TRACK::id).toInt() != -1);
    else
      _centralToolBar->set_enable_jump_to(false);

    if( MEDIA::isUrlLocal(file) ) {
      _collectionScene->updateTrackPlaying(file);
      _playlistModel->updateMediaItemPlaying(file);
    }
    else {
      _streamScene->updateStreamPlaying(file);
      _playlistModel->updateMediaItemPlaying(file);
    }    
}

//! --------- slot_play_from_playqueue -----------------------------------------
// a file is activated from the playlist
void MainWindow::slot_play_from_playqueue()
{
    //Debug::debug() << "Mainwindow -> slot_play_from_playqueue";
    _playRequestFrom = FromPlayQueue;

    const MediaItemPtr media = _playlistView->getActiveMediaItem();

    if(media) {
      _virtualPlayqueue->stopPlaying();
      _player->setMediaItem( media );
    }
}

//! --------- slot_play_from_collection ----------------------------------------
// a MediaItem is activated from collection browser
void MainWindow::slot_play_from_collection()
{
    _playRequestFrom = FromCollection;

    const MediaItemPtr media = _virtualPlayqueue->getActiveMediaItem();

    if(media) {
      _playlistView->stopPlaying();
      _player->setMediaItem( media );
    }
    else {
      _playlistView->stopPlaying();
      _virtualPlayqueue->stopPlaying();
    }
}



//! --------- playOrPause ------------------------------------------------------
void MainWindow::playOrPause()
{
    if (_player->engineState() == ENGINE::PLAYING) {
      _player->pause();
    } 
    else if (_player->engineState() == ENGINE::PAUSED) {
      _player->play();      
    }
    else {
        MediaItemPtr media = _playlistView->getSelectedMediaItem();
        if(media)
          _player->setMediaItem( media );
    }
}

//! --------- stopPlayer -------------------------------------------------------
void MainWindow::stopPlayer()
{    
    _player->stop();
    _collectionScene->updateTrackPlaying("");
    _streamScene->updateStreamPlaying("");
    _virtualPlayqueue->stopPlaying();    

    _playlistView->stopPlaying();
}

//! --------- slot_player_enqueue_next -----------------------------------------
void MainWindow::slot_player_enqueue_next()
{
    //Debug::debug() << "Mainwindow -> slot_player_enqueue_next";
    MediaItemPtr media;

    if(_playRequestFrom == FromCollection)
      media = _virtualPlayqueue->getNextMediaItem();
    else 
      media = _playlistView->getNextMediaItem();
    
    if(media)
      _player->setNextMediaItem(media);
    else if (_playRequestFrom == FromPlayQueue)
      emit playlistFinished();
}


//! --------- playNext ---------------------------------------------------------
void MainWindow::playNext()
{
    //Debug::debug() << "Mainwindow -> playNext requested";
    MediaItemPtr media;
  
    //! We need to known if we are playing from collection or playlist
    if(_playRequestFrom == FromCollection)
      media = _virtualPlayqueue->getNextMediaItem();
    else
      media = _playlistView->getNextMediaItem();
    
    if(media)
      _player->setMediaItem(media);
    else
      stopPlayer();
}

//! --------- playPrev ---------------------------------------------------------
void MainWindow::playPrev()
{
    MediaItemPtr media;

    //! We need to known if we are playing from collection or playlist
    if(_playRequestFrom == FromCollection)
      media = _virtualPlayqueue->getPrevMediaItem();
    else
      media = _playlistView->getPrevMediaItem();

    if(media)
      _player->setMediaItem(media);
    else
      stopPlayer();      
}

//! --------- savePlayingTrack -------------------------------------------------
void MainWindow::savePlayingTrack()
{
    Debug::debug() << "Mainwindow -> savePlayingTrack";
    if (_player->engineState() == ENGINE::PLAYING) 
    {      
      SETTINGS()->_playingUrl = _player->enginePlayingUrl();
      if( MEDIA::isUrlLocal(SETTINGS()->_playingUrl) )
         SETTINGS()->_playingPosition = _player->instance()->mediaObject()->currentTime();
      else
         SETTINGS()->_playingPosition = 0;
    }
}

//! --------- restorePlayingTrack ----------------------------------------------
void MainWindow::restorePlayingTrack()
{
    Debug::debug() << "Mainwindow -> restorePlayingTrack";
    const QString url = SETTINGS()->_playingUrl;
    if(url.isEmpty())
      return;

    MediaItemPtr media;
    if( MEDIA::isUrlLocal(url) ) 
    {
        media = MEDIA::FromDataBase("MainThread", url);
        if(!media) 
          media = MEDIA::FromLocalFile( url );
    }
    else 
    {
        media = MediaItemPtr(new MediaItem(TYPE_STREAM));
        media->data[STREAM::id]        = -1;
        media->data[STREAM::url]       = url;
        media->data[STREAM::name]      = url;
        media->data[STREAM::title]     = QString();
        media->data[STREAM::artist]    = QString();
        media->data[STREAM::album]     = QString();
        media->data[STREAM::categorie] = QString();
        media->data[STREAM::isFavorite]= false;
        media->data[STREAM::isPlaying] = false;
        media->data[STREAM::isBroken]  = false;
        media->data[STREAM::isPlayed]  = false;
    }
      
    if(media) 
    {
        _player->setMediaItem(media);

        if(media->type == TYPE_TRACK) 
        {
          qint64 position = SETTINGS()->_playingPosition;
          Debug::debug() << "Mainwindow -> restorePlayingTrack TYPE_TRACK position = " << position;
          _player->mediaObject()->seek( position );
        }
    }      
}

/*******************************************************************************
    Playback Option
*******************************************************************************/
void MainWindow::slot_stop_after_media_triggered()
{
    MediaItemPtr requested_media = qvariant_cast<MediaItemPtr>( (ACTIONS()->value(PLAYQUEUE_STOP_AFTER))->data() );
    
    if(!requested_media) return;
   
    //Debug::debug() << "MainWindow::slot_stop_after_media_triggered requested_media : " << requested_media;

    MediaItemPtr current_stop_after_media = _playlistModel->get_stop_after_media();   

    if( current_stop_after_media != requested_media ) 
    {
      //! set new media item as stop after
      ACTIONS()->value(PLAYQUEUE_STOP_AFTER)->setChecked( true );      
      requested_media->data[TRACK::isStopAfter] = true;
      _playlistModel->set_stop_after_media(requested_media);
      
      if(current_stop_after_media) {
        current_stop_after_media->data[TRACK::isStopAfter] = false;
      }      
    }
    else 
    {
       ACTIONS()->value(PLAYQUEUE_STOP_AFTER)->setChecked( false );       
       requested_media->data[TRACK::isStopAfter] = false;
      _playlistModel->set_stop_after_media(MediaItemPtr(0));       
    }
}


/*******************************************************************************
    Last Fm utilities
*******************************************************************************/
void MainWindow::slot_LastFmLove() 
{
    MediaItemPtr media = _player->enginePlayingMediaItem();
    if(media) {
      if(media->type == TYPE_TRACK)
        emit _scrobblerManager->loveTrackRequest(media->data.at(TRACK::artist).toString(), media->data.at(TRACK::title).toString() );
    }
}
/*******************************************************************************
    Mpris Interface
*******************************************************************************/
void MainWindow::reloadMprisSettings()
{
    if(SETTINGS()->_useMpris && !_mpris1)
      _mpris1 =  new Mpris1(this);

    if(!SETTINGS()->_useMpris && _mpris1 != 0) {
      delete _mpris1;
      _mpris1 = 0;
    }
  
}
/*******************************************************************************
    User settings methods
*******************************************************************************/
void MainWindow::slot_on_settings_saved()
{
    Debug::debug() << "MainWindow --> slot_on_settings_saved";
  
    SETTINGS::Results r = m_settingsScene->results();

    Debug::debug() << "MainWindow --> isSystrayChanged " << r.isSystrayChanged;
    Debug::debug() << "MainWindow --> isDbusChanged " << r.isDbusChanged;
    Debug::debug() << "MainWindow --> isMprisChanged " << r.isMprisChanged;
    Debug::debug() << "MainWindow --> isRadioChanged " << r.isRadioChanged;
    Debug::debug() << "MainWindow --> isShorcutChanged " << r.isShorcutChanged;
    Debug::debug() << "MainWindow --> isScrobblerChanged " << r.isScrobblerChanged;
    Debug::debug() << "MainWindow --> isCollectionChanged " << r.isCollectionChanged;
        
    if(r.isSystrayChanged)    { reloadSystraySettings();}
    if(r.isDbusChanged)       { _dbusNotification->reloadSettings(); }
    if(r.isMprisChanged)      { reloadMprisSettings(); }
    if(r.isRadioChanged)      { _menuModel->update();}
    if(r.isShorcutChanged)    { _globalShortcuts->reloadSettings();}
    if(r.isScrobblerChanged)  { _scrobblerManager->settingsChanged();}     
    if(r.isCollectionChanged)
    {
      if(!QFile::exists(_dbManager->DB_FILE()))
        createDatabase();
    
      rebuildDatabase();
    }
    
    _statusWidget->startShortMessage(tr("settings saved"),STATUS::TYPE_INFO, 2500);
}



/*******************************************************************************
    Database methods
*******************************************************************************/
void MainWindow::createDatabase ()
{
    Debug::debug() << "Mainwindow -> createDatabase";

    Database db;
    if (!db.connect("MainThread", true)) {
        Debug::debug() << "Failed to create database ";
        return;
    }
    db.create();
}

void MainWindow::rebuildDatabase()
{
    QStringList listDir = QStringList() << _dbManager->DB_PARAM().sourcePathList;
    Debug::debug() << "MainWindow --> rebuildDatabase DB = " << _dbManager->DB_NAME;
    Debug::debug() << "MainWindow --> rebuildDatabase listDir = " << listDir;
      
    // Database Building
    if (!listDir.isEmpty()) {
      _threadManager->databaseBuild(listDir);
    }
}        

void MainWindow::removeDatabase() 
{
    Debug::debug() << "Mainwindow -> removeDatabase";
  
    //! delete existing database
    QFile database(_dbManager->DB_FILE());
    if(database.exists()) database.remove();
    
    //! delete existing cover art 
    //! WARNING SUPPRIME A CAUSE DU MULTI DATABASE WARNING
    /*QString storageLocation = UTIL::CONFIGDIR + "/albums/";
    QDir dir(storageLocation);
 
    if (dir.exists(storageLocation)) {
      Q_FOREACH(QFileInfo info, dir.entryInfoList(QDir::NoDotAndDotDot | QDir::System | 
              QDir::Hidden  | QDir::AllDirs | QDir::Files, QDir::DirsFirst)) 
      {
        QFile::remove(info.absoluteFilePath());
      }
    }*/
}


void MainWindow::slot_database_start()
{
    Debug::debug() << "MainWindow -->  slot_database_start";

    //! check new database entry
    if(!QFile::exists( _dbManager->DB_FILE() ))
    {
        /*-----------------------------------------------------------*/
        /* First start dialog                                        */
        /* ----------------------------------------------------------*/
        Debug::debug() << "MainWindow --> slot_database_start : first start dialog";
      
        QString str = tr("<h2>Welcome to YaRock</h2>" 
                         "<p>No collection seems to be setup</p>"
                         "<p>Do you want to setup your collection Now</p>");
     
        QMessageBox message(this);
        message.setWindowTitle(tr("Setup your music collection directory"));
        message.setText(str.toAscii());
        message.setStandardButtons(QMessageBox::Ok | QMessageBox::Cancel);
        message.resize(445, 120);
    
       if( message.exec() == QMessageBox::Ok) {
         //Debug::debug() << "MainWindow -->  firstStartDialog result" << message.result();
         m_settingsScene->set_page(SETTINGS::LIBRARY);
         _menuModel->activate_view_idx(MAIN::ViewSettings);
       }
    }
    else if (!_dbManager->isVersionOK())
    {
        /*-----------------------------------------------------------*/
        /* Database revision change                                  */
        /* ----------------------------------------------------------*/
        Debug::debug() << "MainWindow --> slot_database_start : database revision change";
        const QString str = tr("<p>Database need to be rebuilt</p>");

        QMessageBox message(this);
        message.setWindowTitle(tr("Database revision update"));
        message.setText(str.toAscii());
        message.setStandardButtons(QMessageBox::Ok);
        message.resize(445, 120);
        message.exec();
      
        removeDatabase();
        createDatabase();
        rebuildDatabase();
    }
    else if (_dbManager->DB_PARAM().autoRebuild) 
    {
        /*-----------------------------------------------------------*/
        /* Start existing database with auto rebuild at startup      */
        /* ----------------------------------------------------------*/
        rebuildDatabase();
    }
    else 
    {
        /*-----------------------------------------------------------*/
        /* Start existing database by populating models              */
        /* ----------------------------------------------------------*/
        slot_model_populate();
    }
}

void MainWindow::slot_database_dialog() 
{
    // case 1 : remove database and rescan all collection directories
    // case 2 : scan directory changes and update database
    DbOperationDialog dialog(this);

    if( dialog.exec() == QDialog::Accepted) 
    {
      if( dialog.isFullRescan()) {
        removeDatabase();
        createDatabase();
      }
        
      rebuildDatabase();
    }
}


/*******************************************************************************
    Collection Model
*******************************************************************************/
void MainWindow::slot_model_populate() 
{
    Debug::debug() << "MainWindow --> slot_model_populate";

    _collectionScene->clear();
    _threadManager->populateCollectionModel();
    _threadManager->populatePlsFileModel();
}

void MainWindow::slot_model_populate(int model) 
{
    _collectionScene->clear();
    if(model == 1) {
      _threadManager->populateCollectionModel();
    }
    else if(model == 2) {
      _threadManager->populatePlsFileModel();
    }
}

/*******************************************************************************
    Graphics Views update
*******************************************************************************/
void MainWindow::slot_views_update(E_MODEL_TYPE model_updated) 
{
    //Debug::debug() << "MainWindow --> slot_views_update model_updated" << model_updated;
    //! no need to update graphics view is the viewmode is not impacted
    
    switch(_graphicViewWidget->mode()) 
    {
      case MAIN::ViewAlbum     : 
      case MAIN::ViewArtist    :
      case MAIN::ViewTrack     :
      case MAIN::ViewGenre     :
      case MAIN::ViewYear      :
      case MAIN::ViewFavorite  :
      case MAIN::ViewDashBoard : /* depends on CollectionModel */
      case MAIN::ViewHistory   : /* depends on CollectionModel */                 
        if(model_updated == MODEL_COLLECTION) {
         _graphicViewWidget->updateView();
        }
      break;
      
      case MAIN::ViewPlaylist      :
      case MAIN::ViewSmartPlaylist :
        if(model_updated == MODEL_PLAYLIST) {    
         _graphicViewWidget->updateView();
        } 
      break;
      
      default : 
        break;
    }
}

/*******************************************************************************
    Now Playing slots
*******************************************************************************/
void MainWindow::slot_jump_to_now_playing()
{
    Debug::debug() << "MainWindow::slot_jump_to_now_playing";
    QAction *action = qobject_cast<QAction *>(sender());
    if(!action) return;

    MAIN::ViewMode mode = MAIN::ViewMode(action->data().toInt());
    
    MediaItemPtr media  = EnginePlayer::instance()->enginePlayingMediaItem();
    //Debug::debug() << "MainWindow::slot_jump_to_now_playing media-> id  " << media->data.at(TRACK::id).toInt();

    // get parent album
    if( media && (mode == MAIN::ViewAlbum || mode == MAIN::ViewArtist) )
      media = media->parent();

    // get parent artist
    if( media && (mode == MAIN::ViewArtist) )
      media = media->parent();

      
    if(media)
    {
      // clear search field
      _graphicViewWidget->setSearch("");
      
      // activate track view throught menu model
      _menuModel->activate_view_idx(mode);
      
      // collection scene shall be populated with graphics trackitem
      QPoint p = _collectionScene->get_item_position(media);

      _graphicViewWidget->set_scroll_to(p);
    }
}    

/*******************************************************************************
    Thread for cover searching
*******************************************************************************/
void MainWindow::startCoverSearchThread() 
{
    _threadManager->startCoverSearch();
}


/*******************************************************************************
    Compact mode widget management
*******************************************************************************/
void MainWindow::slot_widget_mode_switch()
{
    QAction *action = qobject_cast<QAction *>(sender());
    if(!action) return;
    
    /*--------------------------------------------------*/
    /* switch to minimal widget                         */
    /* -------------------------------------------------*/
    if( action == ACTIONS()->value(APP_MODE_COMPACT) )
    {
      if(_minimalwidget == NULL)
        _minimalwidget = new MinimalWidget(this); 
    
      this->hide();
      _minimalwidget->move(this->pos());
      _minimalwidget->show();
    }
    /*--------------------------------------------------*/
    /* switch to full window widget                     */
    /* -------------------------------------------------*/
    else if( action == ACTIONS()->value(APP_MODE_NORMAL) )
    {
      if(_minimalwidget != NULL)
        _minimalwidget->hide();

      this->show();
    }
}
/*******************************************************************************
    AudioEqualizer Dialog
*******************************************************************************/
void MainWindow::slot_eq_openDialog()
{
    Equalizer_Dialog *eqDialog = new Equalizer_Dialog(this);
  
    connect(eqDialog, SIGNAL(enabledChanged(bool)), SLOT(slot_eq_enableChange(bool)));
    connect(eqDialog, SIGNAL(newEqualizerValue(int, QList<int>)), SLOT(slot_eq_paramChange(int, QList<int>)));

    eqDialog->exec();
    eqDialog->deleteLater();
}

void MainWindow::slot_eq_enableChange(bool eqActivated)
{
    Debug::debug() << "MainWindow --> slot_eq_enableChange bool" << eqActivated;
    //! Equalizer ACTIVATED
    if(eqActivated) {
      //! Equalizer available
      if(_player->isEqualizerAvailable()) {
        _player->addEqualizer();
      }
      //! Equalizer NOT available
      else {
        QString str = tr("<h1>Equalizer is not available</h1>" 
        "<p>Equalizer wasn't found, probably you are using a backend that doesn't support it.</p>"
        "<p>Change to another backend (gstreamer is supported) if you want to have equalizer</p>");
   
        QMessageBox dlg(this);
        dlg.setIcon(QMessageBox::Warning);
        dlg.setWindowTitle(tr("Equalizer information"));
        dlg.setText(str.toAscii());
        dlg.setStandardButtons(QMessageBox::Ok);
        dlg.resize(445, 120);
        dlg.exec();
      }
    }
    //! Equalizer DE-ACTIVATED
    else 
    {
        _player->removeEqualizer();
    }
}


void MainWindow::slot_eq_paramChange(int preamp, QList<int> listGain)
{
    Debug::debug() << "MainWindow --> slot_eq_paramChange";

    QList<int> gains;
    gains << preamp << listGain;

    _player->applyEqualizer(gains);
}

/*******************************************************************************
    Command Line options
*******************************************************************************/
void MainWindow::slot_commandline_received(const QByteArray& serialized_options)
{
    //Debug::debug() << "MainWindow --> slot_commandline_received";
    CommandlineOptions options;
    options.Load(serialized_options);

    if (options.isEmpty()) {
      //Debug::debug() << "MainWindow --> slot_commandline_received options is empty";
      if(_minimalwidget != NULL)
         _minimalwidget->hide();
      if (!this->isVisible())
         this->show();

      this->activateWindow();
      this->raise();
    }
    else
      commandlineOptionsHandle(options);
}

void MainWindow::commandlineOptionsHandle(const CommandlineOptions &options) 
{
    switch (options.player_action()) 
    {
      case CommandlineOptions::Player_Play:        this->playOrPause();break;
      case CommandlineOptions::Player_PlayPause:   this->playOrPause();break;
      case CommandlineOptions::Player_Pause:       this->playOrPause();break;
      case CommandlineOptions::Player_Stop:        this->stopPlayer();break;
      case CommandlineOptions::Player_Previous:    this->playPrev();break;
      case CommandlineOptions::Player_Next:        this->playNext();break;
      case CommandlineOptions::Player_None: /* do nothing */      break;
    }

    switch (options.playlist_action()) 
    {
      case CommandlineOptions::Playlist_Load:      
        this->slot_playqueue_clear(); // no break !
      case CommandlineOptions::Playlist_Append:    
        _threadManager->playlistAddUrls(options.urls()); break;
    }
  
    if (options.set_volume() != -1)
      _player->setVolume(options.set_volume()); //! shall be in percent !

    if (options.volume_modifier() != 0)
      _player->setVolume(_player->volume() + options.volume_modifier());

    if (options.seek_to() != -1)    
      _player->mediaObject()->seek(options.seek_to());
    else if (options.seek_by() != 0)
      _player->mediaObject()->seek(
         _player->instance()->mediaObject()->currentTime() +
         options.seek_by()
      );

    if (options.play_track_at() != -1) {
      MediaItemPtr media = _playlistView->getMediaItemAt(options.play_track_at());
      if(media) {
        _playRequestFrom = FromPlayQueue;
        _virtualPlayqueue->stopPlaying();
        _player->setMediaItem(media);
      }
    }
}
